<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace Ron\Controls;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use Tracy\Debugger;


class FormField
{
    const FIELD_TYPE_TEXT = 'text';     // jen text
    const FIELD_TYPE_SECTION_HEADER = 'sectionheader';   // nadpis sekce
    const FIELD_TYPE_GROUP = 'group';   // zacatek skupiny poli (div)
    const FIELD_TYPE_GROUP_END = 'groupend'; // konec skupiny poli
    const FIELD_TYPE_EDIT = 'edit';     // normalni editovatelne pole
    const FIELD_TYPE_MEMO = 'memo';     // viceradkove editovatelne pole
    const FIELD_TYPE_DATE = 'date';     // datum
    const FIELD_TYPE_TIME = 'time';     // cas
    const FIELD_TYPE_DATETIME = 'datetime'; // datum cas
    const FIELD_TYPE_DATEANDTIME = 'dateandtime'; // datum + cas (2 pole)
    const FIELD_TYPE_INT = 'int';       // cele cislo
    const FIELD_TYPE_FLOAT = 'float';   // desetinne cislo
    const FIELD_TYPE_PASSWORD = 'password'; // heslo
    const FIELD_TYPE_CHECK = 'check';   // checkbox
    const FIELD_TYPE_SELECT = 'select'; // sql select
    const FIELD_TYPE_COMBO = 'combo';   // vyber z preddefinovanych hodnot
    const FIELD_TYPE_COMBOSTR = 'combostr';     // vyber z preddefinovanych hodnot
    const FIELD_TYPE_RADIO = 'radio';   // radiobuttonek
    const FIELD_TYPE_HIDDEN = 'hidden'; // skryte pole
    const FIELD_TYPE_CONSTANT = 'constant'; // pevna, nemenna hodnota
    const FIELD_TYPE_ACTUAL_TIME = 'actualtime';
    const FIELD_TYPE_ACTUAL_DATE = 'actualdate';
    const FIELD_TYPE_ID = 'id';
    const FIELD_TYPE_FILE = 'file';
    const FIELD_TYPE_FILENAME = 'filename';
    const FIELD_TYPE_NOT_NULL_FLAG = 'notnullflag'; // priznak nenulove hodnoty (slouzi napr. pro javascriptove ovladani formulare)

    /**
     * @var string
     * Typ polozky
     *   text   - text, ktery se primo zobrazi
     *   edit   - normalni text (string)
     *   check  - zaskrtavatko - hodnoty 0/1
     *   memo   - viceradkovy text
     *   select - combo box - viz $selectSql
     *   combo  - combobox s predvyplnenymi moznostmi, datovy typ INT - viz $options
     *   combostr  - combobox s predvyplnenymi moznostmi - viz $options
     *   hidden - ve formulari se nezobrazi, pri insertu se nastavi defaultni hodnota, pri editaci nelze zmenit
     *   id     - automaticky cislovane pole typu int
     *   password - heslo (pri editaci se nenacita z databaze)
     *   file   - pole pro upload souboru
     *   filename - pole pro upload souboru, jmeno souboru se zapise do databaze
     *   actualtime - aktualni cas
     *   _custom - uzivatelske pole - typ muze byt cokoliv, ale obsluhu musi zajistit formular
     */
    public $fieldType;
    /**
     * @var string
     * jmeno pole
     * pokud jmeno zacina znakem *, tak se nejedna o databazove pole
     * jinak je to databazove pole a nastavi se podle nej take sqlFieldName
     */
    public $fieldName;
    /** @var string     jmeno pole v databazi */
    public $sqlFieldName;
    /** @var integer    index pole v radku nactenem z databaze */
    public $sqlFieldIndex;
    /** @var string     Jmeno tabulky, do ktere pole patri. U formularu, ktere edituji jen data z jedne tabulky se nemusi pouzivat. */
    public $tableName;
    /** @var string     Alias tabulky, do ktere pole patri - pouzije se pro select. U formularu, ktere edituji jen data z jedne tabulky se nemusi pouzivat. */
    public $tableAlias = null;

    /** @var bool|string    Slouzi pro pole typu Id - priznak, ze ma pouzit pro automaticke cislovani seqenci SEQ_<tablename> nebo primo nazev sekvence */
    public $sequence = false;

    /** @var string     Kod pro konfiguraci pomoci XML */
    public $code;

    /** @var bool       priznak, jestli se jedna o primarni klic */
    public $primary;

    /** @var bool|int   Jestli je readonly (plati pouze pro editaci, ne pro vkladani) - od 02-2022 - hodnota 2 znamena, ze nepujde editovat ani pri vkladani */
    public $readOnly = false;

    /** @var bool|null  Pokud je true, tak vynuti vytvoreni DBRecordField, i bez $sqlFieldName. Pokud je false, tak nevytvari DBRecordField. */
    public $includeToDBRecord = null;

    /** @var int        Urcuje, jestli je pole povinne 0=nepovinne, 1=povinne, 2=povinne, ale nezabrani ulozeni formulare (pouze generuje chybovou hlasku) */
    public $required;
    /** @var bool       umoznuje vypnout zobrazeni pole */
    public $visible = true;
    /** @var bool       Urcuje, jestli je mozne zmenit nenullovou hodnotu */
    public $changeable = true;
    /** @var bool       Priznak, ze pole je povinne v zaznamu v databazi */
    public $requiredInDatabase = false;

    /** @var string|false     nadpis polozky */
    public $caption;
    /** @var string     2. nadpis, ktery se zobrazi za controlem */
    public $caption2;
    /** @var bool       Priznak, jestli se ma 2. nadpis zobrazit pred controlem */
    public $caption2First = false;

    /** @var mixed      defaultni hodnota pole - pro insert */
    public $defaultValue;
    /** @var bool       priznak, ze se ma pouzit defaultValue pri editaci, pokud je v datech null */
    public $useDefaultValueToEditNull;

    /** @var string     Pouzije se u nekterych typu poli - napriklad float */
    public $format;

    /** @var string|SqlStatement    sql dotaz pro vyber ( napr. select kod, popis from ... ) */
    public $selectSql;
    /** @var string                 dodatecna sql podminka pro filtrovani - nahradi v selectSql %FILTER% */
    public $selectSqlFilter = null;
    /** @var string                 order by cast sql dotazu - nahradi v selectSql %ORDER% */
    public $selectSqlOrder = null;
    /** @var bool       jestli se ma pri porovnavani porovnavat datovy typ (defaultne ano) */
    public $selectDataTypeTest = true;
    /** @var callable   funkce, kterou lze filtrovat polozky nactene SQL dotazem */
    public $selectFilterFunction = null;
    /** @var bool       priznak, jestli se ma vzdy (i kdyz je required) zobrazit prazdna hodnota - null/true/false */
    public $showEmptyValue = null;
    /** @var array      preferovane hodnoty pro combobox (zobrazi se na zacatku vyberu - mimo poradi) */
    public $preferredValues = null;
    /** @var bool       priznak, jestli se ma v comboboxu zobrazovat kod */
    public $selectShowCode = false;
    /** @var int[]      Asociativni pole indexu, ktere se pouziji pro datove atributy polozek v comboboxu (nazev atributu => index) */
    public $itemsAttributes;

    /** @var string Prefix pro preklad retezcu pro SELECT */
    public $translationPrefix;
    /** @var string Prefix pro preklad titulku SELECT */
    public $titleTranslationPrefix;
    /** @var int    Index pole, ktere se pouzije jako kod pro preklad */
    public $translationValueIndex;

    /** @var SqlStatement|string    sql filtr pro ziskani noveho ID zaznamu (pouzije se po pole typu ID */
    public $idSqlFilter = null;

    /** @var bool       priznak, jestli se ma zobrazit tlacitko "vlozit aktualni cas". Funguje jen kdyz je nastaveno fieldId. */
    public $currentTimeButton = false;

    /** @var string     Pouzije se pro type="file" jako accept atribut HTML tagu <input> */
    public $accept = null;

    /** @var string[]   Dalsi parametry pro prvek formulare (napr. data-atributy a podobne) */
    public $otherHtmlParams;
    /** @var string[]   Dalsi parametry pro druhy prvek formulare (pouziva se pro DATEANDTIME) */
    public $otherHtmlParams2 = [];

    /** @var int[]      Asociativni pole indexu datovych poli, ktera se pouziji pro vygenerovani dalsich atributu HTML pole (napr. datovych) */
    public $dataAttributes = [];

    /** @var array      pole voleb pro combo (napr. 1 => volba 1, 2 => volba 2, ...) */
    public $options;
    /** @var string     zobrazeny text - pro typ text, pro typ password je to nazev pole */
    public $text;
    /** @var int        maximalni pocet znaku - pro edit, memo */
    public $maxLength;
    /** @var string     jmeno funkce pro zjisteni hodnoty daneho pole */
    public $functionName;
    /** @var int        pocet radku pro memo */
    public $rows;
    /** @var int        pocet sloupcu pro memo */
    public $cols;
    /** @var bool       zalamovani slov pro memo */
    public $wrap;
    /** @var string     datovy typ pro hidden pole ('', 'date', 'datetime', 'int') */
    public $dataType;

    /** @var string     html id editacniho prvku */
    public $fieldId;
    /** @var string     css class editacniho prvku. Pokud neni nastavena, doplni se automaticky podle typu */
    public $fieldCssClass;

    /** @var bool       Urcuje, jestli se ma pole vygenerovat na novy radek */
    public $newRow = true;
    /** @var string     html id radku formulare. Standardne nastaveno na row_{$fieldName} */
    public $rowId = null;
    /** @var string     Class radku. Pokud neni nastaveno, pouzije formular vlastni "defaultRowClass" */
    public $rowClass = null;
//    /** @var string[]   Dalsi atributy pro radek */
//    public $rowAttrs = [];

    /** @var string     Css class, ktera se prida k formulari, pokud ma toto pole nenulovou nebo hodnotu (tj. neni null, neni 0, neni prazdny retezec) */
    public $appendFormClass = null;

//    /** @var bool       Urcuje, jestli pole zacina novou skupinu (div slucujici vice radku). Implicitne false a pro FIELD_TYPE_SECTION true. */
//    public $newFieldGroup = false;
//    /** @var string     Id skupiny */
//    public $fieldGroupId = null;
//    /** @var string     Class skupiny */
//    public $fieldGroupClass = null;

    public $showColumns = true;
    /** @var string     Class sloupce s nadpisem */
    public $captionColumnClass = null;
    /** @var string     Class sloupce s daty */
    public $dataColumnClass = null;


    /** @var bool   Priznak, ze se jedna jen o logovaci pole - viz DBRecordField.updateLog */
    public $updateLog;

    /** @var mixed  Aktualni hodnota pole nactena z databaze nebo z POST nebo z implicitni hodnoty */
    public $value;

    /**
     * @param string $fieldType   Typ pole (jedna z konstant self::FIELD_TYPE_...)
     */
    public function __construct($fieldType = self::FIELD_TYPE_EDIT)
    {
        if (is_array($fieldType)) {
            if ($GLOBALS['development'] ?? false) {
                MessageManager::addWarning('Zastarala inicializace formularoveho pole!');
            }
            //$this->rowClass = 'form-row';
            $this->init($fieldType);
        } else {
            $this ->setFieldType($fieldType);
//            if (!$this ->rowClass) {
//                $this->rowClass = 'form-row';
//            }
        }
    }

    /**
     * Nastavi typ pole - musi byt jedna z konstant FormField::FIELD_TYPE_...
     * @param string $fieldType
     * @return FormField
     */
    public function setFieldType($fieldType)
    {
        $this->fieldType = $fieldType;

        switch ($this ->fieldType) {
            case self::FIELD_TYPE_SECTION_HEADER :
                if (!$this ->rowClass) {
                    $this ->rowClass = 'form-section'; // vychozi class pro section
                    //$this ->newFieldGroup = true;
                    $this ->showColumns = false;
                }
                break;

            case self::FIELD_TYPE_DATE :
                $this ->maxLength = 10;
                break;

            case self::FIELD_TYPE_CHECK :
                if (!$this ->options) {
                    $this->setOptions(['' => '', 0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);    // pro moznost prepnuti na combobox
                }
                break;

            case self::FIELD_TYPE_ACTUAL_TIME :
            case self::FIELD_TYPE_ACTUAL_DATE :
            case self::FIELD_TYPE_ID :
            case self::FIELD_TYPE_HIDDEN :
            case self::FIELD_TYPE_CONSTANT :
            case self::FIELD_TYPE_GROUP :
            case self::FIELD_TYPE_GROUP_END :
                $this ->newRow = false;
                $this ->showColumns = false;
                break;

            case self::FIELD_TYPE_FLOAT :
                if (!$this ->format) {
                    $this->format = 'number_2'; // vychozi format pro float
                }
                break;
        }

        if (!$this ->fieldCssClass) {
            $this ->fieldCssClass = $this ->getDefaultClass();
        }

        return $this;
    }


    /**
     * Nastavi css class pro datove pole
     * @return string
     */
    public function getDefaultClass()
    {
        switch ($this->fieldType) {
            case self::FIELD_TYPE_DATE :
                return 'inputDate';


            case self::FIELD_TYPE_INT :
                return 'form-control inputInt';

            case self::FIELD_TYPE_FLOAT :
                return 'form-control inputFloat';

            case self::FIELD_TYPE_CHECK :
            case self::FIELD_TYPE_RADIO :
                return 'form-check-input';

            case self::FIELD_TYPE_GROUP :
                return 'form-field-group';

            case self::FIELD_TYPE_DATETIME :
            case self::FIELD_TYPE_TIME :
            case self::FIELD_TYPE_DATEANDTIME :
            case self::FIELD_TYPE_TEXT :
                return '';

            case self::FIELD_TYPE_EDIT :
            case self::FIELD_TYPE_PASSWORD :
            case self::FIELD_TYPE_MEMO :
            case self::FIELD_TYPE_SELECT :
            case self::FIELD_TYPE_COMBO :
            case self::FIELD_TYPE_COMBOSTR :
                return 'form-control';
        }

        return '';
    }


    /**
     * Nastavi kod pole. Mel by byt jedinecny v ramci formulare. Slouzi predevsim pro identifikaci pole v konfiguracnim XML.
     * @param string $code
     * @return FormField
     */
    public function setCode($code)
    {
        $this->code = $code;
        return $this;
    }

    /**
     * Nastavi, jestli je pole zobrazeno.
     * @param boolean $visible
     * @return FormField
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;
        return $this;
    }


    /**
     * Vraci true, pokud je pole viditelne (ma se zobrazovat)
     * @return bool
     */
    public function isVisible()
    {
        return $this->visible;
    }


    /**
     * @param boolean|int $readOnly
     * @return FormField
     */
    public function setReadOnly($readOnly = true)
    {
        $this->readOnly = $readOnly;
        //$this ->editable = !$readOnly;  // pouze pro kompatibilitu
        return $this;
    }


    /**
     * @param bool|null $includeToDBRecord
     * @return FormField
     */
    public function setIncludeToDBRecord($includeToDBRecord)
    {
        $this->includeToDBRecord = $includeToDBRecord;
        return $this;
    }


    /**
     * @param boolean $changeable
     * @return FormField
     */
    public function setChangeable($changeable)
    {
        $this->changeable = $changeable;
        return $this;
    }



    /**
     * @param bool $value
     * @return FormField
     */
    public function setRequired($value) {
        $this ->required = $value;
        return $this;
    }

    /**
     * @param boolean $requiredInDatabase
     * @return FormField
     */
    public function setRequiredInDatabase($requiredInDatabase)
    {
        $this->requiredInDatabase = $requiredInDatabase;
        return $this;
    }




    /**
     * Nastavi jmeno html pole a take jmeno SQL pole (pokud jiz neni nastaveno)
     * @param string $fieldName     Jmeno pole v HTML formulari
     * @param string $tableName     Jmeno SQL tabulky - pokud je nastaveno, prida se jako prefix i ke jmenu HTML pole (takze vysledne jmeno bude <$tableName>.<$value>)
     * @param string $tableAlias    Alias SQL tabulky pro select
     * @return FormField
     */
    public function setFieldName($fieldName, $tableName = null, $tableAlias = null)
    {
        // POZOR! Nastavuje i sql_fieldname (to se da dodatecne zmenit)
        if ($fieldName[0] == '*') {
            $this->fieldName = substr($fieldName, 1);
            $this->sqlFieldName = '';
        } else {
            $tbl = $tableAlias ?: $tableName;
            $this->fieldName = ($tbl ? $tbl . '#' : '') . $fieldName;  // nepouziva tecku, protoze PHP ji v POSTu nahrazuje podtrzitkem
            if (!$this ->sqlFieldName) {
                $this->sqlFieldName = $fieldName;
            }
            if (isset($tableName)) {
                $this ->tableName = $tableName;
                $this ->tableAlias = $tableAlias;
            }
        }

        return $this;
    }

    public function setUpdateLog($value)
    {
        $this ->updateLog = $value;
        return $this;
    }

    /**
     * @param $value
     * @return FormField
     */
    public function setSqlFieldIndex($value)
    {
        $this ->sqlFieldIndex = $value;
        return $this;
    }

    /**
     * @param string $sqlFieldName
     * @param string $sqlTableName Nepovinne jmeno SQL tabulky. Pouzije se v pripade, ze je ve formulari pouzito vice tabulek.
     * @param string $sqlTableAlias
     * @return FormField
     */
    public function setSqlFieldName($sqlFieldName, $sqlTableName = null, $sqlTableAlias = null)
    {
        $this->sqlFieldName = $sqlFieldName;

        if (isset($sqlTableName)) {
            $this ->tableName = $sqlTableName;
            $this ->tableAlias = $sqlTableAlias;
        }

        return $this;
    }

    /**
     * @param string $tableName
     * @return FormField
     */
    public function setTableName($tableName)
    {
        $this->tableName = $tableName;
        return $this;
    }


    /**
     * @param string $tableAlias
     * @return FormField
     */
    public function setTableAlias($tableAlias)
    {
        $this->tableAlias = $tableAlias;
        return $this;
    }


    /**
     * @param bool|string $sequence
     * @return FormField
     */
    public function setSequence($sequence)
    {
        $this->sequence = $sequence;
        return $this;
    }




    /**
     * @param boolean $primary
     * @return FormField
     */
    public function setPrimary($primary)
    {
        $this->primary = $primary;
        return $this;
    }

    /**
     * @param boolean $useDefaultValueToEditNull
     * @return FormField
     */
    public function setUseDefaultValueToEditNull($useDefaultValueToEditNull)
    {
        $this->useDefaultValueToEditNull = $useDefaultValueToEditNull;
        return $this;
    }


    /**
     * @param string|null $accept
     * @return FormField
     */
    public function setAccept(?string $accept): FormField
    {
        $this->accept = $accept;
        return $this;
    }


    /**
     * @param string[]|string $otherHtmlParams
     * @return FormField
     */
    public function setOtherHtmlParams($otherHtmlParams)
    {
        $this->otherHtmlParams = $otherHtmlParams;
        return $this;
    }

    /**
     * @param string[] $otherHtmlParams2
     * @return FormField
     */
    public function setOtherHtmlParams2($otherHtmlParams2)
    {
        $this->otherHtmlParams2 = $otherHtmlParams2;
        return $this;
    }

    /**
     * @param int[] $dataAttributes
     * @return FormField
     */
    public function setDataAttributes($dataAttributes)
    {
        $this->dataAttributes = $dataAttributes;
        return $this;
    }


    /**
     * @param string $text
     * @return FormField
     */
    public function setText($text)
    {
        $this->text = $text;
        return $this;
    }

    /**
     * @param string $functionName
     * @return FormField
     */
    public function setFunctionName($functionName)
    {
        $this->functionName = $functionName;
        return $this;
    }

    /**
     * @param string $dataType
     * @return FormField
     */
    public function setDataType($dataType)
    {
        $this->dataType = $dataType;
        if ($dataType === 'int') {
            $this ->setSelectDataTypeTest(false);
        }
        return $this;
    }



    /**
     * @param mixed $defaultValue
     * @return FormField
     */
    public function setDefaultValue($defaultValue)
    {
        $this->defaultValue = $defaultValue;
        $this->value = $defaultValue;
        return $this;
    }

    /**
     * @param string $caption
     * @param bool $translate
     * @return FormField
     */
    public function setCaption($caption, $translate = true)
    {
        $this->caption = $translate ? TextManager::_($caption) : $caption;
        return $this;
    }

    /**
     * @param string $caption2
     * @param bool $translate
     * @param bool $caption2First   Urcuje, jestli se caption 2 zobrazi pred obsahem pole
     * @return FormField
     */
    public function setCaption2($caption2, $translate = true, $caption2First = false)
    {
        $this->caption2 = $translate ? TextManager::_($caption2) : $caption2;
        $this ->caption2First = $caption2First;
        return $this;
    }


    /**
     * @param string $format
     * @return FormField
     */
    public function setFormat($format)
    {
        $this->format = $format;
        return $this;
    }


    /**
     * @param string|SqlStatement $selectSql
     * @param string $selectSqlFilter
     * @param string $selectSqlOrder
     * @return FormField
     */
    public function setSelectSql($selectSql, $selectSqlFilter = null, $selectSqlOrder = null)
    {
        $this->selectSql = $selectSql;
        $this->selectSqlFilter = $selectSqlFilter;
        $this->selectSqlOrder = $selectSqlOrder;
        return $this;
    }


    /**
     * @param callable $selectFilterFunction
     * @return FormField
     */
    public function setSelectFilterFunction($selectFilterFunction)
    {
        $this->selectFilterFunction = $selectFilterFunction;
        return $this;
    }


    /**
     * @param string $selectSqlFilter
     * @return FormField
     */
    public function setSelectSqlFilter($selectSqlFilter)
    {
        $this->selectSqlFilter = $selectSqlFilter;
        return $this;
    }

    /**
     * @param string $selectSqlOrder
     * @return FormField
     */
    public function setSelectSqlOrder($selectSqlOrder)
    {
        $this->selectSqlOrder = $selectSqlOrder;
        return $this;
    }

    /**
     * @param boolean $selectDataTypeTest
     * @return FormField
     */
    public function setSelectDataTypeTest($selectDataTypeTest)
    {
        $this->selectDataTypeTest = $selectDataTypeTest;
        return $this;
    }

    /**
     * Nastavi jestli se ma zobrazovat prazdny radek. Hodnota null znamena, ze se bude zobrazovat automaticky (tj. pro required pole pri edit ne, jinak ano)
     * @param boolean|null $showEmptyValue
     * @return FormField
     */
    public function setShowEmptyValue($showEmptyValue)
    {
        $this->showEmptyValue = $showEmptyValue;
        return $this;
    }


    /**
     * @param array $preferredValues
     * @return FormField
     */
    public function setPreferredValues($preferredValues)
    {
        $this->preferredValues = $preferredValues;
        return $this;
    }


    /**
     * @param boolean $selectShowCode
     * @return FormField
     */
    public function setSelectShowCode($selectShowCode)
    {
        $this->selectShowCode = $selectShowCode;
        return $this;
    }


    /**
     * @param SqlStatement|string $idSqlFilter
     * @return FormField
     */
    public function setIdSqlFilter($idSqlFilter)
    {
        $this->idSqlFilter = $idSqlFilter;
        return $this;
    }


    /**
     * @param boolean $currentTimeButton
     * @return FormField
     */
    public function setCurrentTimeButton($currentTimeButton)
    {
        $this->currentTimeButton = $currentTimeButton;
        return $this;
    }


    /**
     * @param int[] $itemsAttributes
     * @return FormField
     */
    public function setItemsAttributes($itemsAttributes)
    {
        $this->itemsAttributes = $itemsAttributes;
        return $this;
    }

    /**
     * @param string $translationPrefix
     * @param int $translationValueIndex
     * @return $this
     */
    public function setTranslationPrefix($translationPrefix, $translationValueIndex = 0)
    {
        $this->translationPrefix = $translationPrefix;
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }

    /**
     * @param string $titleTranslationPrefix
     * @return $this
     */
    public function setTitleTranslationPrefix($titleTranslationPrefix)
    {
        $this->titleTranslationPrefix = $titleTranslationPrefix;
        return $this;
    }

    /**
     * @param int $translationValueIndex
     * @return $this
     */
    public function setTranslationValueIndex($translationValueIndex)
    {
        $this->translationValueIndex = $translationValueIndex;
        return $this;
    }




    /**
     * @param string $fieldId
     * @return FormField
     */
    public function setFieldId($fieldId)
    {
        $this->fieldId = $fieldId;
        return $this;
    }

    /**
     * @param string $fieldCssClass
     * @return FormField
     */
    public function setFieldCssClass($fieldCssClass)
    {
        $this->fieldCssClass = $fieldCssClass;
        return $this;
    }

    /**
     * @param string $fieldCssClass
     * @return FormField
     */
    public function addFieldCssClass($fieldCssClass)
    {
        $this->fieldCssClass .= ($this->fieldCssClass ? ' ' : '') . $fieldCssClass;
        return $this;
    }


    /**
     * @param bool $newRow
     * @return FormField
     */
    public function setNewRow($newRow)
    {
        $this->newRow = $newRow;
        return $this;
    }

    /**
     * @param bool $showColumns
     * @return FormField
     */
    public function setShowColumns($showColumns)
    {
        $this->showColumns = $showColumns;
        return $this;
    }

    /**
     * @param string $rowId
     * @return FormField
     */
    public function setRowId($rowId)
    {
        $this->rowId = $rowId;
        return $this;
    }

    /**
     * @param string $rowClass
     * @return FormField
     */
    public function setRowClass($rowClass)
    {
        $this->rowClass = $rowClass;
        return $this;
    }

    /**
     * @param string $captionColumnClass
     * @return FormField
     */
    public function setCaptionColumnClass($captionColumnClass)
    {
        $this->captionColumnClass = $captionColumnClass;
        return $this;
    }

    /**
     * @param string $dataColumnClass
     * @return FormField
     */
    public function setDataColumnClass($dataColumnClass)
    {
        $this->dataColumnClass = $dataColumnClass;
        return $this;
    }

    /**
     * @param string $className
     * @return FormField
     */
    public function addDataColumnClass($className)
    {
        $this->dataColumnClass .= ($this->dataColumnClass ? ' ' : '') . $className;
        return $this;
    }


    /**
     * @param string $appendFormClass
     * @return FormField
     */
    public function setAppendFormClass($appendFormClass)
    {
        $this->appendFormClass = $appendFormClass;
        return $this;
    }



    /**
     * Nastavi options pro ComboBox
     * @param array $options
     * @param bool $dataTypeTest
     * @return FormField
     */
    public function setOptions($options, $dataTypeTest = true)
    {
        $this->options = $options;
        $this->selectDataTypeTest = $dataTypeTest;

        return $this;
    }

    /**
     * @param int $maxLength
     * @return FormField
     */
    public function setMaxLength($maxLength)
    {
        $this->maxLength = $maxLength;
        return $this;
    }

    /**
     * @param int $rows
     * @param int|null $cols
     * @return FormField
     */
    public function setRows($rows, $cols = null)
    {
        $this->rows = $rows;
        if (isset($cols)) {
            $this ->cols = $cols;
        }
        return $this;
    }

    /**
     * @param int $cols
     * @return FormField
     */
    public function setCols($cols)
    {
        $this->cols = $cols;
        return $this;
    }

    /**
     * @param boolean $wrap
     * @return FormField
     */
    public function setWrap($wrap)
    {
        $this->wrap = $wrap;
        return $this;
    }






    /**
     * Inicializuje pole podle asociativniho pole $options
     * @param array $options
     * @deprecated
     */
    private function init($options)
    {
        if (!$options) {
            return;
        }

        //$this ->editable = true;
        $this ->readOnly = false;

        $this ->selectDataTypeTest = true;

        foreach ($options as $key => $value) {
            switch ($key) {
                case 'type' :
                    $this ->setFieldType($value);
                    break;

                case 'name' :
                    $this ->setFieldName($value);
                    break;

                case 'sqlFieldname' :
                    $this ->sqlFieldName = $value;
                    break;

                case 'sqlIndex' :
                    $this ->sqlFieldIndex = $value;
                    break;

                case 'readOnly' :
                    $this ->readOnly = (bool)$value;
                    //$this ->editable = !$this ->readOnly;
                    break;
                case 'edit' :
                    $this ->readOnly = !$value;
                    //$this ->editable = !$this ->readOnly;
                    break;

                case 'sql' :
                    $this->selectSql = $value;
                    break;

                case 'captionText' :
                    $this ->caption = TextManager::_($value);
                    break;

                case 'defaultValue' :
                    $this ->setDefaultValue($value);
                    break;

                case 'caption2First' :
                case 'primary' :
                case 'required' :
                case 'wrap' :
                    $this ->{$key} = (int)$value;
                    break;

                case 'caption' :
                case 'caption2' :
                case 'dataType' :
                case 'maxLength' :
                case 'rows' :
                case 'cols' :
                case 'text' :
                case 'options' :
                case 'rowId' :
                case 'rowClass' :
                default :
                    $this ->{$key} = $value;
                    break;
            }
        }

        global $development;
        if ($development) {
            MessageManager::addWarning('Deprecated: inicializace polozek formulare pomoci pole je zastarala! (' . $this ->fieldName . ')');
        }
        MessageManager::addLogMessage('Deprecated: inicializace polozek formulare pomoci pole je zastarala! (' . $this ->fieldName . ')');
    }


    /**
     * Nacte nastaveni jedne vlastnosti (atributu) z konfigurace
     * @param $key
     * @param $value
     */
    protected function loadConfigurationItem($key, $value)
    {
        $textValue = ($value instanceof \ArrayAccess)
            ? $value[0]['#text'] ?? '' // pouzije se v pripade, ze je misto atributu zadana hodnota v samostatnem tagu
            : $value;

        if (strpos($key, 'html:') === 0) {
            if (!$this ->otherHtmlParams) {
                $this ->otherHtmlParams = [];
            }

            $this ->otherHtmlParams[substr($key, 5)] = TextManager::_($textValue);
        } else {
            switch ($key) {
                case 'fieldName' :
                case 'sqlFieldName' :
                case 'tableName' :
                case 'tableAlias' :
                case 'fieldType' :
                case 'defaultValue' :
                case 'selectSql' :
                case 'selectSqlFilter' :
                case 'selectSqlOrder' :
                case 'dataType' :
                case 'fieldId' :
                case 'rowId' :
                case 'fieldCssClass' :
                case 'rowClass' :
                case 'captionColumnClass' :
                case 'dataColumnClass' :
                case 'accept' :
                    $this->{$key} = $textValue;
                    break;

                case 'addFieldCssClass' :
                    $this ->fieldCssClass .= ' ' . $textValue;
                    break;

//                case 'addRowClass' : - nefunguje, protoze vychozi je null
//                    $this ->rowClass .= ' ' . $value;
//                    break;

//                case 'addCaptionColumnClass' :
//                    $this ->captionColumnClass .= ' ' . $value;
//                    break;

                case 'caption' :
                case 'caption2' :
                case 'text' :
                    $this->{$key} = TextManager::_($textValue);
                    break;

                case 'visible' :
                case 'required' :
                case 'requiredInDatabase' :
                case 'changeable' :
                case 'sqlFieldIndex' :
                case 'useDefaultValueToEditNull' :
                case 'selectDataTypeTest' :
                case 'selectShowCode' :
                case 'maxLength' :
                case 'rows' :
                case 'cols' :
                case 'wrap' :
                case 'currentTimeButton' :
                case 'newRow' :
                //case 'newFieldGroup' :
                case 'showColumns' :
                    $this->{$key} = (int)$textValue;
                    break;

                case 'readOnly' :
                    $this->readOnly = (int)$textValue;
                    //$this->editable = !$this ->readOnly;
                    break;

                case 'preferredValues' :
                    $this ->preferredValues = $value ? explode(';', $textValue) : null;
                    break;

                case 'options' :
                    $options = $textValue ? explode(';', $textValue) : null;
                    // provede preklad
                    foreach ($options as $index => $optionValue) {
                        $options[$index] = TextManager::_($optionValue);
                    }

                    $this ->options = ($options  &&  $this ->fieldType == self::FIELD_TYPE_COMBOSTR)
                        ? array_combine($options, $options) // pro typ pole "combostr" pouzije stejne hodnoty pro zobrazeni i zapis do databaze
                        : $options;

                    break;
            }
        }
    }


    /**
     * Zpracuje konfiguraci sloupce nactenou z XML pomoci @XmlLoader
     * @param array $confArray
     */
    public function loadConfigurationArray($confArray)
    {
        if (!$confArray instanceof \ArrayAccess) {
            return;
        }

        foreach ($confArray as $key => $value) {
            $this ->loadConfigurationItem($key, $value);
        }
    }


    /**
     * Vraci hodnotu zadanou uzivatelem.
     * @param bool $utf8
     * @return string|array
     */
    public function getPostValueText($utf8 = false)
    {
        if (!$this->fieldName) {
            return '';
        }

        $value = $_POST[$this->fieldName] ?? '';

        if ($this->fieldType === self::FIELD_TYPE_DATEANDTIME) {
            $timeStr = isset($_POST[$this->fieldName . '_time']) ? $_POST[$this->fieldName . '_time'] : '';
            return $utf8
                ? [StringUtils::fromUTF8($value), StringUtils::fromUTF8($timeStr)]
                : [$value, $timeStr];
        }

        return $utf8
            ? StringUtils::fromUTF8($value)
            : $value;
    }


    /**
     * Nacte hodnotu zadanou uzivatelem (z POST) a zpracuje ji (prevede text na hodnotu podle typu pole)
     * @throws RonException
     */
    public function getPostValue($utf8)
    {
        if ($this ->fieldType === self::FIELD_TYPE_CONSTANT) {
            return $this ->defaultValue;
        }

        $value = $this->getPostValueText($utf8);

        try {
            switch ($this->fieldType) {
                case self::FIELD_TYPE_EDIT :
                case self::FIELD_TYPE_MEMO :
                case self::FIELD_TYPE_SELECT :
                    $value = trim($value);
                // pokracuje dale
                case 'passwd' :
                case self::FIELD_TYPE_PASSWORD :
                case self::FIELD_TYPE_COMBOSTR :
                    if ($value === '') {
                        return null;
                    }
                    if ($this->dataType === 'int') {
                        return (int)$value;
                    }
                    return $value;

                case self::FIELD_TYPE_COMBO :
                case self::FIELD_TYPE_RADIO :
                    return (isset($value) && trim($value) !== '')
                        ? $value
                        : null;

                case self::FIELD_TYPE_ACTUAL_TIME :
                    return time();

                case self::FIELD_TYPE_ACTUAL_DATE :
                    return DateUtils::startOfTheDay(time());

                case self::FIELD_TYPE_DATE :
                    return (trim($value) === '')
                        ? null
                        : DateUtils::dateStringToTimestamp($value);

                case self::FIELD_TYPE_TIME :
                    return (trim($value) == '')
                        ? null
                        : DateUtils::timeStringToTimestamp($value);

                case self::FIELD_TYPE_DATETIME :
                    return (trim($value) == '')
                        ? null
                        : DateUtils::datetimeStringToTimestamp($value);

                case self::FIELD_TYPE_DATEANDTIME :
                    return ((trim($value[0]) == '') && (trim($value[1]) == ''))
                        ? null
                        : DateUtils::datetimeStringToTimestamp($value[0] . ' ' . $value[1]);
                //: DateUtils::datetimeStringToTimestamp($value[0]) + DateUtils::timeStringToTimestamp($value[1], 0, true); - nefunguje spravne pri zmene casu, napriklad 31.10.2021

                case self::FIELD_TYPE_INT :
                    return (trim($value) == '')
                        ? null
                        : StringUtils::stringToInteger($value);

                case self::FIELD_TYPE_FLOAT :
                    if (trim($value) == '') {
                        return null;
                    }
                    if (!StringUtils::isFloat($value)) {
                        throw new Exception(TextManager::_('Bad_number_format'));
                    }
                    return StringUtils::stringToFloat($value);

                case self::FIELD_TYPE_HIDDEN :
                case self::FIELD_TYPE_ID :
                    return stripslashes($value);

                case self::FIELD_TYPE_CHECK :
                case self::FIELD_TYPE_NOT_NULL_FLAG :
                    return $value ? 1 : 0;
            }
        } catch (Exception $e) {
            throw new RonException($e ->getMessage(), $this ->fieldName);
        }

        return null;
    }


    public function showCalendarButton()
    {
        $fieldId = $this ->fieldId ?? $this ->otherHtmlParams['id'] ?? null;

        echo
            "<i class='showcalendar inline-glyph glyph-button fa glyph-calendar' " .
            ($fieldId
                ? "data-roncalendar-input='#$fieldId'"
                : "data-roncalendar-inputname='$this->fieldName'") .
            "></i>";
    }


    public function showCurrentTimeButton()
    {
        if (!$this ->fieldId) {
            return;
        }

        echo
            "<button class='button actual-time-button' type='button' data-date-input='{$this ->fieldId}' data-time-input='{$this ->fieldId}_time'>" .
            TextManager::_('Insert_current_time') .
            '</button>';
    }


    public function getFieldCssClass()
    {
        $className = '';

//        switch ($this->fieldType) {
//            case self::FIELD_TYPE_DATE :
//                $className = 'inputDate';
//                break;
//            case self::FIELD_TYPE_DATETIME :
//            case self::FIELD_TYPE_DATEANDTIME :
//                $className = '';
//                break;
//
//            case self::FIELD_TYPE_INT :
//                $className = 'form-control inputInt';
//                break;
//
//            case self::FIELD_TYPE_FLOAT :
//                $className = 'form-control inputFloat';
//                break;
//
//            case self::FIELD_TYPE_CHECK :
//                $className = 'form-check-input';
//                break;
//
//            case self::FIELD_TYPE_EDIT :
//            case self::FIELD_TYPE_MEMO :
//            case self::FIELD_TYPE_SELECT :
//                $className = 'form-control';
//                break;
//
//            case self::FIELD_TYPE_SECTION_HEADER :
//            case self::FIELD_TYPE_GROUP :
//            case self::FIELD_TYPE_TEXT :
//                break;
//
//            default :
//                $className = '';
//        }

        if ($this ->fieldCssClass) {
            $className .=
                ($className ? ' ' : '') . $this ->fieldCssClass;
        }

        if ($this ->fieldName  &&  $msgClass = MessageManager::getClass($this ->fieldName)) {
            $className .=
                ($className ? ' ' : '') . $msgClass;
        }

        return $className;
    }


    /**
     * Vraci asociativni pole HTML atributu pro formularove pole
     * @param string[]|null $defaultAttrs Asociativni pole - umoznuje zadat vychozi hodnoty pro atributy
     * @param array $rowData
     * @return string[]
     */
    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $attrs = $defaultAttrs ?: [];

        $className = $this ->getFieldCssClass();
        if ($className) {
            $attrs['class'] = $className;
        }

        //if ($this ->fieldId  &&  $this ->fieldType != self::FIELD_TYPE_DATEANDTIME) {
        if ($this ->fieldId) {
            $attrs['id'] = $this ->fieldId;
        }

        if ($this ->readOnly) {
            $attrs['data-readonly'] = '1';
        }

        if ($this ->maxLength) {
            $attrs['maxlength'] = $this ->maxLength;
        }

        if ($this ->code) {
            $attrs['data-code'] = $this ->code;
        }

        if ($this ->tableName) {
            $attrs['data-table-name'] = $this ->tableName;
        }

        if ($this ->defaultValue !== false  &&  !is_array($this ->defaultValue)) {
            $attrs['data-ronform-default-value'] = isset($this ->defaultValue)
                ? ($this ->defaultValue)
                : '';
        }

        if ($this ->selectSqlFilter) {
            $attrs['data-filter-supported'] = "yes";
        }

        if ($this ->translationPrefix) {
            $attrs['data-translation-prefix'] = $this ->translationPrefix;
        }

        if (is_array($this ->otherHtmlParams)) {
            $attrs = array_merge($attrs, $this ->otherHtmlParams);
        }

        if ($this ->dataAttributes  &&  $rowData) {
            foreach ($this ->dataAttributes as $name => $index) {
                if (isset($rowData[$index])) {
                    // TODO: umoznit formatovani, napr. pomoci zadani typu za dvojteckou ve jmene
                    $attrs[$name] = StringUtils::htmlTextConvert($rowData[$index]);
                }
            }
        }

        // title z message managera prebije i otherHtmlParams
        if ($this ->fieldName) {
            $title = MessageManager::getTitle($this->fieldName);
            if ($title) {
                $attrs['title'] = $title;
            }
        }

        return $attrs;
    }


    /**
     * Vraci HTML atributy formularove pole
     * @param string[]|null $defaultParams Asociativni pole, ktere umoznuje zadat vychozi hodnoty atributu
     * @param array $rowData
     * @return string Vraci primo text, ktery se muze vlozit do HTML
     */
    public function getHtmlParams($defaultParams = null, $rowData = null)
    {
        $result = GuiUtils::getAttributesHtml($this ->getFieldAttributes($defaultParams, $rowData));

        // TODO: odstranit
        if ($this ->otherHtmlParams  &&  is_string($this ->otherHtmlParams)) {
            // nemelo by nastat - otherHtmlParams uz by melo byt vzdy array
            $result .= ' ' . $this ->otherHtmlParams;
        }
        return $result;
    }


    public function loadOptions($database)
    {
        if ($this->selectSql instanceof SqlStatement) {
            $sql = $this->selectSql->sqlText;
            $params = $this->selectSql->parameters;
        } else {
            $sql = $this->selectSql;
            $params = null;
        }
        $sql = $this->selectSqlFilter
            ? str_replace('%FILTER%', $this->selectSqlFilter, $sql)
            : $sql;
        $sql = $this->selectSqlOrder
            ? str_replace('%ORDER%', $this->selectSqlOrder, $sql)
            : $sql;

        $this->options = (new OptionLoader($database))
            ->setSql($sql, $params)
            ->setAttrIndexes($this ->itemsAttributes)
            ->setPreferredValues($this ->preferredValues)
            ->setFilterFunction($this ->selectFilterFunction)
            ->setTranslationPrefix($this ->translationPrefix, $this ->translationValueIndex)
            ->setTitleTranslationPrefix($this ->titleTranslationPrefix)
            ->loadData();

    }


    public function renderSelect($database, $fieldName, $value, $defaultValue, $insert, $canEdit, $required)
    {
        if (!$this ->options) {
            $this ->loadOptions($database);
        }

        $combo = new ComboBox($fieldName);
        $combo ->dataTypeComparison = $this ->selectDataTypeTest;
        $combo ->preferredValues = $this ->preferredValues;
        if ($this ->showEmptyValue !== false) {
            if ($this->showEmptyValue === true || !($required && $defaultValue)) {
                $combo->emptyRow = isset($this ->showEmptyValue) ? $this ->showEmptyValue : true;
            }
        }
        $combo ->alwaysShowSelected = !$insert  &&  $value !== ''; // true; od 10-2023 - doplnena podminka !$insert
        $combo ->attrIndexes = $this ->itemsAttributes;
        $combo ->htmlParams = $this ->getFieldAttributes(); // . ($this->fieldId ? " id='" . $this->fieldId . "'" : '');;
        $combo ->readOnly = !$canEdit;
        $combo ->showCode = $this ->selectShowCode;
//        $combo ->translationPrefix = $this ->translationPrefix;
//        $combo ->translationValueIndex = $this ->translationValueIndex;
//        $combo ->titleTranslationPrefix = $this ->titleTranslationPrefix;

        $combo ->data = $this ->options;
        $combo->setSelectedOption($value);

//        if ($this ->options) {
//            $combo ->data = $this ->options;
//        } else if ($this ->selectSql) {
//            if ($this->selectSql instanceof SqlStatement) {
//                $sql = $this->selectSql->sqlText;
//                $params = $this->selectSql->parameters;
//            } else {
//                $sql = $this->selectSql;
//                $params = null;
//            }
//            $sql = $this->selectSqlFilter
//                ? str_replace('%FILTER%', $this->selectSqlFilter, $sql)
//                : $sql;
//            $sql = $this->selectSqlOrder
//                ? str_replace('%ORDER%', $this->selectSqlOrder, $sql)
//                : $sql;
//            $combo->loadData($database, $sql, $value, $params, $this->selectFilterFunction);
//        }
        $combo->render(); // width = 0
    }


    public function findOption($value)
    {
        if (!$this ->options) {
            return null;
        }

        foreach ($this ->options as $option) {
            if ($option instanceof Option) {
                if ($option ->key == $value) {
                    return $option;
                }
            }
        }

        return null;
    }

    /**
     * Vraci text, ktery se zobrazi - pro nektere prvky se nepouziva, protoze nezobrazuji text (napr. editovatelny checkbox)
     * @param $database
     * @param $value
     * @param $canEdit
     * @param $insert
     * @param $rowData
     * @return false|string|null
     */
    protected function getFieldText($database, $value, $canEdit, $insert, $rowData)
    {
        switch ($this->fieldType) {
            case self::FIELD_TYPE_SECTION_HEADER :
                return $this ->caption;

            case self::FIELD_TYPE_MEMO :
                return $canEdit
                    ? StringUtils::htmlTextConvert($value)
                    : StringUtils::longTextToHtml($value, true);

            case self::FIELD_TYPE_TEXT :
                return
                    ($this->text ?? '') .
                    ($value ?? '');

            case self::FIELD_TYPE_CHECK :   // pouzije se pouze pro readonly zobrazeni
                return TextManager::_($value ? 'Str_Yes' : 'Str_No');

            case self::FIELD_TYPE_SELECT :
                return StringUtils::htmlTextConvert($this ->findOption($value) ->caption ?? $value);

            default :
                return StringUtils::htmlTextConvert($value);
        }
    }


    /**
     * Zobrazi obsah pole.
     * @param Database $database
     * @param mixed $value
     * @param bool $canEdit Pokud je false, nebude pole editovatelne
     * @param bool $insert Urcuje, jestli se jedna o vkladani noveho zaznamu
     * @param array $rowData Data nactena z databaze
     * @returns void
     * @throws DBException
     */
    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        switch ($this->fieldType) {
            /** @noinspection PhpMissingBreakStatementInspection */
            case self::FIELD_TYPE_ACTUAL_TIME :
            /** @noinspection PhpMissingBreakStatementInspection */
            case self::FIELD_TYPE_ACTUAL_DATE :
                //if (!$canEdit) return;
                // jinak pokracuje dale                     PROC?
                return;

            case self::FIELD_TYPE_ID :
            case self::FIELD_TYPE_HIDDEN :
                echo
                    "<input " . $this ->getHtmlParams(['type' => 'hidden', 'name' => $this ->fieldName], $rowData) .
                    " value='" . $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) . "'/>";
                return;

            case self::FIELD_TYPE_CONSTANT :
                return;

            case self::FIELD_TYPE_SECTION_HEADER :
                echo
                    '<span ' . $this ->getHtmlParams(null, $rowData) . '>' .
                    $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) .
                    '</span>';
                return;


            case self::FIELD_TYPE_DATE :
            case self::FIELD_TYPE_TIME :
            case self::FIELD_TYPE_EDIT :
            case self::FIELD_TYPE_INT :
            case self::FIELD_TYPE_FLOAT :
            case self::FIELD_TYPE_DATETIME :
                if ($canEdit) { // $readonly = ""; else $readonly = 'readonly="readonly"';
                    echo
                        '<INPUT ' .
                        $this ->getHtmlParams(['type' => 'text', 'name' => $this ->fieldName], $rowData) .
                        ' value="' . $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) . '">';

                    if ($this->fieldType === self::FIELD_TYPE_DATE) {
                        $this->showCalendarButton();
                    }
                } else {
                    echo
                        '<div ' . $this ->getHtmlParams($this ->fieldName ? ['data-ronform-field-name' => $this ->fieldName] : null, $rowData) . '>' .
                        $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) . '&nbsp;' .
                        '</div>';
                }
                break;

            case self::FIELD_TYPE_DATEANDTIME : // 2 samostatna pole pro zadani casu a data
                if ($canEdit) {
                    echo '<span>';

                    // Edit pro datum
                    $valuestr = $value ? $value[0] : '';

                    $attrs = $this ->getFieldAttributes(['type' => 'text', 'maxlength' => '10', 'size' => '10']);
                    $attrs['class'] = (isset($attrs['class']) ? $attrs['class'] . ' ' : '') . 'inputDate';
                    $attrs['data-ronform-default-value'] = $this->defaultValue[0] ?? '';

                    echo
                        '<input name="' . $this->fieldName . '" ' .
                        //($this ->fieldId ? " id='{$this ->fieldId}_date'" : '') .
                        GuiUtils::getAttributesHtml($attrs) .
                        ' value="' . StringUtils::htmlTextConvert($valuestr) . '">';

                    $this->showCalendarButton();

                    // Edit pro cas
                    $valuestr = $value ? $value[1] : '';

                    $attrs = $this ->getFieldAttributes();
                    $attrs['class'] = (isset($attrs['class']) ? $attrs['class'] . ' ' : '') . 'inputTime';
                    $attrs['data-ronform-default-value'] = $this->defaultValue[1] ?? '';
                    if (isset($attrs['id'])) {
                        $attrs['id'] .= '_time';
                    }
                    $attrs = array_merge(['type' => 'text', 'maxlength' => '8', 'size' => '8'], $attrs, $this ->otherHtmlParams2);

                    echo
                        ' <input name="' . $this->fieldName . '_time" ' .
                        //($this ->fieldId ? " id='{$this ->fieldId}_time'" : '') .
                        GuiUtils::getAttributesHtml($attrs) .
                        ' value="' . StringUtils::htmlTextConvert($valuestr) . '">';

                    if ($this ->currentTimeButton) {
                        echo ' ';
                        $this->showCurrentTimeButton();
                    }
                    echo '</span>';
                } else {
                    echo $value
                        ? $value[0] . ' ' . $value[1]
                        : '';
                }
                break;

            case 'passwd' :
            case self::FIELD_TYPE_PASSWORD :
                echo
                    '<INPUT value="" ' . $this ->getHtmlParams(['type' => 'password', 'name' => $this ->fieldName], $rowData) . '>';
                break;

            case self::FIELD_TYPE_MEMO :
                $cols = $this->cols ?: 60;
                $rows = $this->rows ?: 8;
                $wrap = $this->wrap ?: 'virtual';
                if ($canEdit) {
                    echo
                        "<TEXTAREA " .
                        $this ->getHtmlParams(['cols' => $cols, 'rows' => $rows, 'name' => $this ->fieldName, 'wrap' => $wrap], $rowData) .
                        ">" .
                        $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) .
                        '</TEXTAREA>';
                } else {
                    echo
                        '<div ' . $this ->getHtmlParams($this ->fieldName ? ['data-ronform-field-name' => $this ->fieldName] : null, $rowData) . '>' .
                        $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) .
                        '</div>';
                }
                break;

            case self::FIELD_TYPE_CHECK :
//                if ($canEdit) { 17.1.2025 zmena chovani, vzdy se zobrazi checkbox (drive se pro needitovatelne zobrazil text Ano/Ne)
                    echo
                        "<INPUT " . ($value ? ' checked="checked" ' : '') .
                        $this ->getHtmlParams(array_merge(['type' => 'checkbox', 'name' => $this ->fieldName, 'value' => '1'], $canEdit ? [] : ['disabled' => 'disabled']), $rowData) .
                        '> ';
//                } else {
//                    echo $this ->getFieldText($database, $value, $canEdit, $insert, $rowData);
//                }
                break;

            case self::FIELD_TYPE_NOT_NULL_FLAG :
                echo
                    "<INPUT " .
                    ($value ? 'checked="checked" ' : '') . ($canEdit ? '' : ' disabled="disabled"') .
                    $this ->getHtmlParams(['type' => 'checkbox', 'name' => $this ->fieldName, 'value' => '1'], $rowData) .
                    '> ';
                break;

            case self::FIELD_TYPE_SELECT :
                if (!$canEdit) {
                    echo '<div ' . $this->getHtmlParams($this->fieldName ? ['data-ronform-field-name' => $this->fieldName] : null, $rowData) . '>';
                }
                $this ->renderSelect($database, $this ->fieldName, $value, $this ->defaultValue, $insert, $canEdit, $this->required);
                if (!$canEdit) {
                    echo '</div>';
                }

                break;

            case self::FIELD_TYPE_COMBO :
            case self::FIELD_TYPE_COMBOSTR :
                if ($canEdit) {
                    ComboBox::renderComboFromArray($this->options,
                        $this->fieldName,
                        $value, $this->getHtmlParams(null, $rowData), $this->selectDataTypeTest);
                } else {
                    echo
                        '<div ' . $this ->getHtmlParams($this->fieldName ? ['data-ronform-field-name' => $this->fieldName] : null, $rowData) . '>' .
                        ($this->options[$value] ?? '') . // zobrazit $value?
                        '</div>';
                }
                break;

            case self::FIELD_TYPE_RADIO :
                if ($canEdit) {
                    foreach ($this ->options as $key => $text) {
                        echo
                            '<label ' . GuiUtils::getAttributesHtml($this ->otherHtmlParams2) . '>' .
                            "<input value='{$key}' " . (($value == $key) ? 'checked="checked"' : '') .
                            $this ->getHtmlParams(['type' => 'radio', 'name' => $this ->fieldName], $rowData) .
                            '/>' .
                            $text .
                            '</label>';
                    }
                } else {
                    echo $this->options[$value];
                }
                break;



            case self::FIELD_TYPE_TEXT :
                $params =
                    $this ->getHtmlParams(null, $rowData);

                echo
                    ($params ? "<div $params>" : '') .
                    $this ->getFieldText($database, $value, $canEdit, $insert, $rowData) .
                    ($params ? '</div>' : '');
                break;

            /** @noinspection PhpMissingBreakStatementInspection */
            case self::FIELD_TYPE_FILENAME :
                if (!$insert && $canEdit) {
                    //echo '<INPUT type=text name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '">';
                    echo '<input ' . $this ->getHtmlParams(['type' => 'text', 'name' => $this ->fieldName, 'value' => $value]) . '>';
                    break;
                }
            // POKRACUJE SE DALE!
            case self::FIELD_TYPE_FILE :
                echo $canEdit
                    //? '<INPUT type="file" name="' . $this->fieldName . '">'
                    ? '<input ' . $this ->getHtmlParams(['type' => 'file', 'name' => $this ->fieldName, 'accept' => $this ->accept]) . '>'
                    : $this ->getFieldText($database, $value, $canEdit, $insert, $rowData);
                break;
        }
    }


    function getCaptionAttributes($defaultClass)
    {
        return [
            'class' => $this ->captionColumnClass ?: $defaultClass . ($this->required ? ' required' : '')
        ];
    }

    function getDataColAttributes($defaultClass)
    {
        return [
            'class' => $this->dataColumnClass ?: $defaultClass
        ];
    }

    /**
     * Zobrazi formularove pole.
     * @param $connection
     * @param $value
     * @param bool $canEdit Pomoci $canedit lze zakazat editaci.
     * @param bool $insert  Urcuje, jestli se jedna o vkladani noveho zaznamu
     * @param array $rowData Data nactena z databaze
     * @throws DBException
     */
    public function showField($connection, $value, $canEdit, $insert, $rowData)
    {
        if (!$insert && $this->primary && $this->fieldType !== self::FIELD_TYPE_HIDDEN && $this->fieldType !== self::FIELD_TYPE_ID) {
            // hidden pole a "id" pole se vlozi automaticky
            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '">';
        }

//        $codeHtml = $this ->code
//            ? " data-code='{$this ->code}'"
//            : '';

//        $idHtml = $this ->fieldId
//            ? " id='{$this ->fieldId}'"
//            : '';

//        $rowid = isset($this ->rowId)
//            ? $this ->rowId
//            : ( $this ->fieldName ? 'row_' . $this ->fieldName : '');
//
//        if ($tableRow) {
//            echo '<div' . ($rowid ? " id='$rowid'" : '') . ($this ->rowClass ? " class='{$this ->rowClass}'" : '') . '>';
//        }

        // Nadpis pole - presunuto do formulare
        //echo '<div class="form-col1 caption' . ($this->required ? ' required' : '') . '">' . $this->caption . '</div>';

        // Hodnota pole
        //echo "<div class='form-col2'>";
        if ($this->caption2  &&  $this ->caption2First) {
            echo $this->caption2 . ' ';
        }

        $this->showFieldContent($connection, $value, $canEdit, $insert, $rowData);

        if ($this->caption2  &&  !$this ->caption2First) {
            echo ' ' . $this->caption2;
        }
        //echo '</div>';

//        if ($tableRow) {
//            echo '</div>';
//        }

        //echo "\n";
    }


    /**
     * Nacte hodnotu pole pro vkladani - naplni ho defaultni hodnotou
     * nebo hodnotou, kterou uz uzivatel zadal (pokud je $usepostvalues true)
     * Pri kopirovani existujiciho zaznamu muze byt zadano i $rowdata.
     * @param Database $database
     * @param bool $usePostValues
     * @param FormRowArray $rowData
     */
    public function loadInsertValue($database, $usePostValues, $rowData = null)
    {
        if ($this ->readOnly === 2) {
            // pole nelze editovat ani pri vkladani
            $usePostValues = false;
        }

        if (!$usePostValues  &&  $rowData  &&  isset($this->sqlFieldIndex) && $this->sqlFieldIndex >= 0) {
            // pokud jsou nactena data (napr. pri kopirovani) pouzije hodnotu z puvodniho (kopirovaneho) zaznamu
            switch ($this->fieldType) {
                case self::FIELD_TYPE_CONSTANT :
                case self::FIELD_TYPE_ID :
                case self::FIELD_TYPE_PASSWORD :
                case 'passwd' :
                case self::FIELD_TYPE_ACTUAL_TIME :
                case self::FIELD_TYPE_ACTUAL_DATE :
                    break;

                default :
                    $this->value = $this->getValueFromDBValue($database, $rowData[$this->sqlFieldIndex], $rowData);
                    return;
            }
        }

        switch ($this->fieldType) {
            case self::FIELD_TYPE_HIDDEN :
            case self::FIELD_TYPE_CONSTANT :
            case self::FIELD_TYPE_ID :
            case self::FIELD_TYPE_PASSWORD :
            case 'passwd' :
                $this ->value = $this->defaultValue;
                break;

            case self::FIELD_TYPE_ACTUAL_TIME :
                $this ->value = null; // $database ->timestampToString(time());
                break;

            case self::FIELD_TYPE_ACTUAL_DATE :
                $this ->value = null; // $database ->timestampToString(DateUtils::startOfTheDay(time()));
                break;

            case self::FIELD_TYPE_DATEANDTIME :
                // datum a cas rozdeleny do dvou poli - jako hodnota se predava pole se dvema prvky!
                if ($usePostValues) {
                    $this ->value = $this->getPostValueText();
                }
                else if ($this->defaultValue) {
                    $this ->value = is_array($this ->defaultValue)
                        ? $this ->defaultValue
                        : [date('j.n.Y', $this->defaultValue), date('H:i', $this->defaultValue)];
                }
                else {
                    $this ->value = ['', ''];
                }
                break;

            default :
                $this ->value = $usePostValues
                    ? $this->getPostValueText()
                    : ($_GET[$this->fieldName] ?? $this->defaultValue);
        }
    }


//    /**
//     * Zobrazi zadane pole pro vkladani - naplni ho defaultni hodnotou
//     * nebo hodnotou, kterou uz uzivatel zadal (pokud je $usepostvalues true) - napr.
//     * v pripade, ze zapomnel vyplnit nejakou povinnou hodnotu
//     * pri kopirovani existujiciho zaznamu muze byt zadano i $rowdata
//     * @param Database $database
//     * @param bool $usePostValues
//     * @param array $rowData
//     * @throws DBException
//     */
//    public function showFieldInsert($database, $usePostValues, $rowData = null)
//    {
//        switch ($this->fieldType) {
//            case self::FIELD_TYPE_HIDDEN :
//            case self::FIELD_TYPE_CONSTANT :
//            case self::FIELD_TYPE_ID :
//            case self::FIELD_TYPE_PASSWORD :
//            case 'passwd' :
//                $value = $this->defaultValue;
//                break;
//
//            case self::FIELD_TYPE_ACTUAL_TIME :
//                $value = null; // $database ->timestampToString(time());
//                break;
//
//            case self::FIELD_TYPE_ACTUAL_DATE :
//                $value = null; // $database ->timestampToString(DateUtils::startOfTheDay(time()));
//                break;
//
//            case self::FIELD_TYPE_DATEANDTIME :
//                // datum a cas rozdeleny do dvou poli - jako hodnota se predava pole se dvema prvky!
//                if ($usePostValues) {
//                    $value = $this->getPostValueText();
//                }
//                else if ($this->defaultValue) {
//                    $value = [date('j.n.Y', $this->defaultValue), date('H:i', $this->defaultValue)];
//                }
//                else {
//                    $value = ['', ''];
//                }
//                break;
//
//            default :
//                $defvalue = isset($_GET[$this->fieldName]) ? $_GET[$this->fieldName] : $this->defaultValue;
//                $value = $usePostValues ? $this->getPostValueText() : $defvalue;
//        }
//
//        $this->showField($database, $value, true, true, $rowData); // canedit = true, insert = true, rowdata = null (insert)
//    }

    /**
     * @param Database $database
     * @param FormRowArray|null $rowData
     * @return void
     */
    public function loadDatabaseValue(Database $database, ?FormRowArray $rowData)
    {
        if (isset($this->sqlFieldIndex)) {
            $index = $this ->sqlFieldIndex >= 0
                ? $this ->sqlFieldIndex
                : count($rowData) + $this ->sqlFieldIndex;

            if (isset($rowData)) { //  &&  count($rowData) > $index) {
                $this->value = $this->getValueFromDBValue($database, ($index >= 0  &&  $index < count($rowData)) ? $rowData[$index] : null, $rowData);
            } else {
                $this ->value = $this ->defaultValue;
            }
        } else {
            $this ->value = $this->defaultValue;
        }
    }


    public function loadEditValue($database, $usePostValues, $rowData, $canEditRecord)
    {
        if ($usePostValues && $canEditRecord && !$this->readOnly) {
            $this ->value = $this->getPostValueText();
        } else {
            // Standardni zobrazeni hodnot z databaze
            $this ->loadDatabaseValue($database, $rowData);
        }

        if ($canEditRecord && !$this ->readOnly) {
            if ($this ->useDefaultValueToEditNull  &&  !isset($this ->value)) {
                $this ->value = $this ->defaultValue;
            }
        }
    }


    /**
     * @param Database $database
     * @param FormRowArray $rowData
     * @return mixed|string|\string[]
     */
    public function loadJsonValue($database, $rowData)
    {
        if ($rowData && isset($this->sqlFieldIndex) && $this->sqlFieldIndex >= 0) {
            if ($this ->fieldType === FormField::FIELD_TYPE_TEXT  &&  !$this ->format) {
                return isset($rowData[$this->sqlFieldIndex])
                    ? $rowData[$this->sqlFieldIndex]
                    : '';
            }

            $result = $this->getValueFromDBValue($database, $rowData[$this->sqlFieldIndex], $rowData);

            if ($this ->options  &&  ($this ->fieldType === FormField::FIELD_TYPE_COMBO || $this ->fieldType === FormField::FIELD_TYPE_COMBOSTR)) {
                return $this ->options[$result];
            }

            return $result;
        }

        return $this->defaultValue;
    }

//    /**
//     * Zobrazi zadane pole pro editaci.
//     * @param Database $database
//     * @param mixed $value
//     * @param array $rowData
//     * @throws DBException
//     */
//    public function showFieldEdit($database, $value, $rowData)
//    {
//        if ($this ->useDefaultValueToEditNull  &&  !isset($value)) {
//            $value = $this ->defaultValue;
//        }
//
//        if ($this->primary && $this->fieldType !== self::FIELD_TYPE_HIDDEN && $this->fieldType !== self::FIELD_TYPE_ID) {
//            // hidden pole a "id" pole se vlozi automaticky
//            echo '<INPUT type="hidden" name="' . $this->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '">';
//        }
//
//        $canEdit = !$this ->readOnly && $this->editable && !$this->primary;     // $this->editable pro kompatibilitu se starym kodem
//        if (!$this ->changeable  &&  $value) {
//            $canEdit = false;
//        }
//
//        $this->showField($database, $value, $canEdit, false, $rowData); // insert = false
//    }


//    /**
//     * Zobrazi zadane pole bez moznosti editace
//     * @param Database $database
//     * @param mixed $value
//     * @param array $rowData
//     * @throws DBException
//     */
//    public function showFieldReadOnly($database, $value, $rowData)
//    {
//        $this->showField($database, $value, false, false, $rowData); // canedit = false, insert = false
//    }


    /**
     * @param Database $database
     * @param string $tableName
     * @param int $insert
     * @param bool $utf8 Urcuje, jestli se maji data konvertovat z UTF8 (pouziva se u dialogu volanych pres AJAX)
     * @return null|DBRecordField|DBRecordField[]
     * @throws DBException
     */
    public function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        if ($this ->includeToDBRecord === false  ||  (!$this->sqlFieldName  &&  $this ->includeToDBRecord !== true)) {
            return NULL;
        }

        switch ($this->fieldType) {
            case self::FIELD_TYPE_EDIT :
            case self::FIELD_TYPE_SELECT :
            case self::FIELD_TYPE_PASSWORD :
            case 'passwd' :
            case self::FIELD_TYPE_COMBOSTR :
                $field = new DBRecordField($this->sqlFieldName, $this ->dataType, $this ->readOnly, $this->maxLength); // format = ''
                $field ->setValue($this->getPostValue($utf8));
                break;

            case self::FIELD_TYPE_MEMO :
                $field = new DBRecordField($this->sqlFieldName, 'memo', $this ->readOnly, $this ->maxLength); // format = ''
                $field->setValue($this->getPostValue($utf8));
                break;

            case self::FIELD_TYPE_ID :
                $field = new DBRecordField($this->sqlFieldName, $this ->sequence ? 'sequence' : 'int', $this ->readOnly, $this->maxLength);
                if ($this ->sequence  &&  is_string($this ->sequence)) {
                    $field ->setSequenceName($this ->sequence);
                }
                if ($insert) {
                    if (!$this ->sequence) {
                        $field->setValue(SQLUtils::newSerialNumber($database, $tableName, $this->sqlFieldName, $this->idSqlFilter));
                    }
                } else {
                    $field->setValue($this->getPostValue($utf8));
                }
                break;

            case self::FIELD_TYPE_HIDDEN :
            case self::FIELD_TYPE_CONSTANT :
                $field = new DBRecordField($this->sqlFieldName, $this ->dataType, $this ->readOnly, $this->maxLength);
                $field->setValue($this->getPostValue($utf8));
                break;

            case self::FIELD_TYPE_COMBO :
            case self::FIELD_TYPE_RADIO :
            case self::FIELD_TYPE_CHECK :
            case self::FIELD_TYPE_INT :
                $field = new DBRecordField($this->sqlFieldName, 'int', $this ->readOnly, $this->maxLength);
                $field->setValue($this->getPostValue($utf8));
                break;

            case self::FIELD_TYPE_FLOAT :
                $field = new DBRecordField($this->sqlFieldName, 'float', $this ->readOnly, $this->maxLength);
                $field->setValue($this->getPostValue($utf8));

                break;

            case self::FIELD_TYPE_TIME :
                $field = new DBRecordField($this ->sqlFieldName, 'time', $this ->readOnly);
                $field->setValue($this->getPostValue($utf8));
                break;

            case self::FIELD_TYPE_ACTUAL_TIME :
            case self::FIELD_TYPE_ACTUAL_DATE :
            case self::FIELD_TYPE_DATE :
            case self::FIELD_TYPE_DATETIME :
            case self::FIELD_TYPE_DATEANDTIME :
                $field = new DBRecordField($this->sqlFieldName, 'datetime', $this ->readOnly, 0); // format = ''
                $field->setValue($this->getPostValue($utf8));

                break;

            case self::FIELD_TYPE_TEXT :
                // pole musi mit nastaveno $includeToDBRecord na true
                $field = new DBRecordField('', '', true);
                $field ->setValue($this->text);
                break;

            default :
                return null;
        }

        $field ->setUpdateLog($this ->updateLog);
        $field ->setChangeable($this ->changeable);

        return $field;
    }


    /**
     * @param Database $database
     * @param mixed $dbValue
     * @param FormRowArray $rowData
     * @return string|string[]  Vraci text, ktery se zobrazi uzivateli pro zadanou hodnotu z databaze
     */
    public function getValueFromDBValue($database, $dbValue, $rowData)
    {
        switch ($this->fieldType) {
            case self::FIELD_TYPE_TIME :
                return $dbValue ? date('H:i', $database ->stringToTimestamp($dbValue)) : null; // letni/zimni cas?

            case self::FIELD_TYPE_DATE :
                return $dbValue ? date('j.n.Y', $database ->stringToTimestamp($dbValue)) : null;

            case self::FIELD_TYPE_DATETIME :
                return $dbValue ? date('j.n.Y H:i', $database ->stringToTimestamp($dbValue)) : null;

            case self::FIELD_TYPE_DATEANDTIME :
                $timestamp = $database ->stringToTimestamp($dbValue);

                return $timestamp
                    ? [date('j.n.Y', $timestamp), date('H:i', $timestamp)]
                    : null;

            case self::FIELD_TYPE_FLOAT :
                return $this ->format
                    ? StringUtils::format($this ->format, $dbValue)
                    : $dbValue;

            case self::FIELD_TYPE_HIDDEN :
                switch ($this->dataType) {
                    case 'date' :
                    case 'datetime' :
                        if ($this ->format) {
                            return isset($dbValue)
                                ? StringUtils::format($this ->format, $database ->stringToTimestamp($dbValue))
                                : null;
                        }
                        return $dbValue ? $database ->stringToTimestamp($dbValue) : null;

                    case 'int' :
                    default :
                        return $dbValue;
                }

            case self::FIELD_TYPE_TEXT :
                return StringUtils::format($this ->format, $dbValue);
        }

        return $dbValue;
    }


    /**
     * Vraci false pokud je pole povinne a jeho hodnota neni zadana, jinak vraci true
     * @param bool $edit Pokud je $edit true a pole neni editovatelne, tak vraci true
     * @return bool
     */
    public function checkRequired($edit)
    {
        if (!$this ->visible) {
            return true;
        }

        if ($edit && ($this ->readOnly)  ||  $this ->readOnly === 2) {
            return true; // pole nelze editovat
        }
        if (!$this->required) {
            return true;
        }
        if (!$this->fieldName) {
            return true;
        }
        if ($this ->otherHtmlParams  &&  isset($this ->otherHtmlParams['disabled'])) {
            return true; // pole je nastaveno "disabled"
        }

        if ($this ->fieldType === self::FIELD_TYPE_CONSTANT) {
            return true; // konstanta - uzivatel to nemuze ovlivnit
        }

        $value = $this ->getPostValueText();

        try {
            if ($this->fieldType === self::FIELD_TYPE_DATE && isset($value) && $value !== '') {
                $value = DateUtils::dateStringToTimestamp($value);
                if (!$value) {
                    return false;
                }
            }

            if ($this->fieldType === self::FIELD_TYPE_TIME && isset($value) && $value !== '') {
                $value = DateUtils::timeStringToTimestamp($value);
                if (!$value) {
                    return false;
                }
            }

            if ($this->fieldType === self::FIELD_TYPE_DATETIME && isset($value) && $value !== '') {
                $value = DateUtils::datetimeStringToTimestamp($value);
                if (!$value) {
                    return false;
                }
            }
        } catch (Exception $e) {
            return false;
        }

        if ($this->fieldType === self::FIELD_TYPE_CHECK || $this->fieldType === self::FIELD_TYPE_ID) {
            return true;
        }

        if ($this->fieldType === self::FIELD_TYPE_FILE && ($_FILES[$this->fieldName]['name'] ?? null)) {
            return true;
        }

        return (isset($value)  &&  $value !== '');
    }

}
