<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use ArrayAccess;
use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use Tracy\Debugger;


/**
 * Pomocna trida, ktera zajistuje pristup k datum. Umoznuje indexovani pomoci ciselnych indexu (tj. podle puvodniho selectu), ale take pomoci kodu poli ve formulari.
 * Class FormRowArray
 */
class FormRowArray implements ArrayAccess, \Countable
{
    /** @var array */
    protected $data = null;
    /** @var FormField[] */
    protected $formFields;


    /**
     * FormRowArray constructor.
     * @param FormField[] $formFields
     * @param ?array $data
     */
    public function __construct($formFields, $data)
    {
        //
        $this ->formFields = [];
        foreach ($formFields as $field) {
            if ($field ->code) {
                $this->formFields[$field->code] = $field;
            }
        }

        $this ->data = $data;
    }


    /**
     * @param array $data
     */
    public function setData($data): void
    {
        $this->data = $data;
    }


    /**
     * Vraci index datoveho pole se zadanym offsetem.
     * Offset muze byt primo index (integer, nebo string obsahujici integer),
     * nebo kod sloupce ve formulari.
     * @param $offset
     * @return int|null
     */
    private function getIndex($offset)
    {
        if (!$this ->data) {
            return null;
        }

        if (is_integer($offset)) {
            $intOffset = $offset;
        } else if (is_string($offset) && $offset !== '' &&  $offset[0] >= '0'  &&  $offset[0] <= '9') {
            // string reprezentujici integer - napriklad nacteny z XML
            $intOffset = (int)$offset;
        } else {
            $field = $this->formFields[$offset] ?? null;
            if (!$field) {
                return null;
            }

            $intOffset = $field->sqlFieldIndex;
        }

        return ($intOffset < 0)
            ? count($this ->data) + $intOffset
            : $intOffset;
    }


    public function offsetExists($offset): bool
    {
        if (isset($this->formFields[$offset])) {
            return true;
        }

        $index = $this ->getIndex($offset);
        return isset($index)  &&  $index >= 0  &&  $index < count($this ->data)  &&  isset($this ->data[$index]);
    }


    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        if (isset($this->formFields[$offset])) {
            return $this ->formFields[$offset] ->value;
        }

        $index = $this ->getIndex($offset);

        return (isset($index)  &&  $index >= 0  &&  $index < count($this ->data))
            ? $this ->data[$index]
            : null;
    }

    public function offsetSet($offset, $value): void
    {
        if (isset($this->formFields[$offset])) {
            $this ->formFields[$offset] ->value = $value;
            return;
        }

        $index = $this ->getIndex($offset);

        if (isset($index)  &&  $index >= 0  &&  $index < count($this ->data)) {
            $this ->data[$index] = $value;
        }
    }

    public function offsetUnset($offset): void
    {
        if (isset($this->formFields[$offset])) {
            $this ->formFields[$offset] ->value = null;
            return;
        }

        $index = $this ->getIndex($offset);

        if (isset($index)  &&  $index >= 0  &&  $index < count($this ->data)) {
            unset($this ->data[$index]);
        }
    }

    public function count(): int
    {
        return $this ->data
            ? count($this ->data)
            : 0;
    }
}


/**
 * Trida zajistuje renderovani HTML formulare, nacitani dat z databaze a zpracovani dat odeslanych uzivatelem (zapis do databaze).
 */
class InputForm implements ViewObject, ConfigurableObject
{
    const NEW_RECORD = 'new';
    const COPY_RECORD = 'copy';
    const INSERT_RECORD = 'insert';
    const EDIT_RECORD = 'edit';
    const SAVE_RECORD = 'save';

    /** @var  Database      Objekt zajistujici pristup k databazi. */
    public $database;

    /** @var ?string        Nadpis formulare */
    public $title;

    /** @var ?string        Kod formulare. Mel by byt jedinecny v ramci aplikace. Slouzi predevsim pro nacteni konfigurace. */
    public $code;

    /** @var ?string        Kod formulare - slouzi pro rozliseni vice formularu na jedne strance. */
    public $formCode;

    /** @var ?string        HTML atribut id pro vyrenderovany formular */
    public $htmlFormId = null;

    /** @var bool|string    Hlaska, ktera se zobrazi uzivateli pri odeslani formulare. Pokud je true, zobrazi se implicitni hlaska. */
    public $submitWorkingInfo = true;

    /** @var string         Jmeno SQL tabulky */
    public $sqlTableName;
    /** @var string         SELECT cast SQL dotazu pro nacteni editovanych dat. */
    public $sqlSelectPart;

    /** @var FormField[]    Pole zobrazovanych formularovych poli - @see FormField */
    public $fields;

    /** @var bool           Nastavit na true, pokud se nema kazde pole zobrazovat v novem radku tabulky */
    public $noTableRows;
    /** @var bool           Nastavit na true, pokud se nema zobrazovat tlacitko "puvodni" */
    public $noOldValuesButton;
    /** @var string         jmeno css tridy pro formular */
    public $cssClass;

    /** @var string         Implicitni css trida pro jeden radek formulare */
    public $defaultRowClass = 'form-row';
    /** @var string         Implicitni css trida pro zobrazeni nadpisu formularoveho pole */
    public $defaultCaptionColClass = 'form-col1 caption';
    /** @var string         Implicitni trida pro zobrazeni datove casti formularoveho pole */
    public $defaultDataColClass = 'form-col2';

    /** @var string         Implicitni prefix pro HTML id radku formulare. K prefixu se prida jmeno formularoveho pole, ktere se na danem radku zobrazuje. */
    public $rowIdPrefix = 'row_';

    /** @var array          Html parametry formulare (napr. id, style, data atd.) - asociativni pole nazev=>hodnota */
    public $htmlParams = [];

    /** @var string         edit,save / new,copy,insert */
    public $formAction;
    /** @var string         akce pro formular v HTML */
    public $htmlAction;
    /** @var string         pokud je nastavena, zobrazi se tlacitko Storno */
    public $stornoAction;

    public $readOnly;

    /** @var FormRowArray|null  Nactena data */
    public $data = null;

    /** @var bool           Vyrenderuje formular jako dialog (skryty atd.) */
    public $dialogMode = false;
    /** @var bool           Priznak, ze ma vstupni data konvertovat z UTF8 (pouziva se pri AJAX) */
    public $utf8 = false;

//    /** @var string         HTML obsah, ktery se vygeneruje pred formularem (lze nastavit v konfiguracnim XML) */
//    public $htmlContentBefore = '';
//    /** @var string         HTML obsah, ktery se vygeneruje za formularem (lze nastavit v konfiguracnim XML) */
//    public $htmlContentBehind = '';

    /** @var bool           Priznak, jestli se ma renderovat HTML formular (<form ....>). */
    public $renderHtmlForm = true;

    /** @var bool           Priznak, ze nebylo vyplneno nektere chybejici pole */
    public $requiredFieldMissing = false;

    /** @var ObjectConfiguration    Konfiguracni objekt - umoznuje nacist konfiguraci z XML */
    public $objectConfiguration;

    /**
     * @param Database $database
     * @param string $formAction
     * @param bool $dialogMode
     */
    public function __construct($database, $formAction, $dialogMode = false)
    {
        $this->database = $database;

        $this->dialogMode = $dialogMode;
        $this ->utf8 = $this ->dialogMode;

        //$this->cssClass = 'inputtable';
        $this->cssClass = 'form large-form form-30-70';
        $this->fields = [];

        if ($this->dialogMode) {
            $this->htmlParams['data-ronform-field-selector'] = 'input, select, textarea';
            $this->htmlParams['tabindex'] = '-1'; // OTESTOVAT s jQueryUI
            $this->htmlParams['aria-hidden'] = 'true';
        }

        $this->formCode = '';

        $this->readOnly = false;

        $post_ok = true;
        if (isset($_POST['formcode'])) {
            $post_ok = $_POST['formcode'] === $this->formCode;
        } else if ($this->formCode) {
            // jedna se o data z jineho formulare - ? ? ? muzu vzit formaction z GET?
            $post_ok = false;
        }

        if ($post_ok && isset($_POST['formaction'])) {
            $this->formAction = StringUtils::secureString($_POST['formaction']);
        } else if (!$dialogMode  &&  isset($_GET['formaction'])) {
            $this->formAction = StringUtils::secureString($_GET['formaction']);
        } else {
            $this->formAction = $formAction;
        }
    }

    /**
     * @param string|null $title
     * @return InputForm
     */
    public function setTitle(string $title): InputForm
    {
        $this->title = TextManager::_($title);
        return $this;
    }

    /**
     * @param string|null $code
     * @return InputForm
     */
    public function setCode(string $code): InputForm
    {
        $this->code = $code;
        return $this;
    }

    /**
     * @param string $cssClass
     * @return InputForm
     */
    public function setCssClass(string $cssClass): InputForm
    {
        $this->cssClass = $cssClass;
        return $this;
    }


    /**
     * @param  string|FormField $field
     * @return FormField
     */
    public function addField($field = FormField::FIELD_TYPE_EDIT)
    {
        if (!($field instanceof FormField)) {
            // array nebo string
            $field = new FormField($field);
        }

        if ($field) {
            $this->fields[] = $field;
        }

        return $field;
    }


    /**
     * Prida BrowserFields ze zadaneho pole $fields - objekty BrowserField
     * @param BrowserField[] $fields
     */
    public function addFields($fields)
    {
        foreach ($fields as $field) {
            $this->addField($field);
        }
    }


    /**
     * Vraci pole se zadanym kodem
     * @param $code
     * @return null|int
     */
    public function findFieldIndexByCode($code)
    {
        if (!$code) {
            return null;
        }

        foreach ($this->fields as $index => $field) {
            if ($field->code === $code) {
                return $index;
            }
        }

        return null;
    }


    /**
     * Najde pole se zadanym jmenem
     * @param string $name
     * @return FormField
     */
    public function findFieldByName($name)
    {
        if (!$name) {
            return null;
        }

        foreach ($this->fields as $field) {
            if ($field->fieldName === $name) {
                return $field;
            }
        }

        return null;
    }


    /**
     * Presune zadane pole ($fieldIndex) pred pole s kodem $before, nebo za pole s kodem $after
     * @param int $fieldIndex
     * @param string $before
     * @param string $after
     */
    public function moveField($fieldIndex, $before, $after)
    {
        if ($before) {
            $field2Index = $this->findFieldIndexByCode($before);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex - 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                $count = 1;
                if ($this->fields[$fieldIndex]->fieldType === FormField::FIELD_TYPE_GROUP) {
                    // presune celou skupinu - spocita, kolik polozek do teto skupiny patri (az po konec skupiny nebo dalsi zacatek skupiny)
                    while ($fieldIndex + $count < count($this->fields) && $this->fields[$fieldIndex + $count]->fieldType !== FormField::FIELD_TYPE_GROUP  &&  $this->fields[$fieldIndex + $count]->fieldType !== FormField::FIELD_TYPE_GROUP_END) {
                        $count++;
                    }
                    if ($fieldIndex + $count < count($this->fields)  && $this->fields[$fieldIndex + $count]->fieldType === FormField::FIELD_TYPE_GROUP_END) {
                        // zahrne i pole typu GROUP_END
                        $count++;
                    }
                }

                $removed = array_splice($this->fields, $fieldIndex, $count);

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index -= $count;
                }

                array_splice($this->fields, $field2Index, 0, $removed);
                return;
            }
        }

        if ($after) {
            $field2Index = $this->findFieldIndexByCode($after);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex + 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index--;
                }

                $removed = array_splice($this->fields, $fieldIndex, 1);
                array_splice($this->fields, $field2Index + 1, 0, $removed);
                return;
            }
        }
    }


    /**
     * @param string $key
     * @param ArrayAccess|string $value
     * @param string $valueText
     */
    public function loadConfigurationItem($key, $value, $valueText)
    {
        if (strpos($key, 'html:') === 0) {
            if (!$this->htmlParams) {
                $this->htmlParams = [];
            }

            $this->htmlParams[substr($key, 5)] = TextManager::_($valueText);
        } else {
            switch ($key) {
                case 'noTableRows' :
                case 'noOldValuesButton' :
                case 'readOnly' :
                    $this->{$key} = (int)$valueText;
                    break;

                case 'cssClass' :
                case 'stornoAction' :
                case 'defaultRowClass' :
                case 'defaultCaptionColClass' :
                case 'defaultDataColClass' :
                case 'htmlFormId' :
                    $this->{$key} = $valueText;
                    break;

                case 'title' :
                    $this ->{$key} = TextManager::_($valueText);
                    break;

                case 'submitWorkingInfo' :
                    if ($valueText === '0') {
                        $this->submitWorkingInfo = false;
                    } else if ($valueText === '1') {
                        $this->submitWorkingInfo = true;
                    } else {
                        $this->submitWorkingInfo = $valueText;
                    }
                    break;

                case 'field' :
                    $this ->loadFieldsConfiguration($value);
                    break;
            }
        }
    }


    /**
     * Vytvori nove pole pro zadanou konfiguraci.
     * @param $fieldSettings
     * @return FormField|null
     */
    protected function createNewFieldFromConfiguration($fieldSettings)
    {
        if (isset($fieldSettings['fieldType'])) {
            return new FormField($fieldSettings['fieldType']);
        }

        return null;
    }


    /**
     * Nastavi vsechna pole ze zadane skupiny
     * @param string $groupFieldCode
     * @param array $fieldSettings
     * @throws Exception
     */
    public function loadGroupFieldsConfiguration($groupFieldCode, $fieldSettings)
    {
        $groupFieldIndex = $this ->findFieldIndexByCode($groupFieldCode);
        if (!isset($groupFieldIndex)) {
            return;
        }

        $fieldIndex = $groupFieldIndex + 1;
        while ($fieldIndex < count($this ->fields)  &&  $this ->fields[$fieldIndex] ->fieldType != FormField::FIELD_TYPE_GROUP  &&  $this ->fields[$fieldIndex] ->fieldType != FormField::FIELD_TYPE_GROUP_END) {
            $this ->fields[$fieldIndex] ->loadConfigurationArray($fieldSettings);
            $fieldIndex++;
        }
    }


    /**
     * Nastavi pole podle zadaneho nastaveni
     * @param array $fieldSettings
     * @throws Exception
     */
    public function loadFieldConfiguration($fieldSettings)
    {
        $field = null;

        if (isset($fieldSettings['context'])) {
            if ($fieldSettings['context'] === 'insert'  &&  !$this ->isInsert()) {
                // konfigurace se ma uplatnit jen pri insert
                return;
            }

            if ($fieldSettings['context'] === 'update'  &&  $this ->isInsert()) {
                // konfigurace se ma uplatnit jen pri update
                return;
            }
        }

        if (strpos($fieldSettings['code'] ?? '', '*') !== false  ||  strpos($fieldSettings['code'] ?? '', '?') !== false) {
            // nazev pole obsahuje znak * nebo ? - aplikuje se na vsechna pole odpovidajici vzoru
            foreach ($this ->fields as $field) {
                if ($fieldSettings['code'] === '*'  ||  fnmatch($fieldSettings['code'], $field->code)) {
                    $field ->loadConfigurationArray($fieldSettings);
                }
            }

            return;
        }

        $fieldIndex = $this ->findFieldIndexByCode($fieldSettings['code']);



        if (isset($fieldIndex)) {
            $field = $this->fields[$fieldIndex];
        } else if ($fieldSettings['group']) {
            $this ->loadGroupFieldsConfiguration($fieldSettings['group'], $fieldSettings);
            return;
        } else {
            // prida nove pole
            $field = $this ->createNewFieldFromConfiguration($fieldSettings);
            if ($field) {
                $field ->code = $fieldSettings['code'];
                $fieldIndex = count($this->fields);
                $this->fields[] = $field;
            }
        }

        if ($field) {
            $field ->loadConfigurationArray($fieldSettings);
            if ($fieldSettings['before'] || $fieldSettings['after']) {
                $this->moveField($fieldIndex, $fieldSettings['before'], $fieldSettings['after']);
            }
        }
    }


    /**
     * Nacte konfiguraci poli ze zadaneho XML-pole.
     * @param ArrayAccess $fieldsConfArray
     */
    public function loadFieldsConfiguration($fieldsConfArray)
    {
        if (!$fieldsConfArray) {
            return;
        }

        foreach ($fieldsConfArray as $fieldSettings) {
            $this ->loadFieldConfiguration($fieldSettings);
        }
    }


//    /**
//     * Nacte konfiguraci ze zadaneho pole nacteneho pomoci XmlLoader
//     * @param $array
//     */
//    public function loadConfigurationArray($array)
//    {
//        if (!$array) {
//            return;
//        }
//
//        foreach ($array as $key => $value) {
//            if ($key === 'html') {
//                foreach ($value as $htmlItem) {
//                    $content = XmlLoader::rawDataToString($htmlItem);
//
//                    if ($content) {
//                        if (isset($htmlItem['#attr']['position'])  &&  $htmlItem['#attr']['position'] === 'behind') {
//                            $this ->htmlContentBehind .= $content;
//                        } else {
//                            $this ->htmlContentBefore .= $content;
//                        }
//                    };
//                }
//            } else {
//
//                if ($value instanceof ArrayAccess) {
//                    // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu (napr. style)
//                    $valueText = '';
//                    foreach ($value as $valA) {
//                        if (isset($valA['#text'])) {
//                            $valueText .= ($valueText ? "\n" : '') . $valA['#text'];
//                        }
//                    }
//                } else {
//                    $valueText = $value;
//                }
//
//                $this->loadConfigurationItem($key, $value, $valueText);
//            }
//        }
//
//        if (isset($array['field'])) {
//            $this ->loadFieldsConfiguration($array['field']);
//        }
//    }

    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        $this ->objectConfiguration = new ObjectConfiguration($this);
        $this ->objectConfiguration ->loadConfigurationXml($xmlText);
    }


    /**
     * volano pri zobrazeni formulare pro editaci - pokud funkce vrati false
     * zobrazi se jen readonly verze formulare
     * @param FormRowArray $dbrow pole nactenych dat z databaze
     * @return bool
     */
    public function canEditRecord($dbrow)
    {
        if ($this->readOnly) {
            return false;
        }

        return true;
    }



    /**
     * Zkontroluje, jestli jsou zadana vsechna povinna pole. Chybove hlasky vlozi do MessageManageru.
     * Vraci true, pokud nechybi zadne pole s required=true
     * @param bool $edit    Urcuje, jestli se jedna o editaci zaznamu (true), nebo o vlozeni noveho (false)
     * @return bool
     */
    public function checkRequiredFields($edit)
    {
        $result = true;
        $this ->requiredFieldMissing = false;

        foreach ($this->fields as $field) {
            if (!$field->checkRequired($edit)) {
                $this ->requiredFieldMissing = true;

                if ($field ->required === 2) { // hodnota 2 znamena, ze pole nelze editovat ani pri vkladani
                    MessageManager::getInstance()->addWarning(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                } else {
                    MessageManager::getInstance()->addError(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                    $result = false;
                }
            }
        }

        return $result;
    }


    /**
     * @return string|SqlStatement
     */
    public function getSqlFromPart()
    {
        return $this ->sqlTableName;
    }


    public function getRecordPrimaryKeyValue()
    {
        // POZOR! Muse se pouzivat i pri vkladani - pro formaction = "copy"
        if (isset($_POST['record'])) {
            return $_POST['record'];
        }

        if (isset($_GET['record'])) {
            return $_GET['record'];
        }

        $primaryField = null;

        foreach ($this ->fields as $field) {
            if ($field ->primary) {
                if ($primaryField) {
                    // uz existuje nejake jine primarni pole => je jich vice => nelze pouzit jedno pole pro identifikaci zaznamu
                    return null;
                }
            }
        }

        return ($primaryField  &&  isset($_POST[$primaryField ->fieldName]) &&  $_POST[$primaryField ->fieldName] != '')
            ? $_POST[$primaryField ->fieldName]
            : null;
    }

    /**
     * @return string|SqlStatement
     * @throws Exception
     */
    public function getWhereCondition()
    {
        $record = $this ->getRecordPrimaryKeyValue();

        // najde primarni pole a urci podle nej podminku
        foreach ($this->fields as $field) {
            if ($field->primary) {
                if ($field ->dataType === 'int') {
                    $record = (int)$record;
                }

                //return $field->sqlFieldName . ' = ' . FormField::fieldValueToSQLConst($this->database, $field->fieldType, $record);
                $params = $this ->database ->createParameters();
                $tbl = $field ->tableAlias ?: $field ->tableName;

                return new SqlStatement(
                    ($tbl ? $tbl . '.' : '') . $field->sqlFieldName . ' = ' . $params ->addParameter($record),
                    $params);
            }
        }

        return '';
    }


    public function getSqlSelectPart()
    {
        // pokud ma formular preddefinovanou selectPart, tak ji da na zacatek ...
        $fieldsStr = $this->sqlSelectPart ?: '';

//        if ($this->sqlSelectPart) {
//            $fieldsStr = $this->sqlSelectPart;
//        } else {
//
//            $fieldsStr = '';

        // ... doplni dalsi pole
        // Pokud neni select part, tak je cisluje od 0
        // Pokud uz je nejaka select part, tak je cisluje od -1 a pokracuje dolu (-2 atd.) - tj. $increment=-1,
        // protoze nevim, kolik poli jiz je nadefinovano
        $increment = $fieldsStr ? -1 : 1;
        $dbFieldIndex = $fieldsStr ? -1 : 0;

        foreach ($this->fields as $field) {
            // v pripade, ze ma formular zadnou sqlSelectPart, tak vynecha pole s kladnym indexem (mela by to by pole, ktera jsou v preddefinovane casti dotazu)
            if ($field ->visible  &&  $field->sqlFieldName  &&  (!isset($field ->sqlFieldIndex)  ||  !($this->sqlSelectPart))) {
                $tableName = $field ->tableAlias ?: $field ->tableName;
                $sqlName = ($tableName ? $tableName . '.' : '') . $field->sqlFieldName;

                $fieldName = ($field->fieldType === FormField::FIELD_TYPE_MEMO)
                    ? $this->database->castWideMemo($sqlName)
                    : $sqlName;

                $field->sqlFieldIndex = $dbFieldIndex;
                $dbFieldIndex += $increment;

                $fieldsStr .= ($fieldsStr ? ', ' : '') . $fieldName;
            }
        }

        // a nakonec otoci poradi pridanych poli (aby to posledni melo index -1)
        foreach ($this ->fields as $field) {
            if ($field ->sqlFieldIndex < 0) {
                $field ->sqlFieldIndex = $dbFieldIndex - $field ->sqlFieldIndex;
            }
        }

        return $fieldsStr;
    }

    /**
     * Sestavi dotaz pro nacteni zaznamu z databaze
     * @return string|SqlStatement
     * @throws Exception
     */
    public function buildSelectSQL()
    {
        $fieldsStr = $this ->getSqlSelectPart();

        $where = $this->getWhereCondition();
        $fromPart = $this ->getSqlFromPart();

        if (!$fromPart  ||  !$fieldsStr) {
            return null;
        }

        $params = null;
        if ($fromPart instanceof SqlStatement) {
            if ($fromPart ->parameters) {
                $params = $this ->database ->createParameters();
                $params ->appendParameters($fromPart ->parameters);
            }
            $fromStr = $fromPart ->sqlText;
        } else {
            $fromStr = $fromPart;
        }

        if ($where instanceof SqlStatement) {
            if ($params) {
                $params ->appendParameters($where ->parameters);
            } else {
                $params = $where ->parameters;
            }
            return new SqlStatement(
                "SELECT $fieldsStr\n FROM $fromStr\n WHERE " . $where ->sqlText,
                $params);
        }

        // $where je jen string
        $str =
            "SELECT $fieldsStr FROM $fromStr\n" .
            ' WHERE ' . $where;

        return $params
            ? new SqlStatement($str, $params)
            : $str;
    }


    /**
     * Nacte data z databaze.
     * @return array|null
     * @throws Exception
     */
    public function loadRecord()
    {
        // Nacte aktualni hodnoty poli z databaze
        $sql = $this->buildSelectSQL();

        if (!$sql) {
            return null; // neni zadana nektera cast SQL dotazu => nic nenacita
            // throw new Exception('Neni zadan SQL vyraz pro nacteni dat z databaze!');
        }

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->select($sql);
        $result = $cursor->fetch();
        $cursor ->close();

        return $result;
    }


    public function loadData($usePostValues)
    {
        $row = $this ->loadRecord();

        $this ->data = isset($row)
            ? new FormRowArray($this ->fields, $row)
            : null;

        $insert = $this ->isInsert();

        // a pak je pripadne preplacne podle POST
        $this ->loadFieldValues($insert,
            $this ->data, $usePostValues,
            $insert || $this->canEditRecord($this->data));
    }


    /**
     * vraci true, pokud se jedna o vkladani noveho zaznamu
     * @return bool
     */
    public function isInsert(): bool
    {
        return
            $this->formAction === 'new'
            || $this->formAction === 'copy'
            || $this->formAction === 'insert';
    }


    protected function renderPrimaryFields()
    {
        if ($this ->isInsert()) {
            // vkladani (i pro akci "copy")
            return;
        }

        $record = $this ->getRecordPrimaryKeyValue();

        if ($record) {
            // id editovaneho zaznamu
            echo '<input type="hidden" name="record" value="' . StringUtils::htmlTextConvert($record) . '"/>';
        }
    }


    /**
     * Vraci id requestu pro CSRF ochranu
     * @return string
     */
    public function getCsrfRequestId()
    {
        return
            ($this ->code ?? '') .
            (!$this ->dialogMode && $this ->isInsert() ? '' : '_' . $this ->getRecordPrimaryKeyValue());
    }


    /**
     * Vlozi zakladni pole (formaction). Potomek muze pridat svoje vlastni dodatecna pole.
     */
    public function renderCommonFields()
    {
        $formAction = $this->isInsert() ? 'insert' : 'save';

        echo '<input type="hidden" name="formaction" class="ronform-field" value="' . $formAction . '"/>';

        if ($this ->renderHtmlForm) {
            // pokud se nerenderuje formulare (typicky v pripade, ze je formular soucasti nejakeho nadrazeneho formulare - napr. XML),
            // tak nepouziva CSRF ochranu, protoze by mela byt soucasti toho nadrazeneho
            echo Csrf::csrfHiddenInput($this->getCsrfRequestId());
        }

        if ($this->formCode) {
            echo '<input type="hidden" name="formcode" value="' . $this->formCode . '"/>';
        }

        $this ->renderPrimaryFields();
    }


//    /**
//     * Zobrazi formular pro vkladani. $action je akce tlacitka pro vlozeni zaznamu.
//     * @param bool $postvalues Urcuje, jestli se maji zobrazit hodnoty z $_POST (true), nebo defaultni hodnoty
//     * @return bool
//     */
//    function showFieldsInsert($postvalues)
//    {
//        foreach ($this->fields as $i => $field) {
//            if ($field ->visible) {
//                $field->showFieldInsert($this->database, $postvalues, !$this->noTableRows);
//            }
//        }
//    }


//    function showFieldsEdit($values, $postvalues, $caneditrecord)
//    {
//        foreach ($this->fields as $i => $field) {
//            if ($field ->visible) {
//                if ($postvalues && $caneditrecord && $field->editable) {
//                    $value = $field->getPostValueText();
//                } else {
//                    // Standardni zobrazeni hodnot z databaze
//                    $value = (isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
//                        ? $field->getValueFromDBValue($this->database, $values[$field->sqlFieldIndex])
//                        : $field->defaultValue;
//                }
//
//
//                if ($caneditrecord) {
//                    $field->showFieldEdit($this->database, $value, !$this->noTableRows, $values);
//                } else {
//                    $field->showFieldReadOnly($this->database, $value, !$this->noTableRows, $values);
//                }
//            }
//        }
//    }

    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
        foreach ($this->fields as $field) {
            if ($field->visible) {
                // pri insert neni nastaveno $values
                // ale pri kopirovani nastaveno je - v tom pripade ale nekopiruje primarni pole
                if (!$values) {
                    $field ->loadInsertValue($this ->database, $usePostValues, null);
                } else {
                    $field ->loadEditValue($this->database, $usePostValues, $values, $canEditRecord);
                }
            }
        }
    }


    /**
     * @throws DBException
     */
    public function renderFields($insert, $values, $postValues, $canEditRecord)
    {
        //$this ->loadFieldValues($insert, $values, $postValues, $canEditRecord);

        $isRow = false;
        $isFieldGroup = false;

        foreach ($this->fields as $field) {
            if ($field ->visible) {
                if ($field ->fieldType === FormField::FIELD_TYPE_GROUP  ||  $field ->fieldType === FormField::FIELD_TYPE_GROUP_END) {
                    // konec skupiny nebo zacatek nove skupiny
                    if ($isRow) {
                        echo "</div>\n";  // konec predchoziho radku
                        $isRow = false;
                    }

                    if ($isFieldGroup) {
                        echo "</div>\n"; // konec predchozi skupiny
                        $isFieldGroup = false;
                    }
                }

                if ($field ->fieldType === FormField::FIELD_TYPE_GROUP) {
                    $attrs = GuiUtils::getAttributesHtml($field ->getFieldAttributes());

                    // zacatek nove skupiny
                    echo "\n<div $attrs>\n";

                    $isFieldGroup = true;
                }

                if (!$this->noTableRows  &&  $field ->newRow) {
                    // pole je zacatek radku (nebo ma vlastni radek)

                    if ($isRow) {
                        echo "</div>\n";  // konec predchoziho radku
                    }

                    $rowId = $field ->rowId
                        ?? ( $field ->fieldName ? $this ->rowIdPrefix . str_replace('#', '_', $field ->fieldName) : '');

                    $className = $field ->rowClass ?: $this ->defaultRowClass;

                    // zacatek noveho radku
                    echo "\n<div" . ($rowId ? " id='$rowId'" : '') . ($className ? " class='{$className}'" : '') . '>';

                    $isRow = true;
                }

                if ($field ->showColumns) {
                    if ($field->caption !== false) {
                        // caption
                        //$captionClass = $field ->captionColumnClass ?: $this ->defaultCaptionColClass . ($field->required ? ' required' : '');
                        $attrsStr = GuiUtils::getAttributesHtml($field->getCaptionAttributes($this->defaultCaptionColClass));
                        echo "<div $attrsStr>{$field->caption}</div>";
                    }

                    $attrsStr = GuiUtils::getAttributesHtml($field ->getDataColAttributes($this ->defaultDataColClass));
                    echo "<div $attrsStr>";
                }

                if ($insert) {
                    //$field ->loadInsertValue($this ->database, $postValues);
                    $field ->showField($this ->database, $field ->value, !$this ->readOnly  &&  $field ->readOnly !== 2, true, $values);
                    //$field->showFieldInsert($this->database, $postValues);
                } else {
                    //$field ->loadEditValue($this ->database, $postValues, $values, $canEditRecord);
//                    if ($postValues && $canEditRecord && !$field->readOnly) {
//                        $value = $field->getPostValueText();
//                    } else {
//                        // Standardni zobrazeni hodnot z databaze
//                        $value = (isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
//                            ? $field->getValueFromDBValue($this->database, $values[$field->sqlFieldIndex])
//                            : $field->defaultValue;
//                    }


                    if ($canEditRecord) {
                        $canEdit = !$field ->readOnly && !$field->primary;     // $this->editable pro kompatibilitu se starym kodem
                        if (!$field ->changeable  &&  $field ->value) {
                            $canEdit = false;
                        }
                        //$field->showFieldEdit($this->database, $value, $values);
                    } else {
                        $canEdit = false;
                        //$field->showFieldReadOnly($this->database, $value, $values);
                        //$field->showField($this ->database, $field ->value, false, false, $values); // canedit = false, insert = false
                    }

                    $field->showField($this ->database, $field ->value, $canEdit, false, $values); // insert = false
                }

                if ($field ->showColumns) {
                    // konec col-2 (sloupec s inputem)
                    echo '</div>';
                }
            }
        }

        if ($isRow) {
            echo "</div>\n"; // konec posledniho radku
        }

        if ($isFieldGroup) {
            echo "\n</div>\n"; // konec posledni skupiny
        }
    }


    /**
     * Vraci seznam tlacitek, ktera se maji vygenerovat do formulare
     * @param $insert
     * @return array
     */
    public function getButtons($insert)
    {
        $result = [];

        if (!$this ->readOnly  ||  !$this ->stornoAction) {
            $result['submit'] = [
                'type' => 'submit',
                'class' => 'submitbutton' . ($this->dialogMode ? ' ronform-submit' : ''),
                'value' => $this->dialogMode ? TextManager::_('OK') : TextManager::_($insert ? 'form.Insert' : 'form.Store')
            ];
        }

        if (!$this ->readOnly  &&  !$insert && !$this->noOldValuesButton) {
            $result['original'] = [
                'type' => 'reset',
                'class' => 'button',
                'value' => TextManager::_('form.Original')
            ];
        }

        if ($this->stornoAction) {
            $classStr =
                'button' .
                ($this ->stornoAction === '#'
                    ? ' cancel'
                    : ' clickHref');

            $result['cancel'] = [
                'type' => 'button',
                'class' => $classStr,
                'value' => TextManager::_('Cancel'),
                'data-ron-href' => $this->stornoAction
            ];
        }

        return $result;
    }

    /**
     * Zobrazeni tlacitek
     * @param bool $insert
     */
    public function renderButtons($insert)
    {
        $buttons = $this ->getButtons($insert);
        if ($buttons) {
            echo '<div class="buttons">';
            foreach ($buttons as $button) {
                echo '<span class="margin-right-2">';

                echo '<input';
                foreach ($button as $attrName => $attrValue) {
                    echo " $attrName='$attrValue'";
                }
                echo '/>';

                echo '</span>';
            }
            echo '</div>';
        }
    }


    /**
     * Zobrazi formularova pole
     * @throws Exception
     */
    public function renderContent()
    {
        $insert = $this ->isInsert();

        if (!$insert) {
            if (!isset($this->data)) {
                throw new Exception('Chyba! Pozadovany zaznam nebyl nalezen!');
            } else if (!$this->data) {
                // napriklad chyba v SQL dotazu
                throw new Exception('Chyba ve formulari (funkce renderFormFields) - nelze nacist hodnoty z databaze!');
            }
        }
        /*
        if ($this->isInsert()) {
            // insert
            $this ->showFieldsInsert($this->formAction == 'insert'); // pri "insert" pouzije POST
        } else {

            $canEditRecord = $this->canEditRecord($this->data);

            // edit
            $this->showFieldsEdit($this->data,
                $this->formAction == 'save',    // pri "save" pouzije postvalues
                $canEditRecord);
        }
        */

        echo '<div class="form-content">';

        $this ->renderFields($insert,
            //new FormRowArray($this ->fields, $this ->data),
            $this ->data ?? new FormRowArray($this ->fields, null),  // SE 5.3.2023 - data uz musi byt instance FormRowArray!
            $this->formAction === 'insert' || $this->formAction === 'save',
            $insert || $this->canEditRecord($this->data)
            );

        echo '</div>';
    }


    public function renderHeader()
    {
        if ($this ->dialogMode) {
            return;
        }

        if ($this->title) {
            echo "<h1>{$this->title}</h1>\n";
        }
    }


    public function getHtmlFormParams()
    {
        $invisibleFields = '';
        foreach ($this ->fields as $field) {
            if ($field ->code  &&  !$field ->isVisible()) {
                $invisibleFields .= ($invisibleFields ? ' ' : '') . $field ->code;
            }
        }

        return
            ($this ->htmlFormId
                ? " id='{$this ->htmlFormId}'"
                : '' ) .
            ($this ->code
                ? " data-code='{$this ->code}'"
                : '') .
            ($this ->htmlAction
                ? " action='{$this->htmlAction}'"
                : '') .
            ($this ->submitWorkingInfo
                ? " class='submitWorkingInfo'"
                : '') .
            ($this ->submitWorkingInfo  &&  $this ->submitWorkingInfo !== true
                ? " data-ron-working-info='" . StringUtils::htmlTextConvert($this ->submitWorkingInfo) . "'"
                : '') .
            ($this ->sqlTableName  &&  strlen($this ->sqlTableName) < 30
                ? " data-table-name='" . StringUtils::htmlTextConvert($this ->sqlTableName) . "'"
                : '') .
            ($invisibleFields
                ? " data-invisible-fields='$invisibleFields'"
                : '')
            ;
    }


    public function getCssClass()
    {
        $result =
            ($this ->cssClass ?: '') .
            ($this ->dialogMode ? ' ronform rondialog modal' : ''); // modal pro bootstrap

        $usePost = $this->formAction === 'insert' || $this->formAction === 'save';

        // prida class podle poli, ktera maji nastaveno "appendFormClass"
        foreach ($this ->fields as $field) {
            if ($field ->appendFormClass) {
                if ($usePost) {
                    if ($field ->fieldName && isset($_POST[$field ->fieldName])  &&  $_POST[$field ->fieldName]) {
                        $result .= ' ' . $field ->appendFormClass;
                    }
                } else {
                    if ($this ->data  &&  $field ->sqlFieldIndex  &&  isset($this ->data[$field ->sqlFieldIndex])  &&  $this ->data[$field ->sqlFieldIndex]) {
                        $result .= ' ' . $field ->appendFormClass;
                    } else if ($this ->isInsert()  &&  $field ->defaultValue) {
                        $result .= ' ' . $field ->appendFormClass;
                    }
                }
            }
        }

        return $result;
    }


    public function renderMessages()
    {
        if (!$this ->dialogMode) {
            MessageManager::getInstance()->showMessages();
        }
    }


    /**
     * Zobrazi formular
     * @param string $formAction
     * @throws Exception
     */
    public function renderForm($formAction = '')
    {
        if ($formAction) {
            $this->formAction = $formAction;
        }

        $this->renderHeader(); // header pred <FORM> - muze obsahovat jiny formular!

        if ($this ->objectConfiguration) {
            echo $this->objectConfiguration->getHtmlContent(['top', '']);
        }
//        if (!$this ->dialogMode  &&  $this ->htmlContentBefore) {
//            echo $this ->htmlContentBefore;
//        }

        $this->renderMessages();

        echo '<div';
        $cssClass = $this ->getCssClass();
        if ($cssClass) {
            echo ' class="' . $cssClass . '"';
        }

        if ($this ->title) {
            echo ' data-title="' . StringUtils::htmlTextConvert($this ->title) . '"'; // jQueryUI
        }

        $params = $this ->dialogMode
            ? array_merge(['title' => $this ->title], $this ->htmlParams ?? [])
            : $this ->htmlParams;

        if ($params) {
            echo ' ' . GuiUtils::getAttributesHtml($params);
        }
        echo '>';

        if ($this ->renderHtmlForm) {
            echo
                "\n<form enctype='multipart/form-data' method='post' " . $this->getHtmlFormParams() . ">\n";
        }

        if ($GLOBALS['test-bootstrap'] ?? false) {
            echo
                '<div class="modal-dialog">' .
                '<div class="modal-content">' .
                '<div class="modal-header">';

            if ($this ->title) {
                echo '<h5 class="modal-title">' . $this ->title . '</h5>';
            }

//            if ($this ->showCloseButton) {
//                echo '<button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>';
//            }

            echo "</div>\n"; // modal-header

            echo
                '<div class="modal-body">' .
                "\n";
        }

        $this->renderCommonFields();
        $this->renderContent();

        if ($GLOBALS['test-bootstrap'] ?? false) {
            echo "</div>\n"; // modal-body
        }

        $this->renderButtons($this ->isInsert());

        if ($GLOBALS['test-bootstrap'] ?? false) {
            echo
                "</div>\n" . // modal-content
                "</div>\n";  // modal-dialog
        }

        if ($this ->renderHtmlForm) {
            echo "\n</form>\n";
        }

        echo '</div>';

//        if (!$this ->dialogMode  &&   $this ->htmlContentBehind) {
//            echo $this ->htmlContentBehind;
//        }
        if ($this ->objectConfiguration) {
            echo $this->objectConfiguration->getHtmlContent(['behind', 'bottom', 'dole']);
        }
    }


    /**
     * Nacte data pro javascriptove formulare
     * @return array
     * @throws Exception
     */
    public function getJsonData()
    {
        $this ->loadData(false);

        //$data = new FormRowArray($this ->fields, $this ->loadRecord());
        //$data = $this ->loadRecord();
        $result = [];

        foreach ($this ->fields as $field) {
            if ($field ->visible  &&  $field->fieldName) {
                $value = $field ->loadJsonValue($this ->database, $this ->data);

//                $value = ($data && isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
//                    ? $field->getValueFromDBValue($this->database, $data[$field->sqlFieldIndex])
//                    : $field->defaultValue;

                if ($field ->fieldType === FormField::FIELD_TYPE_DATEANDTIME) {
                    $result[$field->fieldName] = $value
                        ? $value[0]
                        : null;
                    $result[$field->fieldName . '_time'] = $value
                        ? $value[1]
                        : null;
                } else {
                    $result[$field->fieldName] = StringUtils::jsonText($value);
                }
            }
        }

        return $result;
    }


    /**
     * @param string $tableName
     * @return DBRecord
     */
    public function createDBRecord($tableName = null)
    {
        return new DBRecord($this->database, $tableName ?: $this->sqlTableName);
    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function beforeInsertRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function afterInsertRecord($rec)
    {
        return true;
    }


    /**
     * Vytvori pole pro zaznam zadane tabulky - pridava jen pole, ktera nemaji nastavene jmeno tabulky, nebo patri k zadane tabulce
     * @param DBRecord $rec
     * @param string $tableName
     * @param bool $insert
     * @param bool $checkRequiredFields     Urcuje, jestli se ma kontrolovat, jestli je zadana hodnota u poli, ktere maji nastaveno requiredInDatabase
     * @throws RonException
     */
    public function createDBRecordFieldsForTable($rec, $tableName, $insert, $checkRequiredFields = true)
    {
        foreach ($this->fields as $field) {
            $fieldTableName = $field ->tableAlias ?: $field ->tableName;

            if ($field ->visible  &&  (!$fieldTableName  ||  $fieldTableName === $tableName)) {
                $rec_field = $field->createDBRecordField($this->database, $tableName, $insert, $this ->utf8);
                if ($rec_field) {
                    if (is_array($rec_field)) {
                        foreach ($rec_field as $rec_f) {
                            if ($checkRequiredFields && $field ->requiredInDatabase && !isset($rec_f ->value)) {
                                throw new RonException(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                            }
                            $rec ->addDBRecordField($rec_f);
                        }
                    } else {
                        if ($checkRequiredFields && $field ->requiredInDatabase && !isset($rec_field ->value)) {
                            throw new RonException(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                        }
                        $rec->addDBRecordField($rec_field);
                    }
                }
            }
        }
    }

    /**
     * @param DBRecord $rec
     * @param bool $insert
     * @throws DBException
     */
    public function createDBRecordFields($rec, $insert)
    {
        $this ->createDBRecordFieldsForTable($rec, $this ->sqlTableName, $insert);
    }


    /**
     * @return bool
     * @throws DBException
     */
    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->createDBRecordFields($rec, true);

        if (!$this->beforeInsertRecord($rec)) {
            return false;
        }

        $rec->insertRecord();

        return $this->afterInsertRecord($rec);
    }


//    /**
//     * @return bool
//     */
//    function insert()
//    {
//        if (!$this->checkRequiredFields(false)) {
//            return false;
//        }
//
//        $rec = $this->createDBRecord();
//        $this ->createDBRecordFields($rec, true);
//
//        if (!$this->beforeInsertRecord($rec)) {
//            return false;
//        }
//
//        if (!$rec->insertRecord()) {
//            return false;
//        }
//
//        return $this->afterInsertRecord($rec);
//    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function beforeUpdateRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function afterUpdateRecord($rec)
    {
        return true;
    }


    /**
     * @param DBRecord $rec
     */
    public function setDBRecordWhere($rec)
    {
        $params = $this ->database ->createParameters();
        $whereStr = '';

        foreach ($this->fields as $field) {
            if ($field->primary  &&  $field ->sqlFieldName) {
                $rec_field = $rec ->findFieldByName($field ->sqlFieldName);
                if ($rec_field) {
                    if ($whereStr) {
                        $whereStr .= ' AND ';
                    }
                    $whereStr .= $field->sqlFieldName . ' = ' . $params->addParameter($rec_field->getSQLParam());
                }
            }
        }

        $rec->setWhere($whereStr, $params);
    }

    /**
     * @param DBRecord $rec
     * @throws DBException
     */
    public function setDBRecord($rec)
    {
        /*
        $params = $this ->database ->createParameters();

        $recordId = $this ->getRecordPrimaryKeyValue();

        $this ->createDBRecordFields($rec, false);
        foreach ($this->fields as $field) {
            if ($field->primary  &&  $field ->sqlFieldName) {
                $rec_field = $rec ->findFieldByName($field ->sqlFieldName);
                if ($rec_field) {
                    $rec ->setWhere(
                        $field->sqlFieldName . ' = ' . $params->addParameter($recordId),
                        $params);

                    return;
                }
            }
        }
        */
        $this ->createDBRecordFields($rec, false);
        $this ->setDBRecordWhere($rec);
    }


    /**
     * Provede kontrolu spravnosti dat odeslanych uzivatelem. V pripade chyby vrati false nebo vyhodi vyjimku.
     * @param bool $edit    Urcuje, jestli se jedna o editaci zaznamu (true), nebo o vlozeni noveho (false)
     * @return bool
     * @throws \Exception
     */
    public function validate($edit)
    {
        if ($this ->renderHtmlForm) { // pokud nema svuj vlastni HTML form, tak neresi CSRF
            Csrf::validateCSRFPostParam($this->getCsrfRequestId());
        }

        return $this->checkRequiredFields($edit);
    }


    /**
     * @return bool|int
     * @throws Exception
     */
    public function editRecord()
    {
        if (!$this ->validate(true)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->setDBRecord($rec);

        if (!$rec->loadRecord()) {
            throw new Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
        }

        if (!$this->beforeUpdateRecord($rec)) {
            return false;
        }

        if (!$rec->updateRecord()) {
            return false;
        }

        return $this->afterUpdateRecord($rec);
    }


    /**
     * @return bool|int
     * @throws Exception
     */
    public function handleAction()
    {
        $loadRecord = false;
        $usePostValues = false;

        try {
            switch ($this->formAction) {
                case self::COPY_RECORD :
                    $loadRecord = true;
                    $result = false;
                    break;

                case self::EDIT_RECORD :
                    $loadRecord = true;
                    $result = false;    // zobrazit formular
                    break;

                case self::NEW_RECORD :
                    $result = false;    // zobrazit formular
                    break;

                case self::INSERT_RECORD :
                    $usePostValues = true;
                    if ($this->insertRecord()) {
                        return true;
                    }
                    $result = false;
                    break;

                case self::SAVE_RECORD :
                    $loadRecord = true;
                    $usePostValues = true;
                    $result = $this->editRecord();
                    break;

                default :
                    throw new Exception('Chyba v metode Form::handleAction! Neznama akce!');
            }
        } catch (DBException $e) {
            if ($e ->isPrimaryKeyViolation()) {
                if ($this ->dialogMode) {
                    throw new Exception(TextManager::_('Zaznam_s_kodem_uz_existuje'));
                } else {
                    MessageManager::addErrorMessage(TextManager::_('Zaznam_s_kodem_uz_existuje'));
                }
                $result = false;
            } else {
                throw $e;
            }
        } catch (RonException $e) {
            if ($this ->dialogMode) {
                throw $e;
            }
            MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
            $result = false;
        } catch (Exception $e) {
            if ($this ->dialogMode) {
                throw $e;
            }
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
        }

        if (!$result  &&  $loadRecord) {
            $this->loadData($usePostValues);
        }

        if (!$result  ||  $this ->requiredFieldMissing) {
            // nepodarilo se zapsat zaznam a nebo ma nektere pole required=2 a neni zadana jeho hodnota
            // (napriklad v nastupnim formulari)
            $insert = $this ->isInsert();
            $this ->loadFieldValues($insert,
                $this ->data, $usePostValues,
                $insert || $this->canEditRecord($this->data));
        }

        return $result;
    }


    /**
     * @throws Exception
     */
    public function render()
    {
        $this ->renderForm();
    }

}
