<?php
/**
 * stanislav.elbl, 28.06.2022 10:46
 */

namespace Ron\Http;

use Ron\GuiUtils;
use Ron\TextManager;

class Csrf
{
    const CSRF_RANDOM_LEN = 8;


    private static function getCsrfToken()
    {
        // pri prihlaseni vygeneruje nahodny csrf token platny po celou session
        if (!isset($_SESSION['csrf_token'])  ||  !$_SESSION['csrf_token']) {
            $_SESSION['csrf_token'] = bin2hex(random_bytes(16));
        }

        return $_SESSION['csrf_token'];
    }

    /**
     * Vygeneruje CSRF klic pro zadany request. Tento klic pak lze pouzit pro kontrolu metodou validateCSRFKey
     * @return string   Id requestu - napriklad identifikace editovaneho zaznamu. Pokud je zadan, tak csrf klic bude platny jen pro tento request.
     */
    public static function getCsrfKey($requestId = '')
    {
        if (!($GLOBALS['csrf'] ?? true)) {
            return '';
        }

        $randomStr = bin2hex(random_bytes(self::CSRF_RANDOM_LEN));
        return $randomStr . hash_hmac('sha1', $randomStr . $requestId, self::getCsrfToken());
        //return sha1($requestString. $CSRF_key . $_SESSION['csrf_token']);
    }


    /**
     * Vygeneruje CSRF klic pro zadany request (viz metoda csrfKeyForRequest) a vraci url parametr s timto klicem
     * @param string $requestId
     * @param string $keyParameterName
     * @return string
     * @deprecated  Nepouzivejte GET pro requesty, ktere vyzaduji CSRF! Misto odkazu lze pouzit GuiUtils::postLink
     */
    public static function csrfUrlParameter($requestId = '', $keyParameterName = 'csrfkey')
    {
        $key = self::getCsrfKey($requestId);
        if ($key) {
            return '&' . $keyParameterName . '=' . $key;
        }

        return '';
    }


    public static function csrfPostAttribute($requestId = '', $keyParameterName = 'csrfkey')
    {
        $key = self::getCsrfKey($requestId);
        return $key
            ? " data-post-$keyParameterName='" . $key . "'"
            : '';
    }


    public static function csrfHiddenInput($requestId = '', $htmlParams = [])
    {
        $key = self::getCsrfKey($requestId);
        if ($key) {
            return "<input " . GuiUtils::getAttributesHtml(array_merge(
                ['type' => 'hidden', 'name' => 'csrfkey', 'data-requestid' => $requestId],
                $htmlParams
            )) . " value='$key'/>";
        }

        return '';
    }


    /**
     * Vygeneruje CSRF klic pro zadany request (viz metoda csrfKeyForRequest) a vraci atribut data-csrfkey
     * @param string $requestId
     * @param string $keyParameterName
     * @return string
     */
    public static function csrfDataAttribute($requestId = '', $keyAttributeName = 'csrfkey')
    {
        $key = self::getCsrfKey($requestId);
        if ($key) {
            return ' data-' . $keyAttributeName . '="' . $key . '"';
        }

        return '';
    }


    public static function validateCSRFKey($key, $requestId = '')
    {
        $keyLen = 2 * self::CSRF_RANDOM_LEN;

        if (hash_hmac('sha1', substr($key, 0, $keyLen) . $requestId, self::getCsrfToken()) != substr($key, $keyLen)) {
            throw new \Exception(TextManager::_('Overovaci_klic_jiz_neni_platny'));
        }
    }

    /**
     * Overi platnost CSRF klice z GET pozadavku pro zadany requestString
     * @param string $requestId
     * @param string $keyParameterName
     * @throws \Exception
     */
    public static function validateCSRFGetParam($requestId = '', $keyParameterName = 'csrfkey')
    {
        if ($GLOBALS['csrf'] ?? true) {
            self::validateCSRFKey($_GET[$keyParameterName] ?? '', $requestId);
        }
    }


    /**
     * Overi platnost CSRF klice z POST pozadavku pro zadany requestString
     * @param string $keyParameterName
     * @throws \Exception
     */
    public static function validateCSRFPostParam($requestId = '', $keyParameterName = 'csrfkey')
    {
        if ($GLOBALS['csrf'] ?? true) {
            self::validateCSRFKey($_POST[$keyParameterName] ?? '', $requestId);
        }
    }
}
