<?php
/**
 * UTF8
 * stanislav.elbl, 07.12.2017
 */

namespace Ron;


/**
 * Polozka v seznamu udalosti.
 */
class ProfilerItem
{
    /** @var string */
    public $category;
    /** @var string */
    public $description;
    /** @var string[] */
    public $texts = [];

    /** @var array */
    public $backTrace;

    /** @var int */
    public $startTime;
    /** @var int */
    public $endTime;

    /** @var int    Pocet odsazeni ve vypisu */
    public $indent = 0;

    /**
     * ProfilerItem constructor.
     * @param string [$description]
     * @param string [$category]
     */
    public function __construct($description = null, $category = null)
    {
        $this->description = $description;
        $this->category = $category;
        $this->startTime = microtime(true);
    }


    public function backTraceString()
    {
        if (!$this ->backTrace) {
            return '';
        }

        $result = '';
        foreach ($this ->backTrace as $backTraceItem) {
            $result .=
                ($result ? ' / ' : '') .
                $backTraceItem['file'] . ':' . $backTraceItem['line'] . ' ' .
                ($backTraceItem['class'] ?? '') .
                ($backTraceItem['type'] ?? '') .
                $backTraceItem['function'];
        }

        return $result;
    }

}


/**
 * Pomocna trida pro ladeni aplikace. Zajistuje mereni doby konkretnich udalosti a generovani vystupu s namerenymi hodnotami.
 */
class Profiler
{
    /** @var Profiler */
    private static $profiler;

    /** @var float  Cas startu profileru */
    protected $startTime;

    /** @var ProfilerItem[] */
    protected $items = [];

    /** @var string */
    public $indentString = '  ';

    /** @var int Pokud je pocet zaznamu mensi nez minimalni, tak vystup nezapisuje */
    public $minimalItemsCount = 1;

    /** @var int Po jakem case (pocet sekund) se vyresetuje cislo vystupniho souboru (0 = neresetovat, cislovat porad dale) */
    public $fileNameResetTime = 0;

    /** @var string     Pokud neni zadano, nastavi se automaticky */
    public $fileName = null;

    /** @var string     Vystupni adresar. Musi byt nastaven pro prubezne zapisovani. Pokud neni prubezne zapisovani povoleno, neni nutne nastavovat. */
    public $outputDir = null;

    /** @var bool       Zapina prubezne zapisovani pri Profiler::addItem */
    public $writeOutputOnStartItem = false;
    /** @var bool       Zapina prubezne zapisovani pri Profiler::endItem */
    public $writeOutputOnEndItem = false;
    /** @var bool       Priznak, jestli se ma pri prubeznem zapisovani generovat HTML */
    public $writeOutputHtml = true;



    /**
     * Profiler constructor.
     */
    public function __construct()
    {
        $this ->startTime = microtime(true);
    }


    /**
     * @return Profiler
     */
    public static function getInstance()
    {
        return self::$profiler ?: self::$profiler = new static();
    }


    public function getItemsCount()
    {
        return count($this ->items);
    }

    /**
     * Prida novou polozku do seznamu
     * @param string|ProfilerItem $itemDescription  text polozky, nebo primo polozka, ktera se ma pridat
     * @param null $category
     * @return ProfilerItem
     */
    public static function addItem($itemDescription, $category = null)
    {
        if (!self::$profiler) return null;

        $newItem = ($itemDescription instanceof ProfilerItem) ? $itemDescription : new ProfilerItem($itemDescription, $category);
        if ($newItem ->description) {
            $newItem->backTrace = debug_backtrace();
            array_splice($newItem->backTrace, 0, 1); // odstrani volani teto metody
        }

        self::$profiler ->items[] = $newItem;

        if (self::$profiler ->writeOutputOnStartItem   &&  self::$profiler ->outputDir) {
            self::$profiler ->writeOutput();
        }
        return $newItem;
    }


    /**
     * Ukonci zadanou polozku - nastavi konecny cas a pripadne odsadi vsechny vnorene polozky
     * @param ProfilerItem $item
     * @param string|\Exception|string[] $addText   Nepovinny text, ktery prida k polozce
     */
    public static function endItem($item, $addText = null)
    {
        if (!$item) return;
        if (!self::$profiler) return;

        $item ->endTime = microtime(true);
        if ($addText) {
            if (is_array($addText)) {
                foreach ($addText as $line) {
                    $item ->texts[] = $line;
                }
            } else if ($addText instanceof \Exception) {
                $item ->texts[] = $addText ->getMessage();
            } else {
                $item ->texts[] = $addText;
            }
        }

        $itemIndex = array_search($item, self::getInstance() ->items, true);
        if ($itemIndex !== false) {
            for ($i = $itemIndex + 1; $i < count(self::getInstance() ->items);  $i++) {
                self::getInstance() ->items[$i] ->indent++;
            }
        }

        if (self::$profiler ->writeOutputOnEndItem   &&  self::$profiler ->outputDir) {
            self::$profiler ->writeOutput();
        }
    }


    public function getOutputHtml()
    {
        $result =
            '<html><head><link rel="stylesheet" type="text/css" href="ron_profiler.css"></head><body>' .
            '<table class="profiler"><tr><th class="text-right">Start</th><th>Duration</th><th>Description</th></tr>';

        foreach ($this ->items as $item) {
            $result .=
                "<tr data-level='{$item ->indent}' class='item level_{$item ->indent}'>" .
                "<td class='text-right'>" . (int)(1000 * ($item ->startTime - $this ->startTime)) . '</td>' .
                "<td class='text-right'>" . (int)(1000 * ($item ->endTime - $item ->startTime)) . '</td>' .
                '<td class="description">' . StringUtils::htmlTextConvert($item ->description) . "</td>" .
                "</tr>\n";

            if ($item ->backTrace) {
                $result .=
                    "<tr data-level='{$item ->indent}' class='backTrace level_{$item ->indent}'>" .
                    '<td></td><td></td>' .
                    '<td class="description">' . $item ->backTraceString() . '</td>' .
                    "</tr>\n";
            }

            foreach ($item ->texts as $line) {
                $result .=
                    "<tr data-level='{$item ->indent}' class='comment level_{$item ->indent}'>" .
                    '<td></td><td></td>' .
                    '<td class="description">' . StringUtils::htmlTextConvert($line) . '</td>' .
                    "</tr>\n";
            }
        }

        $result .=
            '</table>' .
            '</body></html>';

        return $result;
    }


    public function getOutputAsText()
    {
        $result = '';
        foreach ($this ->items as $item) {
            $indentStr = str_repeat($this->indentString, $item->indent);

            if  ($item ->description) {
                $result .=
                    str_pad((int)(1000 * ($item->startTime - $this->startTime)), 4, ' ', STR_PAD_LEFT) .
                    ' ' .
                    str_pad((int)(1000 * ($item->endTime - $item->startTime)), 4, ' ', STR_PAD_LEFT) .
                    ' ' .
                    $indentStr .
                    //($item ->endTime ? '- ' . (int)(1000*($item ->endTime - $this ->startTime)) .
                    // ' (' . (int)(1000*($item ->endTime - $item ->startTime)) . 'ms) ' : '') .
                    $item->description;
            }

            $result .= "\n";

            foreach ($item ->texts as $line) {
                $result .= '          ' . $indentStr . $line . "\n";
            }

            if ($item ->backTrace) {
                $result .= '          ' . $indentStr . $item ->backTraceString() . "\n";
            }
        }

        $result .= microtime(true) - $this ->startTime . '-------------------';
        return $result;
    }


    public function getFileName($html)
    {
        if ($this ->fileName) return $this ->fileName;

        $fileIndex = isset($_SESSION['profiler_last_file_index'])
            ? $_SESSION['profiler_last_file_index'] + 1
            : 1;

        if ($this ->fileNameResetTime) {
            if (isset($_SESSION['profiler_last_time'])  &&  $_SESSION['profiler_last_time'] < time() - $this ->fileNameResetTime) {
                $fileIndex = 1;
            }
            $_SESSION['profiler_last_time'] = time();
        }

        $_SESSION['profiler_last_file_index'] = $fileIndex;

        $this ->fileName = "ron_profiler_$fileIndex." . ($html ? 'html' : 'txt');

        return $this ->fileName;
    }


    public function writeOutput($outputDir = null, $html = null)
    {
        if (count($this ->items) < $this ->minimalItemsCount) {
            // neni co zapisovat
            return;
        }

        file_put_contents (
            ($outputDir ?? $this ->outputDir) . '/' . $this ->getFileName($html),
            ($html ?? $this ->writeOutputHtml) ? $this ->getOutputHtml() : $this ->getOutputAsText() );
    }
}
