<?php

/**
 * UTF8
 */

namespace Ron\Tests;

class TestItem
{
    const OPEN_URL = 'openUrl';
    const SELECTOR_EXIST = 'selectorExists';
    const SET_VALUE = 'setValue';
    const CLICK = 'click';
    const WAIT = 'wait';

    public $itemType;

    public $url;          // pro openUrl
    public $maxLoadTime;  // maximalni povolena doba nacitani pozadavku (sekund)

    public $selector;
    public $minCount;
    public $maxCount;

    public $value;

    public $millis;       // pro wait

    /**
     * @param string $type
     * @param array $data
     */
    public function __construct($type, $data)
    {
        $this ->itemType = $type;

        foreach ($data as $name => $value ) {
            $this ->{$name} = $value;
        }
    }


    public function getJson()
    {
        $obj = [
            'itemType' =>   $this ->itemType,
            'url' =>        $this ->url,
            'selector' =>   $this ->selector,
            'minCount' =>   $this ->minCount,
            'maxCount' =>   $this ->maxCount,
            'value' =>      $this ->value,
            'millis' =>     $this ->millis
        ];

        return $obj;
    }
}

class Test
{
    public $code;
    public $name;
    public $description;

    public $fileName;


    /** @var TestItem[] */
    public $items = [];

    /**
     * Vytvori novou instanci testu
     * @param $code
     * @param null $name
     * @param null $description
     * @return Test
     */
    public static function newTest($code, $name = null, $description = null)
    {
        $result = new Test();
        $result ->code = $code;
        $result ->name = $name ?: $code;
        $result ->description = $description;

        return $result;
    }


    /**
     * Cekani zadany pocet milisekund. Pokud neni zadan, tak ceka, dokud se nenacte stranka (napr. po provedeni post nebo kliknuti na link)
     * @param null $millis
     * @return $this
     */
    function wait($millis = null)
    {
        $this ->items[] = new TestItem(TestItem::WAIT, ['millis' => $millis]);
        return $this;
    }


    /**
     * Nacteni zadane URL adresy
     * @param $url
     * @param bool $waitForLoad
     * @param int $maxLoadTime
     * @return Test
     */
    function openUrl($url, $waitForLoad = true, $maxLoadTime = null)
    {
        $this ->items[] = new TestItem(TestItem::OPEN_URL, ['url' => $url, 'maxLoadTime' => $maxLoadTime]);
        if ($waitForLoad) {
            $this ->wait();
        }
        return $this;
    }

    /**
     * Overeni, ze ve strance existuje zadany selektor s pripadnou kontrolou minimalniho a maximalniho poctu prvku
     * @param string $selector
     * @param int $minCount
     * @param int $maxCount
     * @return Test
     */
    function selectorExists($selector, $minCount = 1, $maxCount = null)
    {
        $this ->items[] = new TestItem(TestItem::SELECTOR_EXIST, ['selector' => $selector, 'minCount' => $minCount, 'maxCount' => $maxCount]);
        return $this;
    }


    /**
     * Overeni, ze ve strance neexistuje zadany selektor
     * @param string $selector
     * @return Test
     */
    function selectorNotExists($selector)
    {
        return $this ->selectorExists($selector, 0, 0);
    }

    /**
     * Nastaveni hodnoty $newValue zadaneho prvku (selector)
     * @param $selector
     * @param $newValue
     * @return Test
     */
    function setValue($selector, $newValue)
    {
        $this ->items[] = new TestItem(TestItem::SET_VALUE, ['selector' => $selector, 'value' => $newValue]);
        return $this;
    }

    /**
     * Overeni, ze ve strance neexistuje zadany selektor
     * @param string $selector
     * @return Test
     */
    function click($selector)
    {
        $this ->items[] = new TestItem(TestItem::CLICK, ['selector' => $selector]);
        return $this;
    }


    function getJson()
    {
        $itemsObj = [];
        foreach($this ->items as $item) {
            $itemsObj[] = $item ->getJson();
        }

        return [
            'fileName' => $this ->fileName,
            'items' => $itemsObj
        ];
    }
}


class TestManager
{
    /** @var Test[]  */
    public $tests = [];

    /**
     * @param string $path
     * @return TestManager  Vraci $this
     */
    public function loadTests($path)
    {
        $dh = opendir($path);

        if (!$dh) {
            echo "Chyba pri nacitani testu z adresare $path!";
            return $this;
        }

        $fileList = [];
        while (false !== ($fileName = readdir($dh)))
            if (!is_dir($path . '/' . $fileName)) $fileList[] = $fileName;

        foreach ($fileList as $fileName) {
            /** @noinspection PhpIncludeInspection */
            $test = require($path . '/' . $fileName);

            if ($test) {
                $test ->fileName = $path . '/' . $fileName;
                $this->tests[] = $test;
            }
        }

        return $this;
    }


    function getJson()
    {
        $testsObj = [];
        foreach($this ->tests as $test) {
            $testsObj[] = $test ->getJson();
        }

        return ['tests' => $testsObj];
    }

}
