<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron;

use Tracy\Dumper;

/**
 * Class TextList
 * Pomocna trida - seznam textu pro preklady
 */
class TextList
{
    /** @var string[] $texts     Seznam textu pro preklad (asociativni pole) */
    var $texts;

    /**
     * TextList constructor.
     * @param \string[] $texts
     */
    public function __construct($texts = null)
    {
        if ($texts) $this->texts = $texts;
        else $this ->texts = [];
    }


    /**
     * Prida zadane asociativni pole
     * @param \string[] $texts
     */
    public function addTexts($texts)
    {
        $this ->texts = array_merge($this ->texts, $texts);
    }

    /**
     * Prida zadany text
     * @param $id
     * @param $text
     * @return void
     */
    public function addText($id, $text)
    {
        $this ->texts[$id] = $text;
    }
}



class TextManager
{
    const MAIN_LANG_DIR = 'lang/';

    /** @var string     Jazyk pro preklady */
    var $translationLang;
    /** @var string     Jazyk s implicitnimi texty */
    var $defaultLang;

    var $langDir;
    /** @var string     Znakova sada */
    var $charSet;

    /** @var TextList */
    var $texts;
    /** @var TextList   V tomto seznamu se hleda prednostne */
    var $preferredTexts;

    /** @var bool       Priznak, ze se nema prekladat */
    var $dontTranslate;

    private static $textManager;


    /**
     * TextManager constructor.
     */
    public function __construct()
    {
        $this ->texts = new TextList();
        $this ->preferredTexts = new TextList();

        $this ->langDir = self::MAIN_LANG_DIR;
        $this ->charSet = 'utf-8';

        if (isset($_GET['translate'])) {
            $this ->dontTranslate = !(int)$_GET['translate'];
            $_SESSION['TextManager#translate'] = (int)$_GET['translate'];
        } else {
            $this ->dontTranslate = isset($_SESSION['TextManager#translate']) ? !(int)$_SESSION['TextManager#translate'] : 0;
        }
    }


    public static function getInstance()
    {
        if (!self::$textManager) {
            self::$textManager = new TextManager();
        }

        return self::$textManager;
    }


    /**
     * Nacte texty ze zadaneho ini souboru
     * @param $fileName
     * @param bool $preferred
     * @return bool
     */
    public function loadFile($fileName, $preferred = false)
    {
        if ( !file_exists($fileName) ) {
            return false;
        }

        $a = parse_ini_file($fileName);

        if ($a) {
            if ($preferred) {
                $this ->preferredTexts ->addTexts($a);
            } else {
                $this->texts->addTexts($a);
            }
        }

        return true;
    }


    /**
     * Nacte texty ze souboru se zadanym jmenem. Nacita soubor z defaultLangDir a pak z langDir
     * @param string $fileName
     * @return TextManager  Vraci $this
     */
    public function loadStrings($fileName)
    {
        if ($this ->defaultLang) {
            $this->loadFile($this ->langDir . $this->defaultLang . '/' . $fileName);
        }

        if ($this ->translationLang  &&  $this ->translationLang !== $this ->defaultLang) {
            $this->loadFile($this ->langDir . $this->translationLang . '/' . $fileName);
        }

        return $this;
    }


    /**
     * Nacte retezce ze zadaneho pole - napriklad necteneho pomoci XML loaderu z XML s elementy v nekterem z techto formatu:
     * <translation id="id_textu" text="Vychozi text" text-en="Anglicky text" />
     * <translation id="id_textu" text-en="Anglicky text" >Jinak zadany vychozi text</translation>
     * <translation id="id_textu2" lang="cz">Cesky text</translation>
     * @param \ArrayAccess $array
     * @return void
     */
    public function loadArray($array)
    {
        if (!$array) {
            return;
        }

        $langCode = strtolower($this ->translationLang);

        foreach ($array as $item) {
            $id = $item['id'] ?? null;
            $itemLang = $item['lang'] ?? null;
            if ($id  &&  (!$itemLang || strtolower($itemLang) === $langCode)) {
                // polozka nema zadan atribut lang, nebo je zadan a shoduje se s aktualne zobrazenym jazykem
                $this ->preferredTexts ->addText($id, $item['text-' . $langCode] ?? $item['text'] ?? $item['#text'] ?? '');
            }
        }
    }


    public function isUTF8()
    {
        return $this ->charSet === 'utf-8';
    }


    public function getCharSet()
    {
        return $this ->charSet;
    }

    /**
     * Vraci prelozeny retezec. POZOR! Preklada i kdyz je pouzito &translate=0! Pouzivat jen v pripade nutnosti!
     * @param $text
     * @param string $defaultValue  Pokud je zadana vychozi hodnota, tak ji vraci v pripade, ze dany text neexistuje
     * @return string
     */
    public function getTranslatedText($text, $defaultValue = null)
    {
        if ( isset($this ->preferredTexts ->texts[$text]) ) {
            return $this ->preferredTexts ->texts[$text];
        }

        if ( isset($this ->texts ->texts[$text]) ) {
            return $this ->texts ->texts[$text];
        }

        return isset($defaultValue)
            ? (($defaultValue === false) ? null : $defaultValue)
            : $text;
    }

    /**
     * Vraci prelozeny retezec (pokud neni preklad vypnuty)
     * @param string $text
     * @param string $defaultValue  Vraci tuto hodnotu (pokud neni null) v pripade, ze preklad neexistuje
     * @return string
     */
    public function translate($text, $defaultValue = null)
    {
        return $this ->dontTranslate
            ? $text
            : $this ->getTranslatedText($text, $defaultValue);
    }


    /**
     * Vraci javascriptovy objekt se vsemi preklady
     */
    public function javascript()
    {
        $textArray = [];

        foreach ($this ->texts ->texts as $index => $text) {
            $textArray[$index] = $text;
        }

        foreach ($this ->preferredTexts ->texts as $index => $text) {
            $textArray[$index] = $text;
        }

        $resultData = [];
        foreach ($textArray as $index => $text) {
            $resultData[StringUtils::jsonText($index)] = StringUtils::jsonText($text); // prevadi se i index, aby mohla byt diakritika i v identifikatorech
        }

        return "ron.translations = " . json_encode($resultData);  // ,  JSON_INVALID_UTF8_IGNORE - az od PHP 7.2 :-(
    }


    /**
     * Vraci text pro zadany identifikator
     * @param $text
     * @param string $defaultValue
     * @return string
     */
    public static function _($text, $defaultValue = null)
    {
        return self::getInstance() -> translate($text, $defaultValue);
    }
}


/**
 * @param string $text
 * @param string $defaultValue
 * @return string
 */
function ronText($text, $defaultValue = null)
{
    return TextManager::getInstance() -> translate($text, $defaultValue);
}
