<?php

namespace RonPortal\Controls;

use Ron\ArrayUtils;
use Ron\Controls\ComboBox;
use Ron\Controls\ConfigurableObject;
use Ron\Controls\LinksList;
use Ron\Controls\LinksListItem;
use Ron\Controls\ObjectConfiguration;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NastaveniUzivatele;
use RonPortal\User;
use Tracy\Debugger;
use Tracy\Dumper;


class FilterFormFilterItem extends LinksListItem implements ConfigurableObject
{
    /** @var string|SqlStatement|null   SQL filtr, ktery se pouzije pro filtrovani v seznamu */
    public $sqlFilter = null;

    /** @var bool       Urcuje, jestli se tato hodnota filtru zobrazi a jestli je mozne ji pouzivat */
    public $enabled = true;


    public function __construct($key, $caption)
    {
        parent::__construct($key, $caption);
        $this ->active = false;
    }


    /**
     * @param SqlStatement|string|null $sqlFilter
     * @return FilterFormFilterItem
     */
    public function setSqlFilter($sqlFilter)
    {
        $this->sqlFilter = $sqlFilter;
        return $this;
    }

    /**
     * @param bool $enabled
     * @return FilterFormFilterItem
     */
    public function setEnabled(bool $enabled): FilterFormFilterItem
    {
        $this->enabled = $enabled;
        return $this;
    }

    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'key' :
            case 'sqlFilter' :
            case 'cssClass' :
            case 'group' :
                $this ->{$key} = $valueText;
                break;

            case 'caption' :
            case 'title' :
                $this ->{$key} = TextManager::_($valueText);
                break;

            case 'enabled' :
                $this ->{$key} = (int)$valueText;
                break;
        }
    }
}


/**
 * Reprezentuje jeden konkretni filtr (obvykle jeden radek) v obecnem filtrovacim formulari @link RonPortalFilterForm.
 */
class FilterFormFilter implements ConfigurableObject
{
    /** @var string     Filtr se zobrazi jako odkazy */
    const TYPE_LINKS = 'links';
    /** @var string     Filtr se zobrazi jako combobox */
    const TYPE_OPTIONS = 'options';
    /** @var string     Filtr se zobrazi jako editovaci pole */
    const TYPE_EDIT = 'edit';
    /** @var string     Filtr se zobrazi jako editovaci pole pro zadani celeho cisla */
    const TYPE_INTEGER = 'integer';
    /** @var string     Zobrazi se jen text. Muze obsahovat HTML znacky. */
    const TYPE_TEXT = 'text';
    /** @var string     Filtr se zobrazi jako 2 comboboxy (napr. kod - popis) */
    const TYPE_COMBOBOXES = 'comboboxes';
    /** @var string     Filtr se zobrazi jako checkbox */
    const TYPE_CHECKBOX = 'checkbox';
    /** @var string     Filtr se zobrazi jako checkbox */
    const TYPE_HIDDEN = 'hidden';

    /** @var string     Filtr se zobrazi nahore nad preddefinovanymi filtry */
    const POSITION_TOP = 'top';
    /** @var string     Filtr se zobrazi pod preddefinovanymi filtry */
    const POSITION_DEFAULT = 'default';
    /** @var string     Filtr se zobrazi az pod formularem (lze pouzit napriklad pro odkaz na vlozeni noveho zaznamu a podobne) */
    const POSITION_BELOW_FORM = 'below';
    /** @var string     Filtr se zobrazi nad formularem */
    const POSITION_ABOVE_FORM = 'above';

    /** @var string     Typ filtru - tj. zpusob, jakym se filtr zobrazi (napr. seznam odkazu, select) */
    public $type = self::TYPE_LINKS;

    /** @var string     Jmeno filtru. Pouzije se predevsim pro jmeno GET parametru v URL. */
    public $name;

    /** @var RonPortalFilterForm    Rodicovsky formular */
    public $filterForm;

    /** @var string     Pozice, kde se ma filtr zobrazit. */
    public $position = self::POSITION_DEFAULT;

    /** @var string|bool    Jmeno pro ulozeni hodnoty filtru do SESSION. Pokud je true, tak ze pouzije "filterform_<$name>". Pokud se vyhodnoti jako false, tak se do session neuklada. */
    public $sessionName = true;

    /** @var string|bool    Jmeno pro ulozeni hodnoty filtru do COOKIE. Pokud je true, tak ze pouzije "filterform_<$name>". Pokud se vyhodnoti jako false, tak se do cookie neuklada. */
    public $cookieName = false;

    /** @var int  */
    public $cookieTimeout = 30000000;

    /** @var string|bool    Jmeno pro ulozeni hodnoty filtru do uzivatelskeho nastaveni. Pokud je true, tak se pouzije "filterform_<$name>". Pokud se vyhodnoti jako false, tak se do uzivatelskeho nastaveni neuklada. */
    public $userSettingsName = false;

    /** @var string         Css trida HTML elementu. Pokud neni zadana, urci se automaticky podle typu filtru. */
    protected $defaultCssClass = null;

    /** @var string         Css trida, ktera se prida ke standardni css tride prvku */
    public $addClass;

    /** @var string|bool     Nadpis filtru. Pokud je false, tak se nadpis vubec nezobrazi */
    public $caption;
    /** @var string|bool     Druhy nadpis filtru napriklad pro checkbox (zobrazi se za checkboxem). */
    public $caption2;

    /** @var string[]|null  HTML atributy inputu */
    protected $htmlAttributes = null;
    /** @var string[]|null  HTML atributy druheho selectu pro "comboboxes" */
    protected $htmlAttributes2 = null;
    /** @var string[]|null  HTML atributy nadpisu */
    public $captionHtmlAttributes;

    /** @var FilterFormFilterItem[]     Jednotlive polozky pro zobrazeni odkazu nebo vyber pomoci <select> */
    public $items = [];
    /** @var FilterFormFilterItem[]     Jednotlive polozky pro zobrazeni comboboxu 2 */
    public $items2 = [];

    /** @var bool   U typu TYPE_LINKS umoznuje zabranit zobrazeni aktivni polozky */
    protected $showSelectedItem = true;

    /** @var string U typu TYPE_LINKS umoznuje nastavit primou URL - pokud je nastavena, tak se nepouzije hidden pole. U typu text renderuje HTML odkaz */
    protected $url = null;

    /** @var string|SqlStatement    SQL vyraz pro nacteni polozek ($items) z databaze. */
    protected $optionsSql;
    /** @var string|null|SqlStatement   SQL filtr pro options. Pokud je zadan, tak nahradi ve vyrazu optionsSql retezec %FILTER%. POZOR - SqlStatement je podporovan jen v omezene mire! */
    protected $optionsSqlFilter = null;
    /** @var string|null            SQL razeni pro options. Pokud je zadano, tak nahradi ve vyrazu optionsSql retezec %ORDER% */
    protected $optionsSqlOrder = null;
    /** @var string|null            Prefix pro nalezeni prekladu dat z databaze */
    protected $optionsTranslationPrefix = null;
    /** @var string|null            SQL razeni pro options v comboboxu 2 (typ "comboboxes". Pokud je zadano, tak nahradi ve vyrazu optionsSql retezec %ORDER% */
    protected $optionsSqlOrder2 = null;
    /** @var int[]                  Asociativni pole indexu poli v sql vyrazu, ktere se pridaji jako datove atributy k jednotlivym polozkam */
    protected $attrIndexes = null;
    /** @var int[]                  Asociativni pole indexu poli v sql vyrazu, ktere se pridaji jako datove atributy k jednotlivym polozkam v druhem comboboxu */
    protected $attrIndexes2 = null;

    /** @var bool           Urcuje, jestli se ma zobrazit kod v textu jednotlivych voleb (options) */
    protected $showCode = false;

    /** @var bool           Urcuje, jestli musi byt hodnota vzdy zadana. Pokud neni zadana, tak se pouzije vychozi hodnota (napr. pro EDIT, INTEGER) */
    protected $required = false;

    /** @var mixed          Vychozi hodnota */
    protected $defaultValue;

    /** @var mixed          Maximalni hodnota (napr. pro INTEGER) */
    protected $maxValue;
    /** @var mixed          Minimalni hodnota (napr. pro INTEGER) */
    protected $minValue;

    /** @var bool           Urcuje, jestli je filtr povolen. Pokud neni, tak se vubec nezobrazi a nefiltruje se podle nej. */
    public $enabled = true;

    /** @var string|bool    HTML class pro vygenerovani radku filtru. Pokud je false, tak se radek vubec negeneruje */
    public $rowClass = 'filterrow';
    /** @var string|bool    HTML id radku filtru. Pokud je true, tak se pouzije "filterrow_<$name>" */
    public $rowId = true;

    /** @var mixed          Skutecna aktualni hodnota zvolena uzivatelem */
    protected $value = null;

    /** @var string|null    SQL filtr - prida se k nemu parametr s vybranou hodnotou - takze by mel mit tvar napriklad 'tabulka.pole = ' */
    protected $sqlFilter;

    /** @var mixed[]        Dalsi volby ktere mohou byt vyuzity v kodu a nastaveny v XML konfiguraci */
    public $otherSettings = [];

    /** @var bool           Urcuje, jestli filtr definuje obdobi (takze se pak podle nej omezi napriklad vyber osob nebo rozdeleni). Pokud je true, tak musi vracet hodnotu pro specification datumOd a datumDo. */
    public $definesPeriod = false;

    /** @var bool           Urcuje, jestli ma byt filtr pridan do URL browseru */
    public $includeInUrl = true;

    /**
     * @param string $name
     * @param string $type
     */
    public function __construct(string $name, string $type = self::TYPE_LINKS)
    {
        $this->name = $name;
        $this->type = $type;
        $this->sessionName = 'filterform_' . $name;

        if ($this->type === self::TYPE_HIDDEN) {
            $this->rowClass = false;
            $this->caption = false;
        }
    }


    /**
     * Nastavi pozici filtru (viz konstanty POSITION_...)
     * @param string $position
     * @return FilterFormFilter
     */
    public function setPosition(string $position): FilterFormFilter
    {
        $this->position = $position;
        return $this;
    }


    /**
     * @param bool|string $caption
     * @return FilterFormFilter
     */
    public function setCaption($caption): FilterFormFilter
    {
        $this->caption = $caption;
        return $this;
    }


    /**
     * @param bool|string $caption2
     * @return FilterFormFilter
     */
    public function setCaption2($caption2): FilterFormFilter
    {
        $this->caption2 = $caption2;
        return $this;
    }


    /**
     * @param string[]|null $htmlAttributes
     * @return FilterFormFilter
     */
    public function setHtmlAttributes(?array $htmlAttributes): FilterFormFilter
    {
        $this->htmlAttributes = $htmlAttributes;
        return $this;
    }

    /**
     * @param string[]|null $htmlAttributes2
     * @return FilterFormFilter
     */
    public function setHtmlAttributes2(?array $htmlAttributes2): FilterFormFilter
    {
        $this->htmlAttributes2 = $htmlAttributes2;
        return $this;
    }


    /**
     * @param string[]|null $captionHtmlAttributes
     * @return FilterFormFilter
     */
    public function setCaptionHtmlAttributes($captionHtmlAttributes)
    {
        $this->captionHtmlAttributes = $captionHtmlAttributes;
        return $this;
    }


    /**
     * @param FilterFormFilterItem[] $items
     * @return FilterFormFilter
     */
    public function setItems(array $items): FilterFormFilter
    {
        $this->items = $items;
        return $this;
    }


    /**
     * @param FilterFormFilterItem|string $item
     * @param string|null $caption
     * @return FilterFormFilter
     */
    public function addItem($item, ?string $caption = null): FilterFormFilter
    {
        if ($item instanceof FilterFormFilterItem) {
            $this->items[$item->key] = $item;
        } else {
            $this->items[$item] = new FilterFormFilterItem($item, $caption);
        }
        return $this;
    }

    /**
     * @param FilterFormFilterItem|string $item
     * @param string|null $caption
     * @return FilterFormFilter
     */
    public function addItem2($item, ?string $caption = null): FilterFormFilter
    {
        if ($item instanceof FilterFormFilterItem) {
            $this->items2[$item->key] = $item;
        } else {
            $this->items2[$item] = new FilterFormFilterItem($item, $caption);
        }
        return $this;
    }


    /**
     * @param bool $showSelectedItem
     * @return FilterFormFilter
     */
    public function setShowSelectedItem(bool $showSelectedItem): FilterFormFilter
    {
        $this->showSelectedItem = $showSelectedItem;
        return $this;
    }


    /**
     * @param string|null $url
     * @return FilterFormFilter
     */
    public function setUrl($url): FilterFormFilter
    {
        $this->url = $url;
        return $this;
    }


    /**
     * @param SqlStatement|string $optionsSql
     * @param string|null|SqlStatement $optionsSqlFilter
     * @param string|null $optionsSqlOrder
     * @return FilterFormFilter
     */
    public function setOptionsSql($optionsSql, $optionsSqlFilter = null, $optionsSqlOrder = null): FilterFormFilter
    {
        $this->optionsSql = $optionsSql;
        $this->optionsSqlFilter = $optionsSqlFilter;
        $this->optionsSqlOrder = $optionsSqlOrder;

        return $this;
    }

    /**
     * @param string|null $optionsTranslationPrefix
     * @return FilterFormFilter
     */
    public function setOptionsTranslationPrefix(?string $optionsTranslationPrefix): FilterFormFilter
    {
        $this->optionsTranslationPrefix = $optionsTranslationPrefix;
        return $this;
    }


    /**
     * @param bool $showCode
     * @return FilterFormFilter
     */
    public function setShowCode(bool $showCode): FilterFormFilter
    {
        $this->showCode = $showCode;
        return $this;
    }


    /**
     * @param string|null|SqlStatement $optionsSqlFilter
     * @return FilterFormFilter
     */
    public function setOptionsSqlFilter($optionsSqlFilter): FilterFormFilter
    {
        $this->optionsSqlFilter = $optionsSqlFilter;
        return $this;
    }

    /**
     * @param string|null $optionsSqlOrder
     * @return FilterFormFilter
     */
    public function setOptionsSqlOrder(?string $optionsSqlOrder): FilterFormFilter
    {
        $this->optionsSqlOrder = $optionsSqlOrder;
        return $this;
    }

    /**
     * @param string|null $optionsSqlOrder2
     * @return FilterFormFilter
     */
    public function setOptionsSqlOrder2(?string $optionsSqlOrder2): FilterFormFilter
    {
        $this->optionsSqlOrder2 = $optionsSqlOrder2;
        return $this;
    }


    /**
     * @param int[]|null $attrIndexes
     * @return FilterFormFilter
     */
    public function setAttrIndexes(?array $attrIndexes): FilterFormFilter
    {
        $this->attrIndexes = $attrIndexes;
        return $this;
    }

    /**
     * @param int[]|null $attrIndexes2
     * @return FilterFormFilter
     */
    public function setAttrIndexes2(?array $attrIndexes2): FilterFormFilter
    {
        $this->attrIndexes2 = $attrIndexes2;
        return $this;
    }


    /**
     * @return bool
     */
    public function isRequired()
    {
        return $this->required;
    }


    /**
     * @param bool $required
     * @return FilterFormFilter
     */
    public function setRequired(bool $required): FilterFormFilter
    {
        $this->required = $required;
        return $this;
    }


    /**
     * @param mixed $defaultValue
     * @return FilterFormFilter
     */
    public function setDefaultValue($defaultValue): FilterFormFilter
    {
        $this->defaultValue = $defaultValue;
        return $this;
    }


    /**
     * @param mixed $maxValue
     * @return FilterFormFilter
     */
    public function setMaxValue($maxValue)
    {
        $this->maxValue = $maxValue;
        return $this;
    }

    /**
     * @param mixed $minValue
     * @return FilterFormFilter
     */
    public function setMinValue($minValue)
    {
        $this->minValue = $minValue;
        return $this;
    }


    /**
     * @param bool $enabled
     * @return FilterFormFilter
     */
    public function setEnabled(bool $enabled): FilterFormFilter
    {
        $this->enabled = $enabled;
        return $this;
    }


    /**
     * @param bool|string $sessionName
     * @return FilterFormFilter
     */
    public function setSessionName($sessionName): FilterFormFilter
    {
        $this->sessionName = $sessionName;
        return $this;
    }

    /**
     * @param bool|string $userSettingsName
     * @return FilterFormFilter
     */
    public function setUserSettingsName($userSettingsName): FilterFormFilter
    {
        $this->userSettingsName = $userSettingsName;
        return $this;
    }

    /**
     * @param bool|string $cookieName
     * @return FilterFormFilter
     */
    public function setCookieName($cookieName, $cookieTimeout = null)
    {
        $this->cookieName = $cookieName;
        if (isset($cookieTimeout)) {
            $this ->cookieTimeout = $cookieTimeout;
        }
        return $this;
    }

    /**
     * @param int $cookieTimeout
     * @return FilterFormFilter
     */
    public function setCookieTimeout(int $cookieTimeout): FilterFormFilter
    {
        $this->cookieTimeout = $cookieTimeout;
        return $this;
    }





    /**
     * @param string $type
     * @return FilterFormFilter
     */
    public function setType(string $type): FilterFormFilter
    {
        $this->type = $type;
        return $this;
    }


    /**
     * @param string $addClass
     * @return FilterFormFilter
     */
    public function setAddClass($addClass): FilterFormFilter
    {
        $this->addClass = $addClass;
        return $this;
    }


    /**
     * @param bool|string $rowClass
     * @return FilterFormFilter
     */
    public function setRowClass($rowClass): FilterFormFilter
    {
        $this->rowClass = $rowClass;
        return $this;
    }

    /**
     * @param bool|string $rowId
     * @return FilterFormFilter
     */
    public function setRowId($rowId): FilterFormFilter
    {
        $this->rowId = $rowId;
        return $this;
    }


    /**
     * @return mixed|null
     */
    public function getValue(?string $specification = null)
    {
        return $specification
            ? null
            : $this->value;
    }

    /**
     * @param mixed|null $value
     * @return FilterFormFilter
     */
    public function setValue($value): FilterFormFilter
    {
        $this->value = $value;
        return $this;
    }


    /**
     * @param SqlStatement|string|null $sqlFilter
     */
    public function setSqlFilter($sqlFilter): FilterFormFilter
    {
        $this->sqlFilter = $sqlFilter;
        return $this;
    }


    protected function loadOptionsConfiguration($optionsConfiguration)
    {
        foreach ($optionsConfiguration as $optionConfiguration) {
            $key = $optionConfiguration['key'];
            $item = $this->items[$key] ?? null;
            if (!$item) {
                $item = new FilterFormFilterItem($key, TextManager::_($optionConfiguration['#text']));
                $this->items[$key] = $item;
            } else if (is_string($item)) {
                $item = new FilterFormFilterItem($key, isset($optionConfiguration['#text']) ? TextManager::_($optionConfiguration['#text']) : $item);
                $this->items[$key] = $item;
            }

            $objectConfiguration = new ObjectConfiguration($item);
            $objectConfiguration->loadConfigurationArray($optionConfiguration);
        }
    }


    /**
     * @param string $key
     * @param mixed $value
     * @return FilterFormFilter
     */
    public function setOtherSettings($key, $value)
    {
        $this->otherSettings[$key] = $value;

        return $this;
    }


    /**
     * @param bool $definesPeriod
     * @return FilterFormFilter
     */
    public function setDefinesPeriod(bool $definesPeriod): FilterFormFilter
    {
        $this->definesPeriod = $definesPeriod;
        return $this;
    }


    /**
     * Nastavi, jestli se ma filtr pridavat do URL browseru - tento priznak pak browser pouzije pri sestavovani URL v metode getBrowserUrl
     * @param bool $includeInUrl
     * @return FilterFormFilter
     */
    public function setIncludeInUrl(bool $includeInUrl): FilterFormFilter
    {
        $this->includeInUrl = $includeInUrl;
        return $this;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'type' :
            case 'sqlFilter' :
            case 'name' :
            case 'sessionName' :
            case 'cookieName' :
            case 'userSettingsName' :
            case 'optionsSql' :
            case 'optionsSqlFilter' :
            case 'optionsSqlOrder' :
            case 'optionsTranslationPrefix' :
            case 'optionsSqlOrder2' :
            case 'defaultValue' :
            case 'rowClass' :
            case 'addClass' :
            case 'rowId' :
            case 'minValue' :
            case 'maxValue' :
            case 'position' :
                $this->{$key} = $valueText;
                break;

            case 'addRowClass' :
                $this->rowClass .= ' ' . $valueText;
                break;

            case 'caption' :
            case 'caption2' :
                $this->{$key} = TextManager::_($valueText);
                break;

            case 'enabled' :
            case 'required' :
            case 'showCode' :
            case 'showSelectedItem' :
            case 'definesPeriod' :
            case 'cookieTimeout' :
                $this->{$key} = (int)$valueText;
                break;

            case 'option' :
                $this->loadOptionsConfiguration($value);
                break;

            case '#text' :
                break;

            default :
                if (substr($key, 0, 5) === 'html:') {
                    $this ->htmlAttributes[substr($key, 5)] = $value;
                } else if (is_string($value) || is_numeric($value)) {
                    $this->setOtherSettings($key, $value);
                }
                break;
        }
    }


    /**
     * @param Database $database
     * @param array $row
     * @return FilterFormFilterItem
     */
    public function createOption(Database $database, $row)
    {
        $text = $this->optionsTranslationPrefix
            ? TextManager::_($this->optionsTranslationPrefix . $row[0], $row[1])
            : $row[1];

        $item = new FilterFormFilterItem($row[0], $text);
        if ($this->attrIndexes) {
            $item->loadParams($this->attrIndexes, $row);
        }

        return $item;
    }


    /**
     * @param Database $database
     * @param array $row
     * @return FilterFormFilterItem
     */
    public function createOption2(Database $database, $row)
    {
        $text = $this->optionsTranslationPrefix
            ? TextManager::_($this->optionsTranslationPrefix . $row[0], $row[1])
            : $row[1];

        $item = new FilterFormFilterItem($row[0], $row[0]);
        $item->setTitle($text);
        if ($this->attrIndexes2) {
            $item->loadParams($this->attrIndexes2, $row);
        }

        return $item;
    }

    /**
     * Nacte volby z SQL dotazu
     * @param Database $database
     * @return $this
     * @throws DBException
     */
    public function loadOptions(Database $database): FilterFormFilter
    {
        $params = null;

        if ($this->optionsSql) {
            if ($this->optionsSql instanceof SqlStatement) {
                $optionsSql = $this->optionsSql->sqlText;
                $params = $this->optionsSql->parameters;
            } else {
                $optionsSql = $this->optionsSql;
            }

            // TODO: udelat podporu SqlStatement v OptionsSqlFilter?
            // Ale vzhledem k tomu, ze se muze vkladat doprostred, tak nemusi fungovat vzdy spravne!
            if ($this ->optionsSqlFilter instanceof SqlStatement) {
                throw new \Exception('Chyba ve FilterFormFilter: SqlStatement v optionsSqlFilter neni podporovan!');
            }

            $sql = str_replace(
                ['%FILTER%', '%ORDER%'],
                [$this->optionsSqlFilter ?? '1 = 1', $this->optionsSqlOrder ?? ''],
                $optionsSql
            );

            $cursor = $database->getSqlResult($sql, $params);
            while ($row = $cursor->fetch()) {
                $item = $this->createOption($database, $row);
                $this->items[$row[0]] = $item;
            }
            $cursor->close();

            if ($this->type === self::TYPE_COMBOBOXES) {
                $sql = str_replace(
                    ['%FILTER%', '%ORDER%'],
                    [$this->optionsSqlFilter ?? '1 = 1', $this->optionsSqlOrder2 ?? ''],
                    $optionsSql
                );

                $cursor = $database->getSqlResult($sql, $params);
                while ($row = $cursor->fetch()) {
                    $item = $this->createOption2($database, $row);
                    $this->items2[$row[0]] = $item;
                }
                $cursor->close();
            }
        }

        return $this;
    }


    protected function storeValueByName(Database $database, $value, $defaultValue = null, $suffix = '')
    {
        // ulozi hodnotu do session,
        $sessionName = $this->sessionName === true
            ? "filterform_{$this ->name}$suffix"
            : ($this->sessionName ? $this->sessionName . $suffix : null);

        if ($sessionName) {
            $_SESSION[$sessionName] = $value;
        }

        // do uzivatelskeho nastaveni
        $userSettingsName = $this->userSettingsName === true
            ? "filterform_{$this ->name}$suffix"
            : ($this->userSettingsName ? $this->userSettingsName . $suffix : null);

        if ($userSettingsName && User::isAuthenticated()) {
            NastaveniUzivatele::setUserSettingsItem($database, $userSettingsName, $value, $defaultValue);
        }

        // cookie
        $cookieName = $this ->cookieName === true
            ? "filterform_{$this ->name}$suffix"
            : ($this ->cookieName ? $this->cookieName . $suffix : null);

        if ($this ->cookieName) {
            setcookie($cookieName, $value, $this ->cookieTimeout ? time() + $this ->cookieTimeout : 0);
        }
    }

    /**
     * Ulozi hodnotu do session a/nebo do uzivatelskeho nastaveni (podle konfigurace)
     * @param Database $database
     * @return void
     * @throws \Exception
     */
    public function storeValue(Database $database)
    {
        $this ->storeValueByName($database, $this ->value);
    }


    /**
     * Vraci hodnotu ze sesstion/uzivatelskeho nastaveni/cookie. V
     * @param Database $database
     * @param mixed $defaultValue
     * @param string $suffix
     * @return mixed|null
     */
    protected function getDefaultValueByName(Database $database, $defaultValue, $suffix = '')
    {
        // prednostne ze session
        $sessionName = $this->sessionName === true
            ? "filterform_{$this ->name}$suffix"
            : $this->sessionName . $suffix;

        if ($sessionName && isset($_SESSION[$sessionName])) {
            return $_SESSION[$sessionName];
        }

        // ... dale z uzivatelskeho nastaveni
        $userSettingsName = $this->userSettingsName === true
            ? "filterform_{$this ->name}$suffix"
            : $this->userSettingsName . $suffix;

        if ($userSettingsName && User::isAuthenticated()) {
            return NastaveniUzivatele::getUserSettingsItem($database, $userSettingsName, $defaultValue);
        }

        // ... z cookie
        $cookieName = $this ->cookieName === true
            ? "filterform_{$this ->name}$suffix"
            : $this->cookieName . $suffix;

        if ($cookieName  &&  isset($_COOKIE[$cookieName])) {
            return $_COOKIE[$cookieName];
        }

        // a nakonec vychozi hodnotu
        return $defaultValue;
    }


    /**
     * Nacte hodnotu v pripade, ze nebyla zadana uzivatelem - ze session / z nastaveni uzivatele / vychozi hodnotu
     * @param Database $database
     * @return void
     */
    public function loadDefaultValue(Database $database)
    {
        $this ->value = $this ->getDefaultValueByName($database, $this ->defaultValue);
    }


    /**
     * Overi, jestli je aktualni nactena hodnota filtru platna - napriklad jestli je to jedna z existujicich voleb
     * @param Database $database
     * @return void
     */
    public function verifyValue(Database $database)
    {
        switch ($this->type) {
            case self::TYPE_LINKS :
            case self::TYPE_OPTIONS :
            case self::TYPE_COMBOBOXES :
                // overi, jestli existuje volba se zadanou hodnotou
                // pokud neexistuje, tak vybere prvni volbu ze seznamu
                if (isset($this->value) && !isset($this->items[$this->value])) {
                    $this->value = ArrayUtils::firstKey($this->items);
                    if ($this->value === '') {
                        $this->value = null;
                    }
                }
                break;

            case self::TYPE_INTEGER :
                // overi, jestli je aktualni hodnota v povolenem rozsahu (minValue - maxValue)
                if (isset($this ->value)) {
                    $this ->value = (int)$this ->value;
                }
                if (isset($this ->value)  &&  isset($this ->minValue)  &&  ($this ->value < (int)$this ->minValue)) {
                    $this ->value = (int)$this ->minValue;
                }
                if (isset($this ->value)  &&  isset($this ->maxValue)  &&  ($this ->value > (int)$this ->maxValue)) {
                    $this ->value = (int)$this ->maxValue;
                }
                break;
        }

        if ($this ->required  &&  !isset($this ->value)) {
            $this->value = $this ->defaultValue;
        }
    }


    /**
     * Nacte hodnotu z requestu (z $_GET)
     * @return bool
     */
    protected function loadSubmittedValue()
    {
        if ($this->type !== self::TYPE_TEXT && isset($_GET[$this->name])) {
            if ($this->type === self::TYPE_CHECKBOX) {
                $this->value = (int)$_GET[$this->name];
            } else if ($this->type === self::TYPE_INTEGER) {
                $this ->value = StringUtils::isInteger($_GET[$this->name])
                    ? (int)$_GET[$this->name]
                    : $this ->defaultValue;
            } else {
                $this->value = $_GET[$this->name];
                if ($this->value === '') {
                    $this->value = null;
                }
            }
            return true;
        }

        // hodnota nebyla v requestu odeslana
        return false;
    }

    /**
     * Nacte hodnotu zadanou uzivatelem (podle GET) a pripadne seznam polozek z databaze
     * @param Database $database
     * @return void
     * @throws DBException
     */
    public function load(Database $database)
    {
        if (!$this->enabled) {
            return;
        }

        $this->loadOptions($database);

        if ($this ->loadSubmittedValue()) {
            $this ->storeValue($database);
        } else {
            $this ->loadDefaultValue($database);
            // dat tady storeValue? Mozna jedine kvuli staremu kodu, ktery pouziva SESSION.
        }

//        if ($this->type !== self::TYPE_TEXT && isset($_GET[$this->name])) {
//            if ($this->type === self::TYPE_CHECKBOX) {
//                $this->value = (int)$_GET[$this->name];
//            } else {
//                $this->value = $_GET[$this->name];
//                if ($this->value === '') {
//                    $this->value = null;
//                }
//            }
//
//            $this ->verifyValue($database); - pozor, asi nefunguje spravne napr. pro filtr rozdeleni
//            $this->storeValue($database);
//        } else {
//            $this->loadDefaultValue($database);
//        }
    }


    /**
     * Renderuje nadpis filtru
     * @return void
     */
    public function renderCaption()
    {
        echo $this->caption;
    }


    public function getCssClass()
    {
        $result = $this ->defaultCssClass;

        if (!isset($this ->defaultCssClass)) {
            switch ($this->type) {
                case self::TYPE_EDIT :
                case self::TYPE_OPTIONS :
                    $result = 'adsfilterforminput_full form-control';
                    break;

                case self::TYPE_INTEGER :
                    $result = 'form-control inline-form-control filterInt';
                    break;

                case self::TYPE_CHECKBOX :
                    $result = 'clickAutoSubmit setValue';
                    break;

                case self::TYPE_HIDDEN :
                default :
                    $result = '';
            }
        }

        return $result . ($this ->addClass ? ' ' . $this ->addClass : '');
    }


    /**
     * Pomocna funkce pro renderovani filtru typu "comboboxes"
     * @return void
     */
    protected function renderComboBoxes()
    {
        $combo = new ComboBox($this ->name, $this ->items);
        $combo->showCode = $this ->showCode;
        //$combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Popis, Kod', $this ->value);
//        $combo->htmlParams = ['id' => 'filtr_operace', 'data-ron-select2' => '#filtr_operace_kod', 'data-ron-autosubmit' => '1'];
        $combo->selectedOption = $combo ->findOption($this ->value);
        $combo->titleFieldName = 'title';
        $combo
            ->setClassName('adsfilterforminput_popis ' . $this ->addClass)
            ->setHtmlParams(array_merge($this->getDefaultHtmlAttrs(), $this ->htmlAttributes ?? []));
        $combo->render();

        $combo = new ComboBox('', $this->items2);
//        $combo->keyIndex = 0;
//        $combo->captionIndex = 0;
//        $combo->emptyRow = TextManager::_('_vse_');
//        $combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Kod', $this ->value);
//        $combo->htmlParams = 'id="filtr_operace_kod" data-ron-select2="#filtr_operace" data-ron-autosubmit="1"';
        $combo->selectedOption = $combo ->findOption($this ->value);
        $combo->titleFieldName = 'title';
        $combo
            ->setClassName('adsfilterforminput_kod desktop-only ' . $this ->addClass)  // na mobilu se zobrazi jen popis
            ->setHtmlParams($this ->htmlAttributes2);
        $combo->render();
    }


    /**
     * Vraci asociativni pole parametru pro sestaveni URL adresy - standardne [<jmeno_filtru> => <hodnota>]
     * @return array
     */
    public function getUrlParameters()
    {
        if (!$this ->name  ||  !$this ->enabled  ||  $this ->type == self::TYPE_TEXT) {
            return [];
        }

        return [
            $this ->name => $this ->definesPeriod
                ? DateUtils::dateToUrlString($this ->value)
                : ($this ->type == self::TYPE_CHECKBOX ? ($this ->value ? '1' : '0') : $this ->value)
        ];
    }


    public function buildUrl()
    {
        if (!$this ->url) {
            return null;
        }

        return $this ->filterForm ->replaceUrlParameters($this ->url) ?? '';
    }


    public function getDefaultHtmlAttrs()
    {
        return [
            'data-session-name' => $this ->sessionName ?? '',
            'data-user-settings-name' => $this ->userSettingsName ?? ''
        ];
    }

    /**
     * Renderuje obsah filtru (tj. napriklad input nebo select)
     * @return void
     */
    public function render()
    {
        $url = $this ->buildUrl();

        switch ($this ->type) {
            case self::TYPE_LINKS :
                if ($url) {
                    LinksList::renderLinks($this ->items, $this ->value, $url, '', false);
                } else {
                    LinksList::renderLinks($this->items, $this->value, $this->name, '', false,
                        [
                            'hiddenFieldName' => $this->name,
                            'showSelectedItem' => $this->showSelectedItem,
                            'htmlAttributes' => array_merge($this->getDefaultHtmlAttrs(), $this ->htmlAttributes ?? [])
                        ]);
                }
                break;

            case self::TYPE_OPTIONS :
                ComboBox::renderComboBox($this ->name, $this ->items, $this ->value,
                    [
                        'className' => $this ->getCssClass(),
                        'htmlParams' => array_merge(['data-ron-autosubmit' => '1'], $this->getDefaultHtmlAttrs(), $this ->htmlAttributes ?? []),
                        'showCode' => $this->showCode,
                        'dataTypeComparison' => false
                    ]); // PHP automaticky konvertuje indexy na integer! (napr ['' => 'vse', '0' => 'bez', '1' => 's']
                break;

            case self::TYPE_EDIT :
            case self::TYPE_INTEGER :
                echo "<input " . GuiUtils::getAttributesHtml(array_merge(
                        [
                            'class' => $this ->getCssClass(),
                            'name' => $this ->name,
                            'value' => StringUtils::htmlTextConvert($this->value ?? '')
                        ],
                        $this->getDefaultHtmlAttrs(),
                            $this ->htmlAttributes ?? [])) .
                        '/>';
                break;

            case self::TYPE_CHECKBOX :
                echo
                    "<input type='hidden' id='filtr_checkbox_{$this ->name}' name='{$this ->name}' value='" . ($this ->value ? 1 : 0) . "'/>" .
                    "<label>" .
                    "<input type='checkbox' class='" . $this ->getCssClass() . "'" .
                    " data-selector='#filtr_checkbox_{$this ->name}' data-submit='1' data-value='" . ($this ->value ? 0 : 1) . "'" . ($this ->value ? " checked='checked'" : '') .
                    '/>' .
                    ($this ->caption2 ?? '') .
                    '</label>';
                break;

            case self::TYPE_TEXT :
                echo
                    ($this ->addClass ? "<span class='{$this ->addClass}'>" : '') .
                    ($url ? "<a href='{$url}'>" : '') .
                    $this ->value .
                    ($url ? "</a>" : '') .
                    ($this ->addClass ? '</span>' : '');
                break;

            case self::TYPE_COMBOBOXES :
                $this ->renderComboBoxes();
                break;

            case self::TYPE_HIDDEN :
                echo
                    '<input ' . GuiUtils::getAttributesHtml(array_merge(
                        [
                            'type' => 'hidden',
                            'name' => $this->name,
                            'value' => StringUtils::htmlTextConvert($this ->value ?? ''),
                            'class' => $this ->getCssClass()
                        ],
                        $this ->htmlAttributes ?: [])) . "/>";
        }
    }


    /**
     * Sestavi SQL filtr podle aktualne zvolene hodnoty
     * @param Database $database
     * @param Parameters $params
     * @return string|null
     */
    public function buildSQLFilter(Database $database, Parameters $params)
    {
//        Debugger::dump($this ->items);
//        Debugger::dump($this ->value);
//        Debugger::dump($this ->items[$this ->value]);
        if (!$this ->enabled) {
            return null;
        }

        if ($this ->type === self::TYPE_CHECKBOX) {
            return $this ->value
                ? $this ->sqlFilter
                : null;
        }

        $item = $this ->items[$this ->value] ?? null;

        if (($item instanceof FilterFormFilterItem)  &&  $item ->sqlFilter) {
            if ($item->sqlFilter instanceof SqlStatement) {
                $params->appendParameters($item->sqlFilter->parameters);
                return $item->sqlFilter->sqlText;
            }
            return $item ->sqlFilter;
        }

        if (!isset($this ->value)) {
            return null;
        }

        if ($this ->sqlFilter) {
            return strpos($this ->sqlFilter, ' LIKE ')
                ? $this ->sqlFilter . $params ->addParameter("%{$this ->value}%") // sql filtr obsahuje LIKE
                : $this ->sqlFilter . $params ->addParameter($this ->value);
        }

        return null;
    }


    /**
     * Vraci seznam promennych - napr. pro XML formulare
     * @return array
     */
    public function getVariables()
    {
        if ($this->name) {
            return [$this ->name => $this ->value];
        }

        return [];
    }
}
