<?php
/**
 * pavel.rudol, 19.12.2024 17:49
 */

namespace RonPortal\Mzdy;

use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\DBRecordField;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\UserLoginFormField;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;

class PrirazeneDetiOznameniField extends FormField
{
    /** @var array */
    private $osoby;

    /** @var bool */
    private $lzeMazat;


    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_TEXT);

        // implicitne muze mazat, pokud ma pravo na editaci osoby prirazene k davce (overeno v controlleru)
        $this->lzeMazat = true;
    }

    protected function loadConfigurationItem($key, $value)
    {
        if ($key === 'lzeMazat') {
            $this->lzeMazat = (int)$value;
        }

        parent::loadConfigurationItem($key, $value);
    }


    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);
        $result['data-lzeMazat'] = $this->lzeMazat ? '1' : '0';
        return $result;
    }


    private function nactiDetiProOznameni(Database $database, $idOznameni)
    {
        $this->osoby = [];

        if (!$idOznameni) {
            // neni co nacitat - neni editovane existujici oznameni (jedna se o INSERT)
            return;
        }

        $params = $database->createParameters();
        $cursor = $database->select(
            'SELECT RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno, RodinnyPrislusnik.RC_RodinnehoPrislusnika, RodinnyPrislusnik.DatumNarozeni, ' .
            '(SELECT S_RodinnyVztah.Popis FROM S_RodinnyVztah JOIN ClenoveRodiny ON ClenoveRodiny.RodinnyVztah = S_RodinnyVztah.Kod WHERE ClenoveRodiny.RC_RodinnehoPrislusnika = RodinnyPrislusnik.RC_RodinnehoPrislusnika 
                AND ClenoveRodiny.RC = OznameniKDavkamNZ.RC)' .
            ' FROM PPM_Deti JOIN RodinnyPrislusnik ON PPM_Deti.RC_RodinnehoPrislusnika = RodinnyPrislusnik.RC_RodinnehoPrislusnika ' .
            '   JOIN OznameniKDavkamNZ ON OznameniKDavkamNZ.ID = PPM_Deti.ID' .
            ' WHERE PPM_Deti.ID = ' . $params->addParameter($idOznameni) .
            ' ORDER BY PPM_Deti.Poradi, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno',
            $params
        );

        while ($row = $cursor->fetch()) {
            $this->osoby[] = [
                'rc' => $row[2],
                'prijmeni' => $row[0],
                'jmeno' => $row[1],
                'datumNarozeni' => $database->stringToTimestamp($row[3]),
                'vztah' => $row[4]
            ];

        }
    }


    public function loadDatabaseValue(Database $database, ?FormRowArray $rowData)
    {
        $this->nactiDetiProOznameni($database, $rowData['field-id']);
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // header
        echo
            '<div class="row d-none d-sm-flex background-table-header color-table-header">' .
            '<div class="col-3 padding-all">' . TextManager::_('Jmeno') . '</div>' .
            '<div class="col-3 padding-all">' . TextManager::_('Prijmeni') . '</div>' .
            '<div class="col-3 padding-all">' . TextManager::_('Datum_narozeni') . '</div>' .
            '<div class="col-2 padding-all">' . TextManager::_('rodinny_vztah') . '</div>' .
            '</div>';

        foreach ($this->osoby as $index => $item) {
            $htmlSuffix = $item['rc'];

            echo
                "<div class='dite-osoby row' id='row_diteosoby_$index'>" .
                // jmeno ditete
                '<div class="col-12 col-sm-3">' .
                StringUtils::htmlTextConvert($item['jmeno']) .
                '</div>' .

                // prijmeni ditete
                '<div class="col-12 col-sm-3">' .
                StringUtils::htmlTextConvert($item['prijmeni']) .
                '</div>' .

                // datum narozeni
                '<div class="col-12 col-sm-3">' .
                StringUtils::format('date', $item['datumNarozeni']) .
                '</div>' .

                // vztah
                '<div class="col-12 col-sm-2">' .
                StringUtils::htmlTextConvert($item['vztah']) .
                '</div>' .

                // ikony
                '<div class="col-1">' .
                ($this->lzeMazat
                    ? "<input type='hidden' name='Delete_$htmlSuffix' value='0' id='Delete_$htmlSuffix'/>" .
                    "<i class='inline-glyph glyph-button fa glyph-delete setValue toggleClass' data-value='1' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_diteosoby_$index' data-add-class='row-to-delete'></i>" .
                    "<i class='inline-glyph glyph-button fa glyph-undo d-none setValue toggleClass' data-value='0' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_diteosoby_$index' data-remove-class='row-to-delete'></i>"
                    : ''
                ) .
                '</div>' .
                '</div>';
        }
    }


    /**
     * @param Database $database
     * @param int $idOznameni
     * @param object $logDescription
     * @return bool
     * @throws DBException
     */
    public function zapisZmeny($database, $idOznameni, $logDescription)
    {
        if (!$this->visible || $this->readOnly) {
            return true;
        }

        if (!$this->osoby) {
            $this->nactiDetiProOznameni($database, $idOznameni);
        }

        $result = true;

        foreach ($this->osoby as $item) {
            if ($this->lzeMazat) {
                $htmlSuffix = $item['rc'];

                if ('1' === ($_POST["Delete_$htmlSuffix"] ?? false)) {
                    $recPPMDeti = DavkyNZForm::getRecPPMDeti($database, $logDescription, $idOznameni, $item['rc']);
                    if ($recPPMDeti->loadRecord()) {
                        $recPPMDeti->deleteRecord();
                    }
                }
            }

        }

        return $result;
    }
}

class PecovalOdDoField extends FormField
{
    /** @var array */
    private $data;

    /** @var bool */
    private $lzeEditovatHistorii;


    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_TEXT);

        // implicitne muze mazat, pokud ma pravo na editaci osoby prirazene k davce (overeno v controlleru)
        $this->lzeEditovatHistorii = true;
    }

    protected function loadConfigurationItem($key, $value)
    {
        if ($key === 'lzeEditovat') {
            $this->lzeEditovatHistorii = (int)$value;
        }

        parent::loadConfigurationItem($key, $value);
    }


    public function getFieldAttributes($defaultAttrs = null, $rowData = null)
    {
        $result = parent::getFieldAttributes($defaultAttrs, $rowData);
        $result['data-lzeEditovat'] = $this->lzeEditovatHistorii ? '1' : '0';
        return $result;
    }


    private function nactiDataProOznameni(Database $database, $idOznameni)
    {
        $this->data = [];

        if (!$idOznameni) {
            // neni co nacitat - neni editovane existujici oznameni (jedna se o INSERT)
            return;
        }

        $params = $database->createParameters();
        $cursor = $database->select(
            'SELECT PecovalOd, PecovalDo' .
            ' FROM OznameniDNPPecovalVeDnech' .
            ' WHERE OznameniDNPPecovalVeDnech.ID = ' . $params->addParameter($idOznameni) .
            ' ORDER BY OznameniDNPPecovalVeDnech.PecovalOd',
            $params
        );

        while ($row = $cursor->fetch()) {
            $this->data[] = [
                'od' => $database->stringToTimestamp($row[0]),
                'do' => $database->stringToTimestamp($row[1])
            ];

        }
    }


    public function loadDatabaseValue(Database $database, ?FormRowArray $rowData)
    {
        $this->nactiDataProOznameni($database, $rowData['field-id']);
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        // header
        echo
            '<div class="row d-none d-sm-flex background-table-header color-table-header">' .
            '<div class="col-5 align-right padding-all">' . TextManager::_('Od') . '</div>' .
            '<div class="col-5 align-right padding-all">' . TextManager::_('Do') . '</div>' .
            '</div>';

        if (is_array($this->data)) {
            foreach ($this->data as $index => $item) {
                $htmlSuffix = $item['od'];

                $platnostOdHtmlName = 'PecovalOd_' . $htmlSuffix;
                $platnostOdStr = ($this->lzeEditovatHistorii && isset($_POST[$platnostOdHtmlName]))
                    ? StringUtils::htmlTextConvert($_POST[$platnostOdHtmlName])
                    : StringUtils::format('date', $item['od']);


                $platnostDoHtmlName = 'PecovalDo_' . $htmlSuffix;
                $platnostDoStr = ($this->lzeEditovatHistorii && isset($_POST[$platnostDoHtmlName]))
                    ? StringUtils::htmlTextConvert($_POST[$platnostDoHtmlName])
                    : StringUtils::format('date', $item['do']);

                echo
                    "<div class='doba-pece row' id='row_pecovalod_$index'>" .
                    // od
                    '<div class="col-12 col-sm-5 align-right">' .
                    ($this->lzeEditovatHistorii
                        ? '<input name="' . $platnostOdHtmlName . '" class="form-control date-input ' . MessageManager::getClass($platnostOdHtmlName) . '" data-roncalendar-next="' . $platnostDoHtmlName . '" value="' . $platnostOdStr . '"/>'
                        : $platnostOdStr
                    ) .
                    '</div>' .

                    // do
                    '<div class="col-12 col-sm-5 align-right">' .
                    ($this->lzeEditovatHistorii
                        ? '<input name="' . $platnostDoHtmlName . '" class="form-control date-input ' . MessageManager::getClass($platnostDoHtmlName) . '" data-roncalendar-prev="' . $platnostOdHtmlName . '" value="' . $platnostDoStr . '"/>'
                        : $platnostDoStr
                    ) .
                    '</div>' .

                    // ikony
                    '<div class="col-2 align-right">' .
                    ($this->lzeEditovatHistorii
                        ? "<input type='hidden' name='Delete_$htmlSuffix' value='0' id='Delete_$htmlSuffix'/>" .
                        "<i class='inline-glyph glyph-button fa glyph-delete setValue toggleClass' data-value='1' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_pecovalod_$index' data-add-class='row-to-delete'></i>" .
                        "<i class='inline-glyph glyph-button fa glyph-undo d-none setValue toggleClass' data-value='0' data-setvalue-selector='#Delete_$htmlSuffix' data-toggleclass-selector='#row_pecovalod_$index' data-remove-class='row-to-delete'></i>"
                        : ''
                    ) .
                    '</div>' .
                    '</div>';
            }
        }

        echo
            "<div class='doba-pece row' id='row_pecovalod_new'>" .
            // od
            '<div class="col-12 col-sm-5 align-right">' .
            '<input name="PecovalOd" class="form-control date-input ' . MessageManager::getClass('PecovalOd') . '" data-roncalendar-next="PecovalDo" value="' . ($_POST['PecovalOd'] ?? '') . '"/>' .
            '</div>' .

            // do
            '<div class="col-12 col-sm-5 align-right">' .
            '<input name="PecovalDo" class="form-control date-input ' . MessageManager::getClass('PecovalDo') . '" data-roncalendar-prev="PecovalOd" value="' . ($_POST['PecovalDo'] ?? '') . '"/>' .
            '</div>' .

            '</div>';
    }


    /**
     * @param Database $database
     * @param int $idOznameni
     * @param object $logDescription
     * @return bool
     * @throws DBException
     */
    public function zapisZmeny($database, $idOznameni, $logDescription)
    {
        if (!$this->visible || $this->readOnly) {
            return true;
        }

        if (!$this->data) {
            $this->nactiDataProOznameni($database, $idOznameni);
        }

        $result = true;

        foreach ($this->data as $item) {
            if ($this->lzeEditovatHistorii) {
                $htmlSuffix = $item['od'];

                if ('1' === ($_POST["Delete_$htmlSuffix"] ?? false)) {
                    $recPecoval = DavkyNZForm::getRecPecoval($database, $logDescription, $idOznameni, $item['od']);
                    if ($recPecoval->loadRecord()) {
                        $recPecoval->deleteRecord();
                    }
                } else if (isset($_POST["PecovalDo_$htmlSuffix"])) {
                    // uzivatel odeslal hodnotu pro tento input
                    try {
                        $postPlatnostDo = trim($_POST["PecovalDo_$htmlSuffix"]);
                        $platnostDo = $postPlatnostDo === ''
                            ? null
                            : DateUtils::dateStringToTimestamp($postPlatnostDo, "PecovalDo_$htmlSuffix");

                        $postPlatnostOd = trim($_POST["PecovalOd_$htmlSuffix"]);
                        $platnostOd = $postPlatnostOd === ''
                            ? null
                            : DateUtils::dateStringToTimestamp($postPlatnostOd, "PecovalOd_$htmlSuffix");

                        // kontrola
                        if ($platnostDo && $platnostDo < $platnostOd) {
                            throw new RonException(TextManager::_('DavkyNZ_pecoval_do_je_mensi_nez_pecoval_od'), "PecovalDo_$htmlSuffix");
                        }

                        // zapise zaznam
                        $recPecoval = DavkyNZForm::getRecPecoval($database, $logDescription, $idOznameni, $item['od']);
                        if ($this->lzeEditovatHistorii) {
                            $recPecoval['PecovalOd'] = $platnostOd;
                        }
                        $recPecoval['PecovalDo'] = $platnostDo;

                        if ($recPecoval->loadRecord()) {
                            $recPecoval->updateRecord();
                        }
                    } catch (\Exception $e) {
                        MessageManager::addErrorMessage($e);
                        $result = false;
                    }
                }
            }

        }

        return $result;
    }
}

class OsetrovanaOsobaField extends FormField
{
    private $rcOsoby;
    private $jenDeti;
    /** @var OznameniDavekNZ|null  */
    private $oznameni;

    public function __construct(string $rcOsoby, bool $jenDeti, OznameniDavekNZ $oznameni = null)
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);

        $this
            ->setFieldName('RC_RodinnyPrislusnik')
            ->setCode('field-dite')
            ->setCaption('DavkyNZ_Dite')
            ->setRowId('davkynz-row-dite')
            ->setFieldId('davkynz-dite');

        $this->rcOsoby = $rcOsoby;
        $this->jenDeti = $jenDeti;
        $this->oznameni = $oznameni;
    }

    public function loadOptions($database)
    {
        $this->setSelectSql(DavkyNZForm::getSQLRodinnyPrislusnik($database, $this->rcOsoby, $this->jenDeti, true, $this->oznameni), '1 = 1', 'Prijmeni, Jmeno');

        parent::loadOptions($database);

        $this->options = array_merge($this->options, [new Option('NEW',TextManager::_('DavkyNZ_PridatPrislunika'))]);
    }

    private function vlozNovehoPrislusnika(Database $database)
    {
        $rec = new RonPortalDBRecord($database, 'RodinnyPrislusnik');

        $rec->addStringField('RC_RodinnehoPrislusnika', true, 15);
        $rec->addStringField('Jmeno', true, 15);
        $rec->addStringField('Prijmeni', true, 15);
        $rec->addStringField('RodneCisloRP', true, 15);
        $rec->addDateField('DatumNarozeni', true);

        $params = $database->createParameters();
        $rec->setWhere('RodneCisloRP = ' . $params->addParameter($_POST['novyprislusnik-rc']), $params);

        $rec->checkMultiple = false;

        if (!$rec->loadRecord()) {
            $rec->findFieldByName('RodneCisloRP')->setNewValue($_POST['novyprislusnik-rc']);
        }

        $rec->findFieldByName('Jmeno')->setNewValue($_POST['novyprislusnik-jmeno']);
        $rec->findFieldByName('Prijmeni')->setNewValue($_POST['novyprislusnik-prijmeni']);
        $rec->findFieldByName('DatumNarozeni')->setNewValue(DateUtils::dateStringToTimestamp($_POST['novyprislusnik-datumnarozeni']));

        if ($rec->isChanged()) {
            // vygeneruje nove ID
            $idRC = SQLUtils::getNextRecordSerialCode($database, 'RodinnyPrislusnik', 'RC_RodinnehoPrislusnika',"ID######", 1, 999999);

            $rec->findFieldByName('RC_RodinnehoPrislusnika')->setNewValue($idRC);
            $rec->insertRecord();
        }

        return $rec['RC_RodinnehoPrislusnika'];
    }

    /** Zkontroluje zadani jmena, prijmeni a rodneho cisla osetrovane osoby. Kontrola se provede, pokud neni vybrana osoba z rodinnych prislusniku.
     * @return void
     * @throws \Exception
     */
    private function kontrolaNovehoPrislusnika()
    {
        //kontrola pri vkladani noveho rodinneho prislusnika
        if (empty($_POST['novyprislusnik-rc'])) {
            throw new RonException(TextManager::_('DavkyNZ_chybi_rc'), 'novyprislusnik-rc');
        }
        if (empty($_POST['novyprislusnik-jmeno'])) {
            throw new RonException(TextManager::_('DavkyNZ_chybi_jmeno'), 'novyprislusnik-jmeno');
        }
        if (empty($_POST['novyprislusnik-prijmeni'])) {
            throw new RonException(TextManager::_('DavkyNZ_chybi_prijmeni'), 'novyprislusnik-prijmeni');
        }
    }

    public function createDBRecordField($database, $tableName, $insert, $utf8 = false)
    {
        $field = new DBRecordField($this->sqlFieldName, $this ->dataType, $this ->readOnly, $this->maxLength); // format = ''

        $value = trim($this->getPostValueText($utf8));

        if ($value == 'NEW') {
            $this->kontrolaNovehoPrislusnika();

            $value = $this->vlozNovehoPrislusnika($database);
        }

        $field ->setValue($value);

        return $field;
    }
}

class DavkyNZForm extends RonPortalForm
{
    /** @var int OCR (OSE) - osetrovani */
    const DRUH_DAVKY_OCR = 1;
    /** @var int penezita pomoc v materstvi */
    const DRUH_DAVKY_PPM = 3;
    /** @var int otcovska poporodni pece */
    const DRUH_DAVKY_OPP = 5;
    /** @var int dlouhodobe osetrovne */
    const DRUH_DAVKY_DLO = 6;

    private $fieldDetiPPM;
    private $fieldPecovalOdDo;

    /**
     * @param Database $database
     * @throws NeniPravoException
     */
    public function __construct($database)
    {
        parent::__construct($database);

        $oznameni = null;
        $editovatelnyStatus = true;
        if (!$this->isInsert()) {
            $oznameni = OznameniDavekNZ::load($this->database, $_GET['record'] ?? $_POST['Id']);

            if (!(Prava::vratPrava('davkynz_zapis')->lzeZobrazitPP($oznameni->idPP))) {
                throw new NeniPravoException();
            }

            $editovatelnyStatus = in_array($oznameni->status, [0,3]);
        }

        $this->code = 'davkynzform';

        $this->defaultRowClass = 'row';
        $this->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this->defaultDataColClass = 'col-12 col-sm-9';

        $this->htmlAction = '?menuid=davkynz';
        $this->stornoAction = '?menuid=davkynz';

        $this->title = TextManager::_('DavkyNZ_oznameni');

        $this->sqlTableName = 'OznameniKDavkamNZ';

        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-hlavicka-oznameni');
        // ID
        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id')->setPrimary(true) //->setSequence(true)
            ->setCode('field-id');

        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('CisloRozhodnutiStatus')
            ->setCode('field-cislorozhodnutistatus')
            ->setDefaultValue(0);

        $this->addField()
            ->setFieldName('CisloDokladu')->setReadOnly(!$editovatelnyStatus)
            ->setCode('field-cislodokladu')
            ->setCaption('DavkyNZ_CisloDokladu')
            ->setRowId('davkynz-row-cislodokladu')
            ->setFieldId('davkynz-cislodokladu')
            ->addFieldCssClass('davkynz-vyberdavky')
            ->setDataColumnClass('col-12 col-sm-2')
            ->setMaxLength(18);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('Zahranicni')->setReadOnly(!$editovatelnyStatus)
            ->setCode('field-zahranicni')
            ->setCaption2('DavkyNZ_zahranicni')
            ->setFieldId('davkynz-zahranicni')
            ->setCaptionColumnClass('col-sm-1')
            ->setDataColumnClass('col-12 col-sm-2')
            ->setNewRow(false);

        $this->addField($this->isInsert() ? FormField::FIELD_TYPE_HIDDEN : FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ePodani')->setReadOnly()
            ->setCode('field-epodani')
            ->setCaption2($this->isInsert() ? '' : 'DavkyNZ_epodani')
            ->setFieldId('davkynz-epodani')
            ->setCaptionColumnClass('col-sm-1')
            ->setDataColumnClass('col-12 col-sm-2')
            ->setDefaultValue(true)
            ->setNewRow(false);

        $this->addField(new OsobaFormField('davkynz_zapis', time(), is_null($oznameni) ? null : $oznameni->pracovniPomer))
            ->setFieldName('osobadavky')->setSqlFieldName('')->setReadOnly(!$editovatelnyStatus)
            ->setCode('field-osobadavky')
            ->setCaption('Zamestnanec')
            ->setRowId('davkynz-row-osobadavky')
            ->setFieldId('davkynz-osobadavky')
            ->setIncludeToDBRecord(true);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DruhDavky')->setReadOnly()
            ->setCode('field-druh-davky')
            ->setCaption('DavkyNZ_DruhDavky')
            ->setRowId('davkynz-row-druhdavky')
            ->setFieldId('davkynz-druhdavky')
            ->addFieldCssClass('davkynz-vyberdavky')
            ->setSelectSql('SELECT Kod, Popis FROM DruhDavkyNZ WHERE (%FILTER%) ORDER BY %ORDER%', /*$this->isInsert() ? 'Kod IN (5,6)' :*/ 'Kod IN (1,3,5,6)', 'Popis');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('ZadostOdDne')->setReadOnly(!$editovatelnyStatus)
            ->setRequired(true)
            ->setCode('field-zadostoddne')
            ->setCaption('DavkyNZ_PocatekDavky')
            ->setRowId('davkynz-row-zadostoddne')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setOtherHtmlParams(['data-roncalendar-next' => 'ZadostDoDne']);

        if ($this->isInsert()) {
            //otcovska, osetrovani, dlo
            $this->addField(new OsetrovanaOsobaField(User::getRC(), false))
                ->setCode('field-osetrovanaosoba')
                ->setCaption('DavkyNZ_OsetrovanaOsoba')
                ->setRowId('davkynz-row-osetrovanaosoba')
                ->setFieldId('davkynz-osetrovanaosoba');

            $this->addFieldsNovyPrislusnik();
        }

        if (!is_null($oznameni)) {
            switch ((int)$oznameni->druhDavky) {
                case static::DRUH_DAVKY_OCR: //OCR
                case static::DRUH_DAVKY_DLO: //DLO (dlouhodobe osetrovne)

                    $this->findFieldByName('ZadostOdDne')
                        ->setCaption(($oznameni->druhDavky == 1) ? 'DavkyNZ_Vznik' : 'DavkyNZ_DLO_od');

                    $this->addField(FormField::FIELD_TYPE_DATE)
                        ->setFieldName('ZadostDoDne')->setReadOnly(!$editovatelnyStatus)
                        ->setCode('field-zadostdodne')
                        ->setCaption('Do')
                        ->setRowId('davkynz-row-zadostdodne')
                        ->setDataColumnClass('col-12 col-sm-3')
                        ->setOtherHtmlParams(['data-roncalendar-prev' => 'ZadostOdDne'])
                        ->setNewRow(false);

                    $this->addField(FormField::FIELD_TYPE_GROUP)
                        ->setCode('group-osetrovana-osoba');

                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('header-osetrovana-osoba')->setCaption('DavkyNZ_OsetrovanaOsoba_vztah');

                    $this->addField(new OsetrovanaOsobaField(User::getRC(), false, $oznameni))
                        ->setReadOnly(!$editovatelnyStatus)
                        ->setCode('field-osetrovanaosoba')
                        ->setCaption('DavkyNZ_OsetrovanaOsoba')
                        ->setRowId('davkynz-row-osetrovanaosoba')
                        ->setFieldId('davkynz-osetrovanaosoba');

                    $this->addFieldsNovyPrislusnik();

                    if ($oznameni->druhDavky == static::DRUH_DAVKY_DLO) { //DLO (dlouhodobe osetrovne)
                        $vztahArr = [];
                        for ($i = 0; $i < 29; $i++) {
                            $vztahArr[$i] = TextManager::_("DavkyNZ_Vztah_$i");
                        }

                        $this->addField(FormField::FIELD_TYPE_COMBO)
                            ->setFieldName('KodVztah')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                            ->setCode('field-vztah')
                            ->setCaption('DavkyNZ_Vztah')
                            ->setRowId('davkynz-row-vztah')
                            ->setOptions($vztahArr);

                        $this->addField(FormField::FIELD_TYPE_CHECK)
                            ->setFieldName('JeStridani')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                            ->setCode('field-jestridani')
                            ->setCaption2('DavkyNZ_JeStridani')
                            ->setRowId('davkynz-row-jestridani');
                    }

                    if ($oznameni->druhDavky == static::DRUH_DAVKY_OCR) { //OCR
                        $vztahArr = [];
                        for ($i = 0; $i < 7; $i++) {
                            $vztahArr[$i] = TextManager::_("DavkyNZ_RodVztah_$i");
                        }

                        $this->addField(FormField::FIELD_TYPE_COMBO)
                            ->setFieldName('KodRodVztah')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                            ->setCode('field-rodvztah')
                            ->setCaption('DavkyNZ_RodinnyVztah')
                            ->setRowId('davkynz-row-rodvztah')
                            ->setOptions($vztahArr,false);

                        $duvodArr = [];
                        for ($i = 0; $i < 4; $i++) {
                            $duvodArr[$i] = TextManager::_("DavkyNZ_DuvodOsetrovani_$i");
                        }

                        $this->addField(FormField::FIELD_TYPE_COMBO)
                            ->setFieldName('DuvodPece')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                            ->setCode('field-duvodosetrovani')
                            ->setCaption('DavkyNZ_DuvodOsetrovani')
                            ->setRowId('davkynz-row-duvodosetrovani')
                            ->setFieldId('davkynz-duvodosetrovani')
                            ->setOptions($duvodArr, false)
                            ->setOtherHtmlParams(['data-duvodpece' => $oznameni->duvodPece]);

                        $this->addField()
                            ->setFieldName('nazevZarizeniSkoly')->setReadOnly(!in_array($oznameni->status, [0, 3]))
                            ->setCode('field-nazevzarizeni')
                            ->setCaption('DavkyNZ_NazevZarizeni')
                            ->setRowId('davkynz-row-nazevzarizeni')
                            ->setMaxLength(250);

                        $this->addField()
                            ->setFieldName('ICZarizeniSkoly')->setReadOnly(!in_array($oznameni->status, [0, 3]))
                            ->setCode('field-iczarizeni')
                            ->setCaption('DavkyNZ_ICZarizeni')
                            ->setRowId('davkynz-row-iczarizeni')
                            ->setMaxLength(20);
                    }

                    $this->addField(FormField::FIELD_TYPE_GROUP_END);

                    $this->addField(FormField::FIELD_TYPE_GROUP)
                        ->setCode('group-sdeleni-zamestnance');

                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('header-sdeleni-zamestnance')->setCaption('DavkyNZ_sdeleni_zamestnance');

                    if ($oznameni->druhDavky == static::DRUH_DAVKY_OCR) { //OCR
                        $this->addField(FormField::FIELD_TYPE_CHECK)
                            ->setFieldName('jeOsamely')
                            ->setCode('field-osamely')
                            ->setCaption2('DavkyNZ_Osamely')
                            ->setRowId('davkynz-row-osamely');
                    }

                    $this->addField(FormField::FIELD_TYPE_CHECK)
                        ->setFieldName('spolecnaDomacnost')
                        ->setCode('field-spolecnadomacnost')
                        ->setCaption2('DavkyNZ_SpolecnaDomacnost')
                        ->setRowId('davkynz-row-spolecnadomacnost');

                    $this->addField(FormField::FIELD_TYPE_CHECK)
                        ->setFieldName('narokNaPPMjinouOsobou')
                        ->setCode('field-narokjinouosobou')
                        ->setCaption2('DavkyNZ_NarokNaPPMjinouOsobou')
                        ->setRowId('davkynz-row-narokjinouosobou')
                        ->addFieldCssClass('toggleDisabled')
                        ->setOtherHtmlParams(['data-selector' => '.davkynz-jinaosoba']);

                    $this->addField(FormField::FIELD_TYPE_CHECK)
                        ->setFieldName('jinaFOParagraf57')
                        ->setCode('field-foparagraf57')
                        ->setCaption2('DavkyNZ_JinaFOParagraf57')
                        ->setRowId('davkynz-row-foparagraf57')
                        ->addFieldCssClass('davkynz-jinaosoba')
                        ->setOtherHtmlParams(['disabled' => 'disabled']);

                    if ($oznameni->druhDavky == static::DRUH_DAVKY_OCR) { //OCR
                        $this->addField(FormField::FIELD_TYPE_CHECK)
                            ->setFieldName('narokNaRPjinaOsobaNecerpa')
                            ->setCode('field-narokosobanecerpa')
                            ->setCaption2('DavkyNZ_NarokNaRPjinaOsobaNecerpa')
                            ->setRowId('davkynz-row-narokosobanecerpa')
                            ->addFieldCssClass('davkynz-jinaosoba')
                            ->setOtherHtmlParams(['disabled' => 'disabled']);

                        $this->addField(FormField::FIELD_TYPE_CHECK)
                            ->setFieldName('vPeciDiteDo16Let')
                            ->setCode('field-vpeciditedo16let')
                            ->setCaption2('DavkyNZ_PeceDiteDo16let')
                            ->setRowId('davkynz-row-vpeciditedo16let');
                    }

                    $this->addField(FormField::FIELD_TYPE_CHECK)
                        ->setFieldName('pecovalPoCelouDobu')
                        ->setCode('field-pecovalceloudobu')
                        ->setCaption2('DavkyNZ_PecovalPoCelouDobu')
                        ->setRowId('davkynz-row-pecovalceloudobu')
                        ->setFieldId('davkynz-pecovalceloudobu');

                    $this->fieldPecovalOdDo = $this->addField(new PecovalOdDoField())
                        ->setCode('field-pecovaloddo')
                        ->setRowId('davkynz-row-pecovaloddo')
                        ->setDataColumnClass('col-12 col-sm-3');

                    break;

                case static::DRUH_DAVKY_PPM: //PPM (materska)
                    $this->findFieldByName('ZadostOdDne')
                        ->setCaption('DavkyNZ_NastupPPM');

                    $this->addField(FormField::FIELD_TYPE_DATE)
                        ->setFieldName('predpokladanyDenPorodu')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                        ->setCode('field-denporodu')
                        ->setCaption('DavkyNZ_PredpokladanyDenPorodu')
                        ->setRowId('davkynz-row-denporodu')
                        ->setDataColumnClass('col-12 col-sm-3')
                        ->setNewRow(false);

                    $this->addField(FormField::FIELD_TYPE_COMBO)
                        ->setFieldName('DuvodPece')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                        ->setCode('field-duvodpece')
                        ->setCaption('DavkyNZ_DuvodPece')
                        ->setRowId('davkynz-row-duvodopece')
                        ->setOptions([0 => TextManager::_('DavkyNZ_DuvodPece_0'), 1 => TextManager::_('DavkyNZ_DuvodPece_1'), 2 => TextManager::_('DavkyNZ_DuvodPece_2'),
                            3 => TextManager::_('DavkyNZ_DuvodPece_3'), 4 => TextManager::_('DavkyNZ_DuvodPece_4')], false);

                    $this->addField(FormField::FIELD_TYPE_GROUP)
                        ->setCode('group-deti');

                    $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                        ->setCode('header-deti')->setCaption('DavkyNZ_DetiPPM');

                    $this->fieldDetiPPM = $this->addField(new PrirazeneDetiOznameniField());

                    if (!($oznameni->ePodani || !$editovatelnyStatus)) {
                        $this->addFieldRodinnyPrislusnik($oznameni->pracovniPomer->rc, true, $oznameni)
                            ->setFieldName('ditePPM')->setSqlFieldName('')
                            ->setCode('field-pridatdite')
                            ->setCaption('DavkyNZ_PridatDite')
                            ->setRowId('davkynz-row-pridatdite')
                            ->setFieldId('davkynz-pridatdite');
                    }

                    break;
                case static::DRUH_DAVKY_OPP: //OPP (otcovska)
                    $this->findFieldByName('ZadostOdDne')
                        ->setCaption('DavkyNZ_NastupOtcovska');

                    $this->addField(FormField::FIELD_TYPE_COMBO)
                        ->setFieldName('DuvodOtcovske')->setReadOnly($oznameni->ePodani || !$editovatelnyStatus)
                        ->setCode('field-duvodotcovske')
                        ->setCaption('DavkyNZ_DuvodOtcovske')
                        ->setRowId('davkynz-row-duvodotcovske')
                        ->setOptions([0 => TextManager::_('DavkyNZ_DuvodOtcovske_0'), 1 => TextManager::_('DavkyNZ_DuvodOtcovske_1'), 2 => TextManager::_('DavkyNZ_DuvodOtcovske_2')], false);

                    $this->addFieldRodinnyPrislusnik($oznameni->pracovniPomer->rc, true, $oznameni)
                        ->setReadOnly($oznameni->ePodani || !$editovatelnyStatus);

                    break;
            }
        }

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('DatumCasVlozeni')
            ->setReadOnly(true)
            ->setCode('field-datumcasvlozeni');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp')->setReadOnly(true)
            ->setCode('field-insert-timestamp');

        $this->addField(new UserLoginFormField(true)); // Insert_UserName

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp')->setUpdateLog(true)
            ->setCode('field-update-timestamp');

        $this->addField(new UserLoginFormField(false))->setUpdateLog(true); // Update_UserName
    }

    public static function getRecPPMDeti(Database $database, $logDescription, $idOznameni, $rcRodinnehoPrislusnika)
    {
        $recPPMDeti = new RonPortalDBRecord($database, 'PPM_Deti', $logDescription);

        $recPPMDeti->addIntField('ID', true, $idOznameni);
        $recPPMDeti->addStringField('RC_RodinnehoPrislusnika', true, 15, $rcRodinnehoPrislusnika);
        $recPPMDeti->addIntField('Poradi', true, 1);

        if ($idOznameni && $rcRodinnehoPrislusnika) {
            $params = $database->createParameters();
            $recPPMDeti->setWhere(
                'ID = ' . $params->addParameter($idOznameni) .
                ' AND RC_RodinnehoPrislusnika = ' . $params->addParameter($rcRodinnehoPrislusnika),
                $params
            );
        }

        $recPPMDeti->setXmlEvents('PPM_DETI_BEFORE_POST', 'PPM_DETI_AFTER_POST');

        return $recPPMDeti;
    }

    public static function getRecPecoval(Database $database, $logDescription, $idOznameni, $pecovalOd)
    {
        $recPecoval = new RonPortalDBRecord($database, 'OznameniDNPPecovalVeDnech', $logDescription);

        $recPecoval->addIntField('ID', true, $idOznameni);
        $recPecoval->addDateField('PecovalOd', true, $pecovalOd);
        $recPecoval->addDateField('PecovalDo', true);

        if ($idOznameni && $pecovalOd) {
            $params = $database->createParameters();
            $recPecoval->setWhere(
                'ID = ' . $params->addParameter($idOznameni) .
                ' AND PecovalOd = ' . $params->addDateParameter($pecovalOd),
                $params
            );
        }

        $recPecoval->setXmlEvents('DNP_PECOVAL_BEFORE_POST', 'DNP_PECOVAL_AFTER_POST');

        return $recPecoval;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param bool $dite filtrovat rodinny vztah na deti
     * @param bool $formFieldFilter pridat do SQL filtr pro konfiguraci
     * @param OznameniDavekNZ $oznameni
     * @return SqlStatement
     */
    public static function getSQLRodinnyPrislusnik($database, $rc, $dite = true, $formFieldFilter = true, $oznameni = null)
    {
        $params = $database->createParameters();
        return new SqlStatement("SELECT RC_RodinnehoPrislusnika, Jmeno {fn_concat()} ' ' {fn_concat()} Prijmeni 
                                        FROM RodinnyPrislusnik WHERE EXISTS(SELECT RC FROM ClenoveRodiny WHERE RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika
                                                                                                       AND ClenoveRodiny.RC = " . $params->addParameter($rc) .
            ($oznameni ? ' AND NOT EXISTS(SELECT ID FROM PPM_Deti WHERE ID = ' . $params->addParameter($oznameni->id) . ' AND PPM_Deti.RC_RodinnehoPrislusnika = RodinnyPrislusnik.RC_RodinnehoPrislusnika)' : '') .
            ($dite ? ' AND ClenoveRodiny.RodinnyVztah IN (' . $params->addParameter(4) . ',' . $params->addParameter(5) . ')' : '') .
            ') ' .  //konec exists
            ($oznameni ? ' OR RodinnyPrislusnik.RC_RodinnehoPrislusnika = ' . $params->addParameter($oznameni->idRodinnehoPrislusnika) : '') .
            ($formFieldFilter ? ' AND (%FILTER%) ORDER BY %ORDER%' : 'ORDER BY Prijmeni, Jmeno'), $params);
    }

    /**
     * @return FormField|string
     */
    public function addFieldRodinnyPrislusnik($rc, $dite = true, $oznameni = null)
    {
        return $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('RC_RodinnyPrislusnik')
            ->setCode('field-dite')
            ->setCaption('DavkyNZ_Dite')
            ->setRowId('davkynz-row-dite')
            ->setFieldId('davkynz-dite')
            ->setSelectSql(static::getSQLRodinnyPrislusnik($this->database, $rc, $dite, true, $oznameni), '1 = 1', 'Prijmeni, Jmeno');
    }

    /** Prida pole pro vlozeni noveho rodinneho prislusnika
     * @return void
     */
    public function addFieldsNovyPrislusnik()
    {
        $this->addField(FormField::FIELD_TYPE_INT)
            ->setFieldName('novyprislusnik-rc')->setSqlFieldName(null)
            ->setCode('field-novyprislusnik-rc')
            ->setCaption('rodne_cislo')
            ->setMaxLength(10)
            ->setDataColumnClass('col-12 col-sm-3')
            ->addFieldCssClass('ronRC')
            ->setOtherHtmlParams(['data-ronrc-inputdatename' => 'novyprislusnik-datumnarozeni'])
            ->setRowClass('row novyprislusnik d-none');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('novyprislusnik-datumnarozeni')->setSqlFieldName(null)
            ->setCode('field-novyprislusnik-datumnarozeni')
            ->setCaption('Datum_narozeni')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setNewRow(false);

        $this->addField()
            ->setFieldName('novyprislusnik-jmeno')->setSqlFieldName(null)
            ->setCode('field-novyprislusnik-jmeno')
            ->setCaption('Jmeno')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setRowClass('row novyprislusnik d-none');

        $this->addField()
            ->setFieldName('novyprislusnik-prijmeni')->setSqlFieldName(null)
            ->setCode('field-novyprislusnik-prijmeni')
            ->setCaption('Prijmeni')
            ->setDataColumnClass('col-12 col-sm-3')
            ->setNewRow(false);
    }

    /** Overi zda cislo dokladu splnuje pozadovany format.
     * @param DBRecord $rec
     * @return void
     * @throws \Exception
     */
    private function kontrolaCislaDokladu($rec)
    {
        // Kontrola na delku znaku - ne ePodani 18 znaku (drive bylo 8), ePodani 18 znaku
        if ($rec['ePodani']) {
            if (strlen($rec['cisloDokladu']) != 18) {
                throw new \Exception(sprintf(TextManager::_('DavkyNZ_chyba_delka'), 18));
            }
        } else {
            if (strlen($rec['cisloDokladu']) != 18) {
                throw new \Exception(sprintf(TextManager::_('DavkyNZ_chyba_delka'), 18));
            }
        }

        /*
         * Podle poslední písmeno čísla dokladu musí být z výčtu: N, Z, M, T, L
         * */
        if (!preg_match('/N|Z|M|T|L/', strtoupper(substr($rec['CisloDokladu'], -1)))) {
            throw new \Exception(TextManager::_('DavkyNZ_chyba_posledniznak'));
        }

        // Kontrola toho, ze krome posledniho znaku musi byt vse cisla
        if (!is_numeric(substr($rec['CisloDokladu'], 0, -1))) {
            throw new \Exception(TextManager::_('DavkyNZ_chyba_cisla'));
        }
    }

    /**
     * @param DBRecord $rec
     * @return void
     * @throws \Exception
     */
    private function kontrolaZadanychDat($rec)
    {
        if (!$rec['DruhDavky']) {
            throw new \Exception(TextManager::_('DavkyNZ_chybi_druh_davky'));
        }

        // Kontrola zadani cisla dokladu
        // u OSE a DLO bude vzdy cislo dokladu, je povinne
        if ((($rec['DruhDavky'] == static::DRUH_DAVKY_OCR) or ($rec['DruhDavky'] == static::DRUH_DAVKY_DLO))
            or
            // PPM musi mit cislo dokladu, pokud je duvod pece=0 (neuvedeno)
            ((($rec['DruhDavky'] == static::DRUH_DAVKY_PPM)) and ($rec['DuvodPece'] == 0))) {
            if (empty($rec['CisloDokladu'])) {
                throw new \Exception(TextManager::_('DavkyNZ_chybi_cislo_dokladu'));
            }
        }

        // Kontrola spravnosti cisla dokladu
        // pro davky oznacene jako zahranicni se kontrola dokladu nebude provadet
        if (!$rec['Zahranicni'] && (
                // u OSE a DLO bude vzdy cislo dokladu, je povinne
                (($rec['DruhDavky'] == static::DRUH_DAVKY_OCR) or ($rec['DruhDavky'] == static::DRUH_DAVKY_DLO))
                or
                // U PPM a OPP jen pokud je vyplneno, protoze neni povinne
                ((($rec['DruhDavky'] == static::DRUH_DAVKY_PPM) or ($rec['DruhDavky'] == static::DRUH_DAVKY_OPP)) and (!empty($rec['CisloDokladu']))))) {
            $this->kontrolaCislaDokladu($rec);
        }

        if ($rec['ePodani']) {
            if (($rec['DruhDavky'] == static::DRUH_DAVKY_OCR) or ($rec['DruhDavky'] == static::DRUH_DAVKY_OPP) or ($rec['DruhDavky'] == static::DRUH_DAVKY_DLO)) {
                if (empty($rec['RC_RodinnyPrislusnik'])) {
                   throw new \Exception(TextManager::_('DavkyNZ_chybi_osetrovana_osoba'));
                }
            }
        } else {
            switch ($rec['DruhDavky']) {
                case static::DRUH_DAVKY_OCR:
                    if (empty($rec['RC_RodinnyPrislusnik'])) {
                        throw new \Exception(TextManager::_('DavkyNZ_chybi_osetrovana_osoba'));
                    }

                    //pokud je duvod pece, uzavrena skola, musi byt vyplnena
                    if ($rec['DuvodPece'] == 3) {
                        if (empty($rec['nazevZarizeniSkoly'])) {
                            throw new \Exception(TextManager::_('DavkyNZ_chybi_nazev_skoly'));
                        }
                        if (empty($rec['ICZarizeniSkoly'])) {
                            throw new \Exception(TextManager::_('DavkyNZ_chybi_IC_skoly'));
                        }
                    }
                    break;
                case static::DRUH_DAVKY_PPM:
                    if (empty($rec['predpokladanyDenPorodu'])) {
                        throw new \Exception(TextManager::_('DavkyNZ_chybi_osetrovana_osoba'));
                    }
                    break;
                /** @noinspection PhpDuplicateSwitchCaseBodyInspection */
                case static::DRUH_DAVKY_OPP:
                    if (empty($rec['RC_RodinnyPrislusnik'])) {
                        throw new \Exception(TextManager::_('DavkyNZ_chybi_osetrovana_osoba'));
                    }
                    break;
                /** @noinspection PhpDuplicateSwitchCaseBodyInspection */
                case static::DRUH_DAVKY_DLO:
                    if (empty($rec['RC_RodinnyPrislusnik'])) {
                        throw new \Exception(TextManager::_('DavkyNZ_chybi_osetrovana_osoba'));
                    }
                    break;
            }
        }
    }

    private function priradNovehoPrislusnika($idOznameni)
    {
        if ($_POST['ditePPM'] ?? null) {
            try {
                $rcRodinnehoPrislusnika = $_POST['ditePPM'];

                // zapise dite k oznameni
                $recPPMDeti = static::getRecPPMDeti($this->database, $this, $idOznameni, $rcRodinnehoPrislusnika);

                $recPPMDeti->insertRecord();
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e, 'ditePPM');
                return false;
            }
        }

        return true;
    }

    private function vlozNoveDatumPece($idOznameni)
    {
        if ($_POST['PecovalOd'] ?? null) {
            try {
                $pecovalOd = DateUtils::dateStringToTimestamp($_POST['PecovalOd']);
                $pecovalDo = DateUtils::dateStringToTimestamp($_POST['PecovalDo']);

                // zapise termin pece k oznameni
                $recPecoval = static::getRecPecoval($this->database, $this, $idOznameni, $pecovalOd);

                if ($recPecoval->loadRecord()) {
                    throw new \Exception(TextManager::_('DavkyNZ_termin_pece_jiz_existuje'));
                } else {
                    $recPecoval['PecovalDo'] = $pecovalDo;
                    $recPecoval->insertRecord();
                }
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e, 'PecovalOd');
                return false;
            }
        }

        return true;
    }

    function beforeInsertRecord($rec)
    {
        /*
         * Podle posledního písmena čísla dokladu, se určuje druh dávky:
            N, Z  -> OSE (DruhDavky=1)
            M      -> PPM (DruhDavky=3)
            T       -> OPP (DruhDavky=5)
            L        -> DLO (DruhDavky=6)
         * */
        switch (strtoupper(substr($rec['CisloDokladu'], -1))) {
            case 'N':
            case 'Z':
                $rec->findFieldByName('DruhDavky')->setNewValue(static::DRUH_DAVKY_OCR);
                break;
            case 'M':
                $rec->findFieldByName('DruhDavky')->setNewValue(static::DRUH_DAVKY_PPM);
                break;
            case 'T':
                $rec->findFieldByName('DruhDavky')->setNewValue(static::DRUH_DAVKY_OPP);
                break;
            case 'L':
                $rec->findFieldByName('DruhDavky')->setNewValue(static::DRUH_DAVKY_DLO);
                break;
        }

        $this->kontrolaZadanychDat($rec);

        return parent::beforeInsertRecord($rec);
    }

    function beforeUpdateRecord($rec)
    {
        if ($rec->isChanged() && $rec['CisloRozhodnutiStatus'] == 3) {
            $rec->findFieldByName('CisloRozhodnutiStatus')->setNewValue(0);
        }

        $this->kontrolaZadanychDat($rec);

        return parent::beforeUpdateRecord($rec);
    }

    function afterUpdateRecord($rec)
    {
        $result = true;

        if ($this->fieldDetiPPM) {
            if (!$this->fieldDetiPPM->zapisZmeny($this->database, $rec['id'], $this)) {
                $result = false;
            }
        }

        if ($this->fieldPecovalOdDo) {
            if (!$this->fieldPecovalOdDo->zapisZmeny($this->database, $rec['id'], $this)) {
                $result = false;
            }
        }

        if (!$this->priradNovehoPrislusnika($rec['id'])) {
            $result = false;
        }

        if (!$this->vlozNoveDatumPece($rec['id'])) {
            $result = false;
        }

        if (!parent::afterUpdateRecord($rec)) {
            return false;
        }

        return $result;
    }


}