<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBRecord;
use Ron\MessageManager;


class NastaveniUzivatele implements \ArrayAccess
{
    const CASOVA_SLOZKA_ROZDELENI_DV = 'rozdelenidv_slozka';  // prednastavena casova slozka pro rozdeleni dennich vysledku

    const ICAL_KEY = 'icalkey';
    const ICAL_SMENY = 'ical_smeny';
    const ICAL_SCHVALENE_ABSENCE = 'ical_schvaleneabsence';
    const ICAL_VSECHNY_ABSENCE = 'ical_vsechnyabsence';
    const ICAL_SLUZEBNI_CESTY = 'ical_sluzebnicesty';

    var $loaded;
    var $changed; // priznak, ze doslo ke zmene a je potreba ulozit

    var $loginName;
    var $rc;

    var $items = []; // asociativni pole dalsich nastaveni (klic => hodnota), ktera se mohou pouzivat napr. v XML

    /** @var  NastaveniUzivatele */
    private static $nastaveniUzivatele;


    /**
     * NastaveniUzivatele constructor.
     */
    public function __construct()
    {
        $this ->loaded = false;
        $this ->changed = false;
    }


    /**
     * @return NastaveniUzivatele
     */
    public static function getInstance()
    {
        if ( !self::$nastaveniUzivatele ) {
            self::$nastaveniUzivatele = new NastaveniUzivatele();
        }

        return self::$nastaveniUzivatele;
    }


    public function offsetExists($offset): bool
    {
        return isset($this ->items[$offset]);
    }

    #[\ReturnTypeWillChange]    // kvuli podpore PHP 7 nelze nastavit typy
    public function offsetGet($offset)
    {
        return $this ->items[$offset];
    }

    public function offsetSet($offset, $value): void
    {
        $this ->items[$offset] = $value;
    }

    public function offsetUnset($offset): void
    {
        unset($this ->items[$offset]);
    }


    public static function saveDefaultInstance($database)
    {
        if ( !self::$nastaveniUzivatele ) {
            return false;
        }

        if ( !self::$nastaveniUzivatele ->changed ) {
            return true;
        }

        self::$nastaveniUzivatele ->save($database);
    }

    /*
     * dalsi pouzivana nastaveni:
     * pk_velikostlegendy = 2 pro uzkou legendu
     */

    function getItem($name, $defaultValue = null)
    {
        return isset($this ->items[$name]) ? $this ->items[$name] : $defaultValue;
    }


    function setItem($name, $value, $defaultValue = '')
    {
        if ($value == $defaultValue) {
            if (isset($this ->items[$name])) {
                unset($this ->items[$name]);
                $this ->changed = true;
            }
        } else if ( !isset($this ->items[$name])  ||  $this ->items[$name] != $value) {
            $this ->items[$name] = $value;
            $this ->changed = true;
        }
    }



    function konvertujNactenyText($hodnota) {
        return strtr($hodnota ?? '', array("\\n" => "\n", "\\\\" => "\\")); // nahrazeni musi probehnout najednou! nelze pouzit str_replace!
    }


    function nactiNastaveni($nastaveni_str)
    {
        $str_settings = explode("\n", $nastaveni_str ?? '');

        $this ->items = [];

        foreach ($str_settings as $setstr) {
            //$pom = sscanf($setstr, '%[^=]=%[^$]'); // , &$key, &$value
            $pom = explode('=', $setstr, 2);
            $key = strtolower($pom[0]);
            $value = (count($pom) > 1) ? $pom[1] : null;

            $this ->items[$key] = $this ->konvertujNactenyText($value);
        }
    }


    /**
     * Nacte nastaveni uzivatele se zadanym loginem nebo rc
     * @param Database $database
     * @param string $login
     * @param string $rc
     * @return string
     * @throws \Ron\DB\DBException
     */
    public static function loadUserSettingsText($database, $rc = null, $login = null)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT ' . $database ->castWideMemo('IntrawebNastaveni') .
            ' FROM Osoba ';

        if ($login) {
            $sql .=
                ' WHERE IntrawebLogin = ' . $params->addParameter($login);
        } else {
            $sql .=
                ' WHERE RC = ' . $params->addParameter($rc ?: User::getRC());
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) return false;

        return $row[0];
    }

    /**
     * @param Database $database
     * @param string $login
     * @param string $rc
     * @return bool
     */
    function load($database, $rc = null, $login = null)
    {
        if (!$login &&  !isset($rc)) {
            $rc = User::getRC();
        }

        $nastaveniStr = self::loadUserSettingsText($database, $rc, $login);
        if ($nastaveniStr === false) return false;

        $this ->loginName = $login;
        $this ->rc = $rc;

        $this ->nactiNastaveni($nastaveniStr);

        return true;
    }


    /**
     * @param $hodnota
     * @return string
     */
    function konvertujProUlozeni($hodnota) {
        //$novahodnota = strtr($hodnota, array("\n" => "\\n", "\\" => "\\\\"));
        //echo "<div> zkonvertoval jsem $hodnota => $novahodnota </div>";
        return strtr($hodnota, array("\n" => "\\n", "\\" => "\\\\")); // nahrazeni musi probehnout najednou! nelze pouzit str_replace!
    }


    function vratNastaveniStr()
    {
        $result = '';

        foreach($this ->items as $key => $value) {
            if ($key)
                $result .=
                    $key . '=' . $this ->konvertujProUlozeni($value) . "\n";
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param bool $log
     * @throws Exception
     */
    public function save($database, $log = false)
    {
        MessageManager::addDebugMessage("Ulozeni uzivatelskeho nastaveni (Osoba.IntrawebNastaveni)");

        $str = $this ->vratNastaveniStr();

        MessageManager::addDebugLog("Ukladam nastaveni uzivatele: $str");

        if ($log) {
            $rec = new RonPortalDBRecord($database, 'Osoba');
            $rec->logDescription = 'Nastaveni uzivatele';
        } else {
            // bez logovani
            $rec = new DBRecord($database, 'Osoba');
        }

        $rec ->addField('RC', '', true, 0, null); // readonly = true
        $rec ->addField('Jmeno', '', true, 0, null);
        $rec ->addField('Prijmeni', '', true, 0, null);
        $rec ->addField('IntrawebLogin', '', true, 0, null);

        $rec ->addField('IntrawebNastaveni', 'memo', false, 0, $str);

        $params = $database ->createParameters();
        if ($this ->rc) {
            $rec->setWhere('RC = ' . $params->addParameter($this->rc), $params);
        } else {
            $rec->setWhere('IntrawebLogin = ' . $params->addParameter($this->loginName), $params);
        }

        if ($rec ->loadRecord()) {
            $rec ->updateRecord();
            $this ->changed = false;
        } else {
            throw new Exception('Chyba! Nepovedlo se nacist uzivatelske nastaveni!');
        }
    }


    public function vygenerujKlic()
    {
        $this ->setItem( self::ICAL_KEY, md5($this ->loginName . rand() . time()) );
    }


    /**
     * @param Database $database
     * @return bool|NastaveniUzivatele
     */
    public static function loadUserSettings($database)
    {
        if ( !self::getInstance() ->loaded ) {
            if ( !self::getInstance() ->load($database) ) {
                return false;
            }

            self::getInstance() ->loaded = true;
        }

        return self::getInstance();
    }


    public static function getUserSettingsItem($database, $itemName, $defaultValue = null)
    {
        $settings = self::loadUserSettings($database);
        if ($settings) {
            return $settings ->getItem($itemName, $defaultValue);
        }

        return $defaultValue;
    }

    /**
     * @param Database $database
     * @param string $itemName
     * @param mixed $value
     * @param mixed $defaultValue
     * @param bool $autoSave
     * @return bool
     * @throws Exception
     */
    public static function setUserSettingsItem($database, $itemName, $value, $defaultValue = null, $autoSave = true)
    {
        if (!User::userAuthenticated()) {
            throw new Exception('Nelze ulozit nastaveni - uzivatel neni prihlasen');
        }

        $settings = self::loadUserSettings($database);
        if (!$settings) {
            return false;
        }

        $settings ->setItem($itemName, $value, $defaultValue);

        if ($autoSave  &&  $settings ->changed) {
            $settings ->save($database);
        }

        return true;
    }

}
