<?php
/**
 * UTF8
 * stanislav.elbl, 23.10.2015
 */

namespace Ron\RonPortal\XML;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Globals;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\Profiler;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Utils\Parser;
use Ron\Utils\SymbolTable;
use RonPortal\NeniPravoException;
use RonPortal\User;
use RonPortal\XML\Classes;
use RonPortal\XML\Classes\XMLElementLogItem;
use RonPortal\XML\XMLFormElement;
use RonPortal\XML\XMLFormElementDropThis;
use RonPortal\XML\XMLFormElementTextNode;
use RonPortal\XML\XMLSessionAccess;


class XmlFormException extends RonException {
}




/**
 * Implementace XML formulare.
 * Class XMLForm
 */
class XMLForm implements SymbolTable
{
    const XML_MODE_SHOW = 'show';
    const XML_MODE_PROCESS = 'process';
    const XML_MODE_DATA = 'data';
    const XML_MODE_TRANSFORM = 'transform';
    const XML_MODE_EVENT = 'event';
    const XML_MODE_INCLUDE = 'include';
    const XML_MODE_HODNOCENI = 'hodnoceni';
    const XML_MODE_JSON = 'json';

    const XML_NELZE_PREDAT = 0;
    const XML_LZE_PREDAT = 1;
    const XML_LZE_POUZE_VRATIT = 2;

    /** @var Database */
    var $database;

    /**
     * @var int
     *      0, 1 nebo nic = promenne ve formatu %...%
     *      2 = promenne ve formatu {$...}, podpora poli, prekladu, sql dotazy s vice vyslednymi poli vytvari promennou typu pole
     */
    var $verze;

    /**
     * @var string  Mod dokumentu
     *  data - datove XML - provadi se show() a process() - doporuceno pro predavaci formulare
     *  transform - vyplnovaci xml, ktere se pri predavani transformuje - provadi se show() a fill() - misto "transform" doporucuji pouzit "data"
     *  process - nevyplnovaci formular, provadi se show() a pri odeslani process() a nasledne redirect
     *  show - nevyplnovaci formular, provadi se show()
     *  event - udalost - nelze pouzit jinym zpusobem nez jako udalost - provadi se event()
     *  include - formular je urcen k pripojeni do jineho formulare pomoci <include> - nelze pouzit jinym zpusobem
     */
    var $mode;

    /**
     * @var string  HTML action formulare
     */
    var $formAction = null;

    /**
     * @var bool        priznak, jestli je povolen json - pokud neni nastaven a mod je jiny nez json, tak nelze formular pouzit pro json volani
     */
    var $enableJson;

    /** @var bool       priznak, jestli se maji pro zobrazeni pouzit postnute hodnoty */
    var $usePostValues;

    /** @var string     context bloku, ktere se maji zobrazit */
    var $context;
    /** @var string     xml akce (tag action/button) */
    var $xmlAction;

    /** @var bool */
    var $isError;
    /** @var bool       Nastavi se na true, pokud je jiz uzavreno, nebo neni pravo na editaci*/
    var $readOnly;
    /** @var boolean    Priznak, ze se ma provadeni formulare ukoncit */
    var $exit;
    /** @var int        Priznak, jestli je mozne formular predat dale nebo vratit zpet (0=nelze, 1=ano, 2=lze pouze vratit) */
    public $lzePredat = self::XML_LZE_PREDAT;


    /** @var array      pole promennych, ktere lze pouzit ve vyrazech   jmeno => hodnota */
    var $variables;

    /** @var string[]   seznam promennych, ktere se ulozi do vysledneho datoveho xml */
    var $dataVariables = [];
    /** @var XMLElementLogItem[] */
    var $logItems = [];

    var $hiddenFields = [];

    /** @var bool    nastaveno na true, dokud nenajde v dokumentu prvni sekci, pak se nastavi na false */
    var $prvniSekce;
    /** @var bool
     *      true, pokud dokument obsahuje vice sekci (tj. jeho vyplnovani jeste neni ukonceno)
     *      Pozor! Na zacatku dokumentu je nastaveno na false. Spravnou hodnotu ma teoreticky az na konci dokumentu.*/
    var $viceSekci;
    /** @var string  jmeno prave vyplnene sekce - na konci se prida do promennych */
    var $sectionName = '';

    /** @var string     vysledna hodnota pro predani do XML v AJAX */
    var $xmlResultCode;
    var $xmlResultMessage;

    /** @var int        priznak, ze se formular nema po vyplneni zavrit (tj. nezobrazi se seznam formularu) */
    var $pokracovat;
    /** @var  Redirect */
    var $redirect;


    /** @var bool       priznak, jestli se ma zobrazit nadpis - defaultne true */
    var $showHeader;
    /** @var string     Jmeno souboru s view - nastavuje se tagem viewfile */
    var $viewFileName;

    /** @var string     XML text vyplneneho formulare */
    var $xml;
    /** @var string     XML text puvodni sablony - pouzije se u datovych XML */
    var $xmlSablony;

    /** @var XMLFormElement */
    var $rootElement;

    /** @var  ViewObject[]  Nektere tagy vytvari viewObject - napr pro zobrazeni formulare naprogramovaneho v PHP */
    var $viewObjects = [];
    var $viewObjectPlaceholder = '$VIEW$';
    var $showErrorMessages = true;

    // TODO: doplnit primo do datoveho XML i log zmen?


    /**
     * XMLForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this ->database = $database;

        $this ->variables = [];
        foreach ($_GET as $getName => $getValue) {
            $this ->variables['GET_' . $getName] = $getValue;
        }

        if (isset($_POST)) {
            foreach ($_POST as $postName => $postValue) {
                $this->variables['POST_' . $postName] = $postValue;
            }
        }

        $this ->xml = '';

        $this ->readOnly = false;

        $this ->isError = false;
        $this ->prvniSekce = true;
        $this ->viceSekci = false;
        $this ->pokracovat = 0;
    }


    /**
     * @param XMLFormElement $parentElement
     * @param string $tag
     * @return XMLFormElement
     */
    public function createElementForTag($parentElement, $tag)
    {
        switch ($tag) {
            case 'ronform' :
            case 'ronxml' :
            case 'ronxmldata' :
            case 'xmlform' :
                return new Classes\XMLElementRonForm($this, $parentElement);

            case 'text' : // netreba pouzivat
                return new XMLFormElementDropThis($this, $parentElement);

            case 'edit' :
                return new Classes\XMLElementEdit($this, $parentElement);

            case 'memo' :
                return new Classes\XMLElementMemo($this, $parentElement);

            case 'checkbox' :
                return new Classes\XMLElementCheckbox($this, $parentElement);

            case 'hidden' :
                return new Classes\XMLElementHidden($this, $parentElement);

            case 'radio' :
                return new Classes\XMLElementRadio($this, $parentElement);

            case 'sqlselect' :
                return new Classes\XMLElementSQLSelect($this, $parentElement);

            case 'select' :
                return new Classes\XMLElementSelect($this, $parentElement);
            case 'option' :
                return new Classes\XMLElementOption($this, $parentElement);

            case 'variablevalue' :
                return new Classes\XMLElementVariableValue($this, $parentElement);

            case 'variable' :
            case 'longvariable' :
            case 'datavariable' :
                return new Classes\XMLElementVariable($this, $parentElement);

            case 'expression' :
                return new Classes\XMLElementExpression($this, $parentElement);

            case 'postvariable' :
                return new Classes\XMLElementPostVariable($this, $parentElement);

            case 'usepostvalues' :
                return new Classes\XMLElementUsePost($this, $parentElement);

            case 'sql' :
            case 'sqlvariable' :
                return new Classes\XMLElementSQL($this, $parentElement);

            case 'sqlaction' :
                return new Classes\XMLElementSQLAction($this, $parentElement);
//                case 'sqlparam' :
//                    $element = new XMLFormElementSQLParam($xmlForm, $parentElement);
//                    break;

            case 'block' :
            case 'section' :
            case 'show' :
            case 'fill' :
            case 'lang' :
            case 'posledniodpoved' :
                return new Classes\XMLElementBlock($this, $parentElement);

            case 'while' :
                return new Classes\XMLElementWhile($this, $parentElement);

            case 'test' :
                return new Classes\XMLElementTest($this, $parentElement);

            case 'testok' :
            case 'else' :
                return new XMLFormElementDropThis($this, $parentElement);

            case 'if' :
                return new Classes\XMLElementIf($this, $parentElement);

            case 'try' :
                return new Classes\XMLFormElementTry($this, $parentElement);
            case 'catch' :
                return new Classes\XMLFormElementCatch($this, $parentElement);

            case 'ulozit' :
            case 'predatdale' :
            case 'vratitzpet' :
                return new Classes\XMLElementSubmit($this, $parentElement);

            case 'action' :
            case 'button' :
                return new Classes\XMLElementAction($this, $parentElement);

            case 'pokracovat' :
            case 'redirect' :
                return new Classes\XMLElementRedirect($this, $parentElement);

//            case 'json' :
//                return new Classes\XMLElementJson($this, $parentElement);

            case 'exit' :
                return new Classes\XMLElementExit($this, $parentElement);

//            case 'mapovanimenu' :
//                require_once('xml/xml_commands.php');
//                return new XMLFormElementSimpleCommand($this, $parentElement);

            case 'infomessage' :
            case 'infomsg' :
            case 'debugmessage' :
            case 'debugmsg' :
            case 'debuglog' :
            case 'errormessage' :
            case 'errormsg' :
            case 'error' :
            case 'warning' :
            case 'exception' :
            case 'successmsg' :
            case 'successmessage' :
                return new Classes\XMLElementMessage($this, $parentElement);

            case 'include' :
                return new Classes\XMLElementInclude($this, $parentElement);

            case 'inputtable' :
            case 'inputtablerow' :
                return new Classes\XMLElementInputTable($this, $parentElement);

            case 'record' :
                return new Classes\XMLElementRecord($this, $parentElement);

            case 'recordfield' :
            case 'recordwhere' :
            case 'loadrecord' :
            case 'insertrecord' :
            case 'editrecord' :
            case 'updaterecord' :
            case 'updateorinsertrecord' :
            case 'deleterecord' :
                return new Classes\XMLElementRecordPart($this, $parentElement);

            case 'browser' :
                return new Classes\XMLElementBrowser($this, $parentElement);

            case 'browserfield' :
            case 'browsersql' :
            case 'browserdetail' :
            case 'browserlink' :
                return new Classes\XMLElementBrowserPart($this, $parentElement);

            case 'for' :
                return new Classes\XMLElementFor($this, $parentElement);

            case 'foreach' :
                return new Classes\XMLElementForEach($this, $parentElement);

            case 'viewobject' :
            case 'errormessages' :
                return new Classes\XMLElementViewObject($this, $parentElement);

            case 'XMLRESULT' :
            case 'xmlresult' :
                return new XMLFormElementDropThis($this, $parentElement);

            default :
                return new XMLFormElement($this, $parentElement);
        }
    }

    /**
     * @param XMLFormElement $parentElement
     * @param \XMLReader $xmlReader
     * @return XMLFormElement
     */
    public function createXMLFormElement($parentElement, $xmlReader)
    {
        //if (!$dom_node) return null;
        switch ($xmlReader->nodeType) {
            case \XMLReader::COMMENT :
            case \XMLReader::WHITESPACE :
                $xmlReader->read(); // preskoci comment
                return null;

            case \XMLReader::TEXT :
            case \XMLReader::CDATA :
            case \XMLReader::SIGNIFICANT_WHITESPACE :
                $element = new XMLFormElementTextNode($this, $parentElement);
                $element->loadXML($xmlReader);
                return $element;

            case \XMLReader::ELEMENT :
                break;

            default :
                MessageManager::addErrorMessage('Neznamy typ dat v XML souboru! (' . $xmlReader->nodeType . ')');
                $xmlReader->read(); // preskoci
                return null;
        }

        if ($xmlReader->prefix == 'html') {
            // prefix html: - vzdy zapisuje primo do html
            $element = new XMLFormElement($this, $parentElement);
        } else {
            $element = $this ->createElementForTag($parentElement, $xmlReader ->localName);
        }

        $element->loadXML($xmlReader);

        return $element;
    }


    /**
     * @param string $xmlText
     * @param XMLFormElement $parentElement
     * @return XMLFormElement
     * @throws \Exception
     */
    public function loadXMLElements($xmlText, $parentElement = null)
    {
        $xmlReader = new \XMLReader();
        $xmlReader->XML(trim($xmlText), TextManager::getInstance() ->charSet); // , 'utf-8'  doplnit parametr se znakovou sadou?
        $rootElement = null;

        while (!$rootElement && $xmlReader->read()) {
            if ($xmlReader->nodeType == \XMLReader::ELEMENT) {
                if ($xmlReader->localName == "ronxmldata") {
                    // datove xml
                    $this->mode = 'data';
                }
                $rootElement = $this ->createXMLFormElement($parentElement, $xmlReader);
                break;
            }
        }

        if (!$rootElement) {
            throw new \Exception('Nepovedlo se nacist obsah XML formulare. Formular je prazdny nebo neni validni!');
        }

        return $rootElement;
    }


    /**
     * Nacte elementy ze zadaneho xml textu pripadne z $this ->xml
     * @param null $xmlText
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function loadXML($xmlText = null)
    {
        $this ->rootElement = $this ->loadXMLElements($xmlText ?: $this ->xml);

        switch ($this ->rootElement ->elementName) {
            case 'ronxml' :
            case 'ronform' :
            case 'ronxmldata' :
                break;

            default :
                // zabrani zobrazeni konfiguracnich XML formularu
                throw new NeniPravoException();
        }
    }



    function dataMode()
    {
        return $this ->mode == self::XML_MODE_DATA  ||  $this ->mode == self::XML_MODE_HODNOCENI;
    }


    /**
     * Vraci true, pokud neni mozne tento formular samostatne zobrazit a zpracovat
     * @return bool
     */
    function forbiddenMode()
    {
        return $this ->mode == self::XML_MODE_EVENT  ||  $this ->mode == self::XML_MODE_INCLUDE  ||  $this ->mode == self::XML_MODE_JSON;
    }

    /**
     * Prida chybovou hlasku $msg a pokud je zadane jmeno pole s chybou, tak zaznamena take toto pole
     * @param \Exception|string|bool $msg
     * @param string $errorField    Jmeno pole (napriklad formularoveho), ke kteremu se chyba vztahuje
     * @param string $errorId       Id chyby
     */
    function addErrorMsg($msg, $errorField = '', $errorId = null)
    {
        $this ->isError = true;

        if ($msg !== ''  &&  $msg !== true) {
            MessageManager::addErrorMessage($msg, $errorField, $errorId);
        }
    }

//    function addInfoMessage($msg)
//    {
//        MessageManager::addInfoMessage($msg);
//    }

    function addDebugMessage($message, $long = false)
    {
        if ($long) {
            MessageManager::addDebugLog($message);
        } else {
            MessageManager::addDebugMessage($message);
        }
    }


    function isError($fieldName = null, $throwException = false)
    {
        if (!$this ->isError) {
            return false;
        }

        if ($fieldName ||  $throwException) {
            $error = MessageManager::getInstance() ->findError($fieldName);

            if ($throwException  &&  $error) {
                throw new XmlFormException($error->text, $error->fieldName, $error->description);
            }

            if ($error) return true;
            elseif ($fieldName) return false;
        }

        return $this ->isError;
    }


    /**
     * Vraci true, pokud zadany vyraz ($contextStr) odpovida aktualnimu kontextu formulare
     * Lze zadat i vice kontextu oddelenych strednikem
     * Take lze pouzit ! na zacatku - pak vraci true v pripade, ze aktualni kontext formulare neni zadny z vyjmenovanych
     * @param string $contextStr
     * @return bool
     */
    function isContext($contextStr)
    {
        if ($contextStr == '') return true;

        if ($contextStr == '#nocontext') {
            return !$this ->context;
        }

        // moznost negace - tj. (
        $negate = $contextStr[0] == '!';
        if ($negate) {
            $contextStr = substr($contextStr, 1);
        }

        // kontrola na konkretni kontext
        if ($contextStr == $this ->context) {
            return !$negate;
        }

        // jeste zkusi, jestli neni zadano vice contextu oddelenych strednikem
        $contexts = explode(';', $contextStr);
        foreach ($contexts as $context) {
            if ($context == $this ->context) {
                return !$negate;
            }
        }

        return $negate;
    }


    function nastavXmlAction()
    {
        if (isset($_POST['xmlaction'])) $this ->xmlAction = $_POST['xmlaction'];
        else if (isset($_GET['xmlaction'])) $this ->xmlAction = $_GET['xmlaction'];
        else $this ->xmlAction = '';
    }



    private function parametr($nazev, $formatovani)
    {
        // parametr je tvaru NAZEVPARAMETRU:TYP - TYP je nepovinny, vychozi je string
        $poma = explode(':', $nazev, 2);

        $value = $this ->getVariableValue($poma[0]);
        return $this ->formatValue($value, isset($poma[1]) ? $poma[1] : '', $formatovani);
    }

    /**
     * Parametry v SQL dotazu nebo textu nahradi promennymi ve tvaru %JMENOPARAMETRU:TYP%
     * Typ je nepovinny (defaultni hodnota je string)
     * Pouziva se pro starsi verze formularu
     * @param $text
     * @param string $formatovani Muze byt sql, html nebo prazdne
     * @return string
     */
    private function nahradPromenne_OLD($text, $formatovani = '')
    {
        $result = '';

        // rozdeli retezec na casti - liche casti jsou primo sql, sude jsou parametry
        $sqlParts = explode('%', $text);

        $jeParametr = false; // priznak jestli je aktualni retezec parametr
        foreach ($sqlParts as $sqlPart) {
            if ($jeParametr) {
                if (!$sqlPart) {
                    // 2 procenta za sebou (%%) => vlozi %
                    $result .= '%';
                } else {
                    $result .= $this ->parametr($sqlPart, $formatovani);
                }
                $jeParametr = false; // dalsi cast nebude parametr
            } else {
                $result .= $sqlPart;
                $jeParametr = true; // dalsi cast bude parametr
            }
        }

        return $result;
    }


    /**
     * Parametry v SQL dotazu nebo textu nahradi promennymi. Pouzije tridu Parser.
     * @param $text
     * @param string $formatovani
     * @return string
     */
    function nahradPromenne_NEW($text, $formatovani = '')
    {
        $parser = new Parser($this);
        return $parser ->parsuj($text, $formatovani);
    }


    /* @param string $text Text, ve kterem se nahradi promenne
     * @param string $formatovani Muze byt sql, html nebo prazdne - puvodne tu bylo bool, ktere urcovalo, jestli se ma pouzit 'sql'
     * @return string
     */
    function nahradPromenne($text, $formatovani = '')
    {
        if ( !isset($text) ) return '';
        if ( $text == '' ) return '';

        if ($this ->verze > 1) {
            return $this ->nahradPromenne_NEW($text, $formatovani);
        } else {
            return $this ->nahradPromenne_OLD($text, $formatovani);
        }
    }


    /**
     * @param mixed $value
     * @param string $format
     * @param string $formatovani  muze byt sql, html, cmd (pro command line), nebo prazdne. Pro format date a sqldate muze byt i primo format pro funkci date()
     * @return float|int|mixed|null|string
     */
    function formatValue($value, $format, $formatovani)
    {
        $format = strtolower($format);

        switch ($format) {
            case 'sqltimestamp' :
            case 'sql2datetimestr' :
            case 'sqldate' :
            case 'sql2datestr' :
            case 'sqltime' :
            case 'sql2timestr' :
                $value = (isset($value) && $value !== '') ? $this ->database ->stringToTimestamp($value) : null;
                break;
        }

        // podle typu hodnotu naformatuje
        switch ($format) {
            case 'int' :
            case 'integer' :
                return (int)$value;


            case 'date' :
            case 'sqldate' :
                if ($formatovani == 'sql') {
                    return $this ->database ->dateToString($value);
                } else if ($formatovani == 'html'  ||  $formatovani === false  ||  $formatovani == ''  ||  $formatovani == null) {
                    return isset($value) ? date('j.n.Y', $value) : '';
                } else if ($formatovani == 'cmd') {
                    return date('Y-m-d', $value);
                } else {
                    return isset($value) ? date($formatovani, $value) : '';
                }

            case 'sql2datestr' :
                return isset($value) ? date('j.n.Y', $value) : '';

            case 'time' :
            case 'sqltime' :
                if ($formatovani == 'sql') {
                    return $this ->database ->timeToString($value);
                } else {
                    return isset($value) ? date('G:i', $value) : '';
                }

            case 'sql2timestr' :
                return isset($value) ? date('G:i', $value) : '';

            case 'fulltime' :
                if ($formatovani == 'sql') {
                    return $this ->database ->timeToString($value);
                } else {
                    return isset($value) ? date('G:i:s', $value) : '';
                }

            case 'fulldatetime' :
                if ($formatovani == 'sql') {
                    return $this ->database ->timestampToString($value);
                } else if ($formatovani == 'cmd') {
                    return date('Y-m-d G:i:s', $value);
                } else {
                    return isset($value) ? date('j.n.Y G:i:s', $value) : '';
                }

            case 'datetime' :
            case 'timestamp' :
            case 'sqltimestamp' :
                if ($formatovani == 'sql') {
                    return $this ->database ->timestampToString($value);
                } else if ($formatovani == 'cmd') {
                    return date('Y-m-d G:i', $value);
                } else {
                    return isset($value) ? date('j.n.Y G:i', $value) : '';
                }

            case 'sql2datetimestr' :
                return isset($value) ? date('j.n.Y G:i', $value) : '';

            case 'datestr' :
                // textove zadany datum (uzivatelem)
                if ($formatovani != 'sql') {
                    return $value;
                }

                try {
                    $value = DateUtils::dateStringToTimestamp($value);
                }
                //22.7.2016 RON-PR potlaceni vyjimky pro zachovani puvodni funkcnosti (puvodni funkce datestringtotimestamp mela stejne chovani)
                catch (\Exception $e) {
                    $value = 0;
                }

                return $this ->database ->timestampToString($value);

            case 'str2date' :
                return (isset($value) && trim($value) !== '') ? DateUtils::dateStringToTimestamp($value) : null;

            case 'sekundy' :
            case 'minuty' :
            case 'hodiny' :
            case 'timeofday' :
            case 'secondsofday' :
                return StringUtils::format($format, $value);

            case 'float' :
            case 'double' :
                return is_string($value) ? StringUtils::stringToFloat($value) : (is_null($value) ? 0 : $value);

            case 'sql' :
                return $this->nahradPromenne($value, $formatovani);
                // vraci primo hodnotu zadane promenne - ta ale muze taky obsahovat prommenne, takze je nahradi

            case 'sql2timestamp' :
                return (isset($value) && $value !== '') ? $this ->database ->stringToTimestamp($value) : null;

            case 'text' :
            case 'html' :
                return $value;

            case 'string' :
            default :
                switch ($formatovani) {
                    case 'sql' :
                        return " '" . $this ->database ->addSlashes($value) . "' ";
                    //case 'html' : return StringUtils::htmlTextConvert($value);
                    //case 'html' : return StringUtils::htmlTextConvert(StringUtils::format($format, StringUtils::stringToFloat($value)));
                    case 'html' :
                    case 'cmd' :
                        return StringUtils::format($format, $value);
                }
                return $value;
        }
    }


    function getVariableValue($varName)
    {
        switch ($varName) {
            case 'NOW' :
                return time();
            case 'NOWSTR' :
                return date('j.n.Y G:i', time());
            case 'TODAY' :
                return DateUtils::startOfTheDay(time());
            case 'TODAYSTR' :
                return date('j.n.Y', time());
            case 'MONTH' :
                return DateUtils::firstDayOfMonth(time());
            case 'MONTHSTR' :
                return date('j.n.Y', DateUtils::firstDayOfMonth(time()));

            case 'XML_CONTEXT' :
                return $this ->context;

            case 'XML_SHOWHEADER' :
                return $this ->showHeader ? '1' : '0';

            case 'XML_ERROR' :
                return $this ->isError() ? '1' : '0';

            case 'XML_DEBUG' :
                return App::getDebugMode() ? '1' : '0';

            case 'XML_ACTION' :
                return $this ->xmlAction;

            case 'XML_MODE' :
                return $this ->mode;

            case 'XML_RESULTCODE' :
                return $this ->xmlResultCode;

            case 'XML_READONLY' :
                return $this ->readOnly ? 1 : 0;

            case 'SESSION' :
                return new XMLSessionAccess();

            case 'POST' :   // asi nefunguje!
                return $_POST;

            case 'GLOBALS' :
                return Globals::getInstance();
        }

        return isset($this ->variables[$varName]) ? $this ->variables[$varName] : null;
    }


    function getVariableReference($varName)
    {
        return ' $xmlform ->getVariableValue("' . $varName . '", false) ';
    }


    /**
     * Nastavi hodnotu promenne
     * @param $jmenoPromenne
     * @param $value
     * @throws \Exception
     */
    function setVariable($jmenoPromenne, $value)
    {
        switch ($jmenoPromenne) {
            case 'XML_ACTION' :
                $this ->xmlAction = $value;
                break;

            case 'XML_SHOWHEADER' :
                $this ->showHeader = (int)$value;
                break;

            case 'SESSION' :
            case 'GLOBALS' :
            case 'XML_MODE' :
                throw new \Exception('Nelze zmenit hodnotu promenne ' . $jmenoPromenne . '!');

            default :
                $this ->variables[$jmenoPromenne] = $value;
                break;
        }
    }


    /**
     * @param $variables array   asociativni pole promennych nazev => hodnota (pouziva se v XML events)
     */
    function setVariables($variables)
    {
        if (!$variables) return;

        foreach ($variables as $name => $value) {
            $this ->setVariable($name, $value);
        }
    }


    function addDataVariable($name)
    {
        $this ->dataVariables[$name] = $name;
    }


    /**
     * @param XMLElementLogItem $item
     */
    function addLogItem($item)
    {
        $this ->logItems[] = $item;
    }

    /**
     * @param string $name
     * @param string $value
     * @return XMLElementLogItem
     */
    function addLogData($name, $value)
    {
        $item = new XMLElementLogItem($this, null);
        $item ->name = $name;
        $item ->value = $value;
        $item ->userName = User::getLoginName();
        $item ->time = date('j.n.Y G:i:s');
        $this ->addLogItem($item);

        return $item;
    }



    public function getHtmlOutput()
    {
        return $this ->rootElement
            ? $this ->rootElement ->getHTMLOutput()
            : '';
    }


    function process()
    {
        if ($this ->mode != self::XML_MODE_PROCESS) {
            throw new \Exception("Chyba - v metode XMLForm::process - spatny mod formulare ({$this ->mode})!");
        }

        $profilerItem = Profiler::addItem('XMLForm::process()', 'XML');
        $this ->rootElement->process();
        Profiler::endItem($profilerItem);
    }


    /**
     * Zapise datove XML
     * @param \XMLWriter $xmlWriter
     */
    function writeData($xmlWriter)
    {
        if ($this ->sectionName) {
            // prida do promennych aktualne vyplnenou sekci
            // (dela to az na konci, aby se behem fill neprovadel kod, ktery se ma provest az po vyplneni teto sekce)
            if (!$this->isError()) {
                $this->setVariable($this->sectionName, 1);
                $this->addDataVariable($this->sectionName);
            }
        }

        foreach ($this ->dataVariables as $varName) {
            $value = StringUtils::toUTF8($this ->getVariableValue($varName));

            $xmlWriter ->startElement('variable');
            $xmlWriter ->writeAttribute('name', StringUtils::toUTF8($varName));
            $xmlWriter ->writeAttribute('value', $value);
            $xmlWriter ->endElement();
        }

        foreach ($this ->logItems as $logItem) {
            $logItem ->writeXMLOutput($xmlWriter);
        }
    }



    function event($returnContent = false)
    {
        $profilerItem = Profiler::addItem('XMLForm::event()', 'XML');
        $this ->addDebugMessage('XMLForm::event');

        $this ->prvniSekce = true;
        $this ->viceSekci = false;

        if ($returnContent) {
            return $this ->rootElement -> getHTMLOutput();
        }

        $this ->rootElement ->event();

        Profiler::endItem($profilerItem);

        return !MessageManager::getInstance() ->isError();
    }
}


/**
 * Class XMLFormViewObject
 * @package Ron\RonPortal\XML
 * @deprecated
 */
class XMLFormViewObject extends \RonPortal\XML\XMLFormViewObject
{
}
